#!/usr/bin/env python3
import os
import sys
import time
import shutil
import logging
import platform
import datetime
import argparse
import subprocess

#
#
#
class Command:

    #
    # accepts arguments as [] and returns (exit_code, stdout, stderr) or throws an exception
    #
    def run(self, args):
    
        # run this command
        process = subprocess.Popen(args, stdout=subprocess.PIPE, stderr=subprocess.PIPE)

        # wait until it is finished
        (stdout, stderr) = process.communicate()

        # and return
        return (process.returncode, stdout, stderr)


#
# environment information (a copy of that in /opt/websafety/bin/certmgr.py - for now no sharing of code exists)
#
class Info:

    @staticmethod
    def get_exe():

        if os.path.isfile("/etc/centos-release") or os.path.isfile("/etc/redhat-release"):
            return "/usr/sbin/squid"

        if platform.system().lower() == "windows":
            return "c:\\Squid\\bin\\squid.exe"

        if platform.system().lower() == "freebsd":
            return "/usr/local/sbin/squid"

        return "/usr/sbin/squid"

    @staticmethod
    def get_cache():

        if os.path.isfile("/etc/centos-release") or os.path.isfile("/etc/redhat-release"):
            return "/var/spool/squid"

        if platform.system().lower() == "windows":
            return "c:\\Squid\\var\\cache\\squid"

        if platform.system().lower() == "freebsd":
            return "/var/squid/cache"

        return "/var/spool/squid"

    @staticmethod
    def get_user():

        if platform.system().lower() == "windows":
            return "system"

        if platform.system().lower() == "freebsd":
            return "squid"

        assert (platform.system().lower() == "linux")

        if os.path.isfile("/etc/centos-release") or os.path.isfile("/etc/redhat-release"):
            return "squid"

        return "proxy"

    @staticmethod
    def get_group():
        return Info.get_user()


#
#
#
class SquidExe:

    def __init__(self):

        self.exe   = Info.get_exe()
        self.cache = Info.get_cache()
        self.user  = Info.get_user()
        self.group = Info.get_group()

        logging.debug("Detected Squid path : %s" % self.exe)
        logging.debug("Detected Squid cache: %s" % self.cache)
        logging.debug("Detected Squid user : %s" % self.user)
        logging.debug("Detected Squid group: %s" % self.group)

    def stop(self):

        logging.debug("Squid needs to be stopped.")

        args = ["systemctl", "stop", "squid"]
        if platform.system().lower() == "freebsd":
            args = ["service", "squid", "stop"]

        code = subprocess.call(args)
        if code != 0:
            # lets hope for the best
            pass

        logging.debug("Squid stop result arguments: %s, exit code: %d" % (str(args), code))
            

    def start(self):

        logging.debug("Squid needs to be started.")

        args = ["systemctl", "start", "squid"]
        if platform.system().lower() == "freebsd":
            args = ["service", "squid", "start"]

        code = subprocess.call(args)
        if code != 0:
            # lets hope for the bexst
            pass     

        logging.debug("Squid start result arguments: %s, exit code: %d" % (str(args), code))

    def chown_recursive(self):

        args = ["chown", "-R", "%s:%s" % (self.user, self.group), self.cache]
        code = subprocess.call(args)
        if code != 0:

            # construct message
            msg = "Cannot change owner of the cache folder '%s' with command '%s'." % (self.cache, " ".join(args))

            # dump it
            logging.debug(msg)

            # and throw
            raise Exception(msg)

        else:

            # just dump
            logging.debug("Cache owner for folder '%s' is updated with command '%s'." % (self.cache, " ".join(args)))


    def reset_cache(self):

        # remove the cache dir
        if os.path.exists(self.cache):
            shutil.rmtree(self.cache)

        # create it
        os.makedirs(self.cache)

        logging.debug("Erased and created empty Squid cache folder %s" % self.cache)

        # as squid is doing cache initialization as squid user, we must now chown
        self.chown_recursive()

        # call squid -z to initialize the cache dir
        args    = [self.exe, "-z"]
        command = Command()
        
        (code, out, err) = command.run(args)
        if code != 0:

            # format message
            msg = "Cannot initialize Squid's cache dir with command '%s', stdout '%s', stderr '%s'" % (" ".join(args), out, err)

            # and dump it
            logging.error(msg)

            # something is definitely failed, lets throw then
            raise Exception(msg)

        # otherwise dump success to log
        logging.debug("Squid's cache dir was initialized with command '%s', stdout '%s', stderr '%s'" % (" ".join(args), out, err))        

        # the cache needs to be owned by squid, so run chown (on non windows only!)
        if platform.system().lower() != "windows":

            # and chown again just to be sure
            self.chown_recursive()

        # ok good then
        logging.debug("Cache folder is created successfully")

#
#
#
def reset_cache():


    exe = SquidExe()
    
    exe.stop()
    exe.reset_cache()
    exe.start()

#
# action dispatcher
#
def perform_action(args):

    logging.debug("Action '%s' is requested. Running..." % args.action)

    if args.action == "reset-cache":
        return reset_cache()

    raise Exception("Uknown action '%s' provided." % args.action)

#
# main
#
def main():

    verbose = False
    start   = time.time()
    
    # create parser
    parser = argparse.ArgumentParser(description='CacheMgr.py Squid cache manipulation script.')

    # add args
    parser.add_argument(
        "--action", 
        required=True, 
        help="action to perform (one of reset-cache)"
    )
    parser.add_argument("--verbose", help="print additional debug information", action="store_true")

    # parse actual arguments
    args = parser.parse_args()

    # setup logging
    level = logging.INFO
    if args.verbose:
        level = logging.DEBUG
        
    logging.basicConfig(format='%(asctime)s %(message)s', level=level)
            
    logging.debug("Cache manager script is started at %s" % datetime.datetime.now().strftime("%Y-%m-%d, %H:%M:%S"))    
    perform_action(args)
    logging.debug("Cache manager manipulation script is finished SUCCESSFULLY in %d seconds!" % (time.time() - start))

if __name__ == "__main__":

    # call the entry code
    main()

    # if we got here everything is fine
    sys.exit(0)
