#!/usr/bin/env python3
import os
import sys
import time
import json
import shutil
import platform
import datetime
import argparse
import subprocess
import OpenSSL



#
#
#
class Command:

    #
    # accepts arguments as [] and returns (exit_code, stdout, stderr) or throws an exception
    #
    def run(self, args):
    
        # run this command
        process = subprocess.Popen(args, stdout=subprocess.PIPE, stderr=subprocess.PIPE)

        # wait until it is finished
        (stdout, stderr) = process.communicate()

        # in python 3 the stdout and stderr are bytes, so we need to recode
        stdout = stdout.decode("utf-8")
        stderr = stderr.decode("utf-8")

        # and return
        return (process.returncode, stdout, stderr)


#
# environment information
#
class Info:

    @staticmethod
    def get_exe():

        if os.path.isfile("/etc/centos-release") or os.path.isfile("/etc/redhat-release"):
            if os.path.isfile("/usr/lib64/squid/security_file_certgen"):
                return "/usr/lib64/squid/security_file_certgen"
            return "/usr/lib64/squid/ssl_crtd"

        if platform.system().lower() == "windows":
            return "c:\\Squid\\lib\\squid\\ssl_crtd.exe"

        if platform.system().lower() == "freebsd":
            return "/usr/local/libexec/squid/security_file_certgen"

        # for squid 4 running on ubuntu, debian
        if os.path.isfile("/usr/lib/squid/security_file_certgen"):
            return "/usr/lib/squid/security_file_certgen"

        # for squid 2 running on ubuntu, debian
        return "/usr/lib/squid/ssl_crtd"

    @staticmethod
    def get_dir():

        if os.path.isfile("/etc/centos-release") or os.path.isfile("/etc/redhat-release"):
            return "/var/spool/squid_ssldb"

        if platform.system().lower() == "windows":
            return "c:\\Squid\\var\\cache\\squid_ssldb"

        if platform.system().lower() == "freebsd":
            return "/var/squid/ssldb"

        return "/var/spool/squid_ssldb"

    @staticmethod
    def get_user():

        if platform.system().lower() == "windows":
            return "system"

        if platform.system().lower() == "freebsd":
            return "squid"

        assert (platform.system().lower() == "linux")

        if os.path.isfile("/etc/centos-release") or os.path.isfile("/etc/redhat-release"):
            return "squid"

        return "proxy"

    @staticmethod
    def get_group():
        return Info.get_user()


#
#
#
class SslCrtdExe:

    def __init__(self):

        # set the params 
        self.exe   = Info.get_exe()
        self.dir   = Info.get_dir()
        self.user  = Info.get_user()
        self.group = Info.get_group()

        # and validate them
        if not os.path.isfile(self.exe):
            raise Exception("Path to sslcrtd binary does not exist or is not accessible (%s)" % self.exe)

    def recreate(self):

        # we will do everything in a new dir and then move it in place of the current dir
        dir_cur = self.dir
        dir_new = dir_cur + ".tmp"

        if os.path.exists(dir_new):
            shutil.rmtree(dir_new)

        # initialize the new dir
        args = [self.exe, "-c", "-s", dir_new]
        if self.exe.find("security_file_certgen") != -1:
            
            # this is squid 4, we need one more parameter - bad that it is 4MB hardcoded :(
            args.append("-M")
            args.append("4MB")

        # and run
        command = Command()
        
        (code, out, err) = command.run(args)
        if code != 0:

            # something is definitely failed, lets throw then
            raise Exception(
                "Cannot initialize Squid's mimicked certificate storage with command '%s', stdout '%s', stderr '%s'" 
                % (" ".join(args), out, err)
            )

        # both success and failures are indicated as EXIT_SUCCESS by sslcrtd, so here we try to analyze output
        combined = out.lower() + " " + err.lower()

        if "initialization ssl db..." not in combined or "done" not in combined:

            raise Exception(
                "Cannot find magic strings in output of command '%s', stdout '%s', stderr '%s'." 
                % (" ".join(args), out, err)
            )

        # well magic strings are found, now see if the folder now exists
        if not os.path.isdir(dir_new):

            raise Exception("Certificate storage folder '%s' does not exist or is not accessible after successful run of command '%s', stdout '%s', stderr '%s'." % 
                (dir_new, " ".join(args), out, err)
            )

        # the certificate storage needs to be owned by squid, so run chown (on non windows only!)
        if platform.system().lower() != "windows":

            args = ["chown", "-R", "%s:%s" % (self.user, self.group), dir_new]
            code = subprocess.call(args)
            if code != 0:

                raise Exception("Cannot change owner of a temporary folder '%s' with command '%s'." %
                    (dir_new, " ".join(args))
                )

        # good, finally remove the old folder and move the new one in place
        if os.path.exists(dir_cur):
            shutil.rmtree(dir_cur)

        shutil.move(dir_new, dir_cur)

        # ok good then

#
#
#
def regenerate_certificate_storage():

    exe = SslCrtdExe()
    exe.recreate()

#
#
#
class OpenSslExe:

    def __init__(self):
        self.exe = OpenSslExe.get_exe()

    @staticmethod
    def get_exe():

        if platform.system().lower() == "windows":
            return r"c:\OpenSSL-Win32\bin\openssl.exe"

        return "/usr/bin/openssl"

    def convert_pem2der(self, path_pem, path_der):

        args    = [self.exe, "x509", "-in", path_pem, "-outform", "DER", "-out", path_der]
        command = Command()

        (code, out, err) = command.run(args)
        if code != 0:

            # something is definitely failed, lets throw then
            raise Exception(
                "Cannot convert PEM file into DER with command '%s', stdout '%s', stderr '%s'"
                % (" ".join(args), out, err)
            )

        # if we got here the DER certificate is nicely saved in output
        assert (os.path.isfile(path_der))

    def generate_pem(self, country, province, city, organization, organizational_unit, e_mail, common_name, lifetime, output_file):

        # construct subj
        subj = "/C=%s/ST=%s/L=%s/O=%s/OU=%s/CN=%s/emailAddress=%s" % (
            country, province, city, organization, organizational_unit, common_name, e_mail
        )

        # construct arguments
        args = [
            self.exe, "req", "-new", "-newkey", "rsa:2048", "-sha256", "-days", str(lifetime), "-nodes", "-x509",
            "-keyout", output_file,
            "-out", output_file,
            "-subj", subj
        ]
        command = Command()

        # run it
        (code, out, err) = command.run(args)
        if code != 0:

            # something is definitely failed, lets throw then
            raise Exception(
                "Cannot create self-signed RootCA certificate with command '%s', stdout '%s', stderr '%s'"
                % (" ".join(args), out, err)
            )

        # ok then just check the output is there
        assert (os.path.isfile(output_file))

#
#
#
def convert_root_certificate(input_file, output_file):

    # check input file indeed exists
    if input_file is None or not os.path.isfile(input_file):
        raise Exception("Cannot convert RootCA certificate '%s', file does not exist or not accessible!" % input_file)

    # check output file is set
    if output_file is None:
        raise Exception("Cannot convert RootCA certificate '%s', please provide path to output file!" % input_file)

    # if output exists, remove it
    if os.path.exists(output_file):
        os.unlink(output_file)

    # and do the conversion
    exe = OpenSslExe()
    exe.convert_pem2der(input_file, output_file)


#
#
#
class RootCaCert:

    def __init__(self, pem):

        # save members
        self.pem     = pem
        self.x509    = OpenSSL.crypto.load_certificate(OpenSSL.crypto.FILETYPE_PEM, pem)
        self.subject = self.x509.get_subject()

        # parse out extensions
        self.extensions = self.collect_extensions(self.x509)

    def collect_extensions(self, x509):

        extensions = {}

        index = 0
        count = x509.get_extension_count()

        while index < count:

            extension = x509.get_extension(index)
            index += 1

            # ignore all errors and hope for the best
            try:
                name = extension.get_short_name() # name is now a byte array
                name = name.decode("utf-8")       # name is now a string
                if name:
                    extensions[name] = str(extension)
            except Exception as e:
                pass

        return extensions

    def get_country(self):
        return self.subject.countryName

    def get_province(self):
        return self.subject.stateOrProvinceName

    def get_locality(self):
        return self.subject.localityName

    def get_organization(self):
        return self.subject.organizationName

    def get_organizationalunit(self):
        return self.subject.organizationalUnitName

    def get_cn(self):
        return self.subject.commonName

    def get_email(self):
        return self.subject.emailAddress

    def get_version(self):
        return self.x509.get_version()

    def get_serial_number(self):
        return self.x509.get_serial_number()

    def get_signature_algorithm(self):
        return self.x509.get_signature_algorithm().decode("utf-8")

    def get_not_before(self):
        return self.x509.get_notBefore().decode("utf-8")

    def get_not_after(self):
        return self.x509.get_notAfter().decode("utf-8")

    def get_subject_keyid(self):

        name = "subjectKeyIdentifier"

        if name not in self.extensions:
            return ""

        value = self.extensions[name]

        return str(value)

    def get_isca(self):

        if "basicConstraints" not in self.extensions:
            return False

        name = "basicConstraints"
        value = self.extensions[name]

        if value.find("CA:TRUE") != -1:
            return True

        return False


#
#
#
class RootCaKey:

    def __init__(self, pem):

        # note here we do NOT specify passphrase and thus if passphrase was specified while generating key
        # which is not supported by squid, exception will be thrown and this is exactly what we need
        self.pem = pem
        self.key = OpenSSL.crypto.load_privatekey(OpenSSL.crypto.FILETYPE_PEM, pem)

    def check(self):
        self.key.check()





#
# loads root ca certificate from file and dumps it out as JSON
#
def verify_root_certificate(input_file):

    # check the input file indeed exists
    if input_file is None or not os.path.isfile(input_file):
        raise Exception("Cannot verify RootCA certificate '%s', file does not exist or not accessible!" % input_file)

    # read file into memory
    with open(input_file, "rb") as fin:
        pem = fin.read()

    # load the certificate
    cert = RootCaCert(pem)

    # fill the dict
    info = {
        "Country" : cert.get_country(),
        "Province" : cert.get_province(),
        "Locality" : cert.get_locality(),
        "Organization" : cert.get_organization(),
        "OrganizationalUnit": cert.get_organizationalunit(),
        "CommonName": cert.get_cn(),
        "EMail": cert.get_email(),
        "Version": cert.get_version(),
        "SerialNumber": cert.get_serial_number(),
        "SignatureAlgorithm": cert.get_signature_algorithm(),
        "NotBefore": str(cert.get_not_before()),
        "NotAfter": str(cert.get_not_after()),
        "SubjectKeyId": cert.get_subject_keyid(),
        "IsCA": cert.get_isca()
    }

    if info["SubjectKeyId"].upper() in [
        "B4:93:ED:1B:0B:F0:9E:ED:A2:A9:20:62:D2:16:30:4F:A1:8A:43:C6",  # default before 7.2
        "65:BF:03:7C:CA:C3:50:56:8F:44:87:A7:69:0E:D0:3F:BB:44:FF:98"   # default after 7.2
    ]:
        info["DefaultDiladeleKey"] = True
    else:
        info["DefaultDiladeleKey"] = False

    # we only support working with RootCA pem files that contain BOTH cert and private key, so try to load
    key = RootCaKey(pem)
    key.check()

    print (json.dumps(info, sort_keys=True, indent=4))

#
#
#
def create_root_certificate(args):

    # check output file is set
    output_file = args.output
    if output_file is None:
        raise Exception("Path to output file is not set")

    # and do the generation
    exe = OpenSslExe()
    exe.generate_pem(

        args.country, 
        args.province, 
        args.city, 
        args.organization, 
        args.organizational_unit, 
        args.e_mail, 
        args.common_name, 
        args.lifetime,
        output_file
    )

#
# action dispatcher
#
def perform_action(args):

    if args.action == "regenerate-certificate-storage":
        return regenerate_certificate_storage()

    elif args.action == "create-root-certificate":
        return create_root_certificate(args)

    elif args.action == "convert-root-certificate":
        return convert_root_certificate(args.input, args.output)

    elif args.action == "verify-root-certificate":
        return verify_root_certificate(args.input)

    raise Exception("Uknown action '%s' provided." % args.action)

#
# main
#
def main():

    # create parser
    parser = argparse.ArgumentParser(description='CertMgr.py Root CA certificate manipulation script.')

    # add args
    parser.add_argument(
        "--action", 
        required=True, 
        help="action to perform (one of regenerate-certificate-storage, create-root-certificate, convert-root-certificate, verify-root-certificate)"
    )
    parser.add_argument(
        "--input", 
        help="full path to input file (for convert-root-certificate and verify-root-certificate actions)"
    )
    parser.add_argument(
        "--output", 
        help="full path to output file (for convert-root-certificate action only)"
    )
    parser.add_argument(
        "--lifetime", 
        default=365*5, 
        type=int, 
        help="number of days the certificate is valid starting from today (valid only for --action=create-root-certificate"
    )
    parser.add_argument(
        "--country", 
        default="NL", 
        help="2-digit country code (valid only for --action=create-root-certificate)"
    )
    parser.add_argument(
        "--province", 
        default="Noord-Holland", 
        help="state or province name (valid only for --action=create-root-certificate"
    )
    parser.add_argument(
        "--city", 
        default="Amsterdam", 
        help="city (valid only for --action=create-root-certificate"
    )
    parser.add_argument(
        "--organization", 
        default="Example Ltd.", 
        help="organization (valid only for --action=create-root-certificate"
    )
    parser.add_argument(
        "--organizational-unit", 
        default="IT", 
        help="organizational unit (valid only for --action=create-root-certificate"
    )
    parser.add_argument(
        "--e-mail", 
        default="support@example.lan", 
        help="e-mail (valid only for --action=create-root-certificate"
    )
    parser.add_argument(
        "--common-name", 
        default="Squid Proxy", 
        help="common name (valid only for --action=create-root-certificate"
    )

    # parse actual arguments
    args = parser.parse_args()

    # ok let's go
    perform_action(args)

if __name__ == "__main__":

    # call the entry code
    main()

    # if we got here everything is fine
    sys.exit(0)
