#!/usr/bin/env python3
#
# (c) Diladele B.V. 2020, Web Safety
#  
import os
import shutil
import logging
import argparse

# 
#
#
from daemon.core import UiPath, SquidPath
from daemon.stats.collector import CronCollector, CronPurger, Config

#
# 
#
def run(args):

    logging.info("Statistics collector script is starting...")

    # create the config object
    config = Config(args.logs, args.stats)
    config.dump()

    # do nothing if not enabled    
    if not config.collector.enable:

        # dump to log
        logging.info("History collection and reporting is disabled, nothing to do.")

        # and exit
        return

    # we never collect statistics if disk space is low
    usage   = shutil.disk_usage(config.stats_dir)
    free_gb = usage.free / 1024 / 1024 / 1024
    if free_gb < 5:

        # dump to log
        logging.info("History collection requires at least 5GB of free space on disk (you have %dGB), exiting." % free_gb)

        # and exit
        return

    # force means ignore the already collected state, we do that by removing the state file
    if args.force:

        state_file = os.path.join(config.stats_dir, "collector.state")
        if os.path.isfile(state_file):

            # dump to log
            logging.info("Forcing the collection by removing the state file '%s'." % state_file)

            # and force
            os.unlink(state_file)
            
    # first purge the collected statistics if needed based on the log retention settings
    purger = CronPurger(config)
    purger.purge()

    # now enumerate access log storage, parse all access log files and split them by days
    collector = CronCollector(config)
    collector.collect()

    logging.info("Script finished successfully!")

#
# entry point
#
def main():

    # define arguments
    parser = argparse.ArgumentParser(
        description='''
            Statistics collector. Builds CSV files out of Squid's access_log files. 
            Note: assumes the records in access logs are monotonously increasing 
            from day to day. Usually run daily by cron.
        '''
    )

    parser.add_argument("--logs", default="", help="path to folder with logs (optional)")
    parser.add_argument("--stats", default="", help="path to folder with statistics parsed from logs (optional)")
    parser.add_argument("--verbose", help="print additional debug information", action="store_true")
    parser.add_argument("--force", help="force recollection of all statistics", action="store_true")

    # parse them
    args = parser.parse_args()

    # init logger
    level = logging.INFO
    if args.verbose:
        level = logging.DEBUG

    logging.basicConfig(format='%(asctime)s %(message)s', level=level)   

    # build the reports
    run(args)

if __name__ == '__main__':

    main()