import os
import csv
import time
import shutil
import logging
import collections

#
#
#
from daemon.utils import file_size_human

#
#
#
from .entry_factory import EntryFactory
from .logs          import Logs
from .writers       import Writers
from .reader        import Reader
from .excludes      import CronExcludes
from .cluster       import cluster_folders
from .state         import State


#
# walks the log dir and creates daily CSV files in the output folder
#
class CronCollector:

    def __init__(self, config):

        # store dirs        
        self.config  = config
        self.log_dir = config.logs_dir
        self.out_dir = config.stats_dir

        # construct the log folder names
        names = []
        if True:

            # add the squid logs on this machine
            names.extend([self.log_dir])

            # and additional logs that admin would like to manually add
            names.extend(config.collector.folders)

            # if set also the cluster folder
            if config.collector.cluster:
                names.extend(cluster_folders())

            # dump the folders to logging
            logging.info("Will collect access log data from the following folders:")
            for name in names:
                logging.info("\t%s" % name)

        # read all file names from the storage
        self.files = Logs(names).files

        # allocate writers
        self.writers = Writers(self.out_dir)

        # and finally create the state
        self.state = State(config.stats_dir)

    def collect(self):

        logging.info("Collecting files:")

        # these are files that were collected
        collected_files = []

        # look over
        for file in self.files:

            # see if the file needs collection
            (name, checksum, shall_collect) = self.state.shall_collect(file)
            if not shall_collect:

                # just debug check
                assert (name == file)

                # dump it
                logging.info("\t%s has already been collected previously, skipping" % file)

                # and continue
                continue

            # otherwise indeed collect it
            try:

                # note we wrap it into the try catch block to protect us from a bad file, for example a GZ file that is broken
                self.collect_file(file)

                # and save if everything went fine
                collected_files.append((file, checksum))

            except Exception as e:
                logging.warning("\t%s cannot be collected (error: %s), skipping..." % (file, str(e)))

        # flush the writers
        self.writers.flush()

        # if we got here everything was collected successfully, save the state
        self.state.mark_as_collected(collected_files)

    def collect_file(self, file):

        logging.info("\t%s: %s size" % (file, file_size_human(file)))

        # start time
        start = time.time()

        # the collection of errors and total error count
        errors  = collections.deque([], maxlen=50)
        nerrors = 0

        # line number
        parsed  = 0
        
        # create a reader and exclude analyzer
        reader   = Reader(file)
        excludes = CronExcludes(self.config)

        # create entry factory that will construct entries from log lines
        parser = EntryFactory(self.config)

        # until file is completely read
        while True:

            # get a line
            line = reader.getline()
            if not line:
                break

            parsed = parsed + 1

            # create an entry
            try:
                entry = parser.get(line)

                # see if we have to skip this entry
                if excludes.shall_skip(entry):
                    continue

                # and process it                
                self.collect_entry(entry)

            except Exception as e:

                # add this error to the deque that can hold at most N entries
                errors.append("Parsing line %d failed (starts with \"%s\"), error: %s" % (parsed, line[:24], str(e)[:64]))

                # increase the total number of errors
                nerrors = nerrors + 1

        # end time
        end = time.time()

        # dump the end of processing
        logging.info("\t%s: %d lines, %d errors, time %s" % (file, parsed, nerrors, self.elapsed(start, end)))

        # dump warning
        if len(errors) > 0:

            logging.warning("\t\tWarning: %d errors in %s, dumping last %d errors..." % (nerrors, file, len(errors)))
            for error in errors:

                # dump it
                logging.warning("\t\t\t%s" % error)

            logging.warning("\t\tignoring and continuing...")


    def collect_entry(self, entry):

        # just in case
        assert(len(entry.date) > 0)

        # and append it
        self.writers.append(entry)

    def elapsed(self, start, end):

        hours, rem = divmod(end-start, 3600)
        minutes, seconds = divmod(rem, 60)
        return "{:0>2}:{:0>2}:{:0>2}".format(int(hours),int(minutes),int(seconds))

