import os
import csv
import gzip

#
#
#
from daemon.utils import Tailer

#
#
#
from .entry_factory import EntryFactory
from .entry         import Entry
from .excludes      import NowExcludes
from .cluster       import cluster_folders

#
#
#
class NowCollector:

    def __init__(self, config):

        # assign
        logs_dir    = config.logs_dir
        reports_dir = config.reports_dir

        # generate the list of files with realtime log information
        names = []

        if True:

            # always add the squid logs on this machine
            names.append(os.path.join(logs_dir, "access.log"))

            # if set also the cluster folder
            if config.collector.cluster:
                for folder in cluster_folders():
                    realtime_log = os.path.join(folder, "realtime.log")
                    if os.path.isfile(realtime_log):
                        names.append(os.path.join(folder, "realtime.log"))

        # debug just in case
        assert(os.path.isdir(reports_dir))
        assert(os.path.isdir(logs_dir))

        # store it
        self.access_logs = names
        self.reports_dir = reports_dir
        self.config      = config
        
    def collect(self, lines_limit):

        # read entries from all access logs at once
        entries = self.read_entries(lines_limit)
        if not entries:
            raise Exception("No entries were read from access.log, try again later.")

        # this is the folder to store the csv file
        folder = os.path.join(self.reports_dir, "now")
        if not os.path.isdir(folder):

            # recreate
            os.makedirs(folder)

        # create the csv file
        output = os.path.join(folder, 'data.csv')

        # write it
        with open(output, 'w', newline='') as fout:

            # create writer
            writer = csv.writer(fout, delimiter='\t', quotechar='|', quoting=csv.QUOTE_MINIMAL)

            # write header
            writer.writerow(Entry.csv_header())

            # write all entries
            for entry in entries:
                writer.writerow(entry.csv_row())

        # repack data.csv as gzip
        output_gz = output + '.gz'

        if True:
            with open(output, 'rb') as fin:
                with gzip.open(output_gz, 'wb') as fout:
                    fout.writelines(fin)

        # and return the path to that gzipped file (ideally we need to return the CSV itself)
        return output_gz

    def read_entries(self, lines_limit):

        # these are the entries to return
        entries = []

        # loop over all access logs
        for access_log in self.access_logs:

            # and read 2*lines_limit from each access log
            entries.extend(self.read_entry(access_log, 2*lines_limit))

        # now sort them in reversed order to have only fresh entries
        entries.sort(key=lambda x: x.timestamp, reverse=True)

        # cap the final array overall lines
        entries = entries[:lines_limit]

        # return normally
        return entries

    def read_entry(self, access_log, lines_limit):

        # construct the tailer
        tailer = Tailer(access_log)

        # read N lines from it
        lines = tailer.lines(lines_limit)

        # create entry factory that will contruct entries from log lines
        parser   = EntryFactory(self.config)
        excludes = NowExcludes(self.config)

        # loop over the lines and create array of entries
        entries = []

        for line in lines:

            # empty lines are not interesting
            if not line:
                continue

            try:

                # get entry
                entry = parser.get(line)

                # see if we have to skip this entry
                if excludes.shall_skip(entry):
                    continue

                # and append it
                entries.append(entry)

            except Exception as e:

                # while in development
                print("Ignoring exception while parsing log entries: %s", str(e))
            
        # nice, we have entries now
        return entries
