import re
import json
import logging
import datetime

#
#
#
from .entry    import Entry
from .enricher import Enricher

#
#
#
class EntryFactory:

    def __init__(self, config):

        # save config        
        self.config  = config

        #
        # order of these fields must be the same as in logformat directive in /opt/websafety/etc/squid/logfile.conf
        #
        self.squid_fields = [
            "%ts.%03tu",
            "%6tr",
            "%>a",
            "%Ss/%03>Hs",
            "%<st",
            "%rm",
            "%ru",
            "%[un",
            "%Sh/%<a",
            "%mt"
        ]

        #
        # order of these fields must be the same as in logformat directive in /opt/websafety/etc/squid/logfile.conf
        #
        self.web_safety_fields = [
            "\"ws-iid=%{X-WebSafety-IID}adapt::<last_h\"",
            "\"ws-mac=%>eui\"",
            "\"ws-duration=%{X-WebSafety-Duration}adapt::<last_h\"",
            "\"ws-timing=%{X-WebSafety-Timing}adapt::<last_h\"",
            "\"ws-mtime=%{X-WebSafety-Mtime}adapt::<last_h\"",
            "\"ws-scanflags=%{X-WebSafety-ScanFlags}adapt::<last_h\"",
            "\"ws-categories=%{X-WebSafety-Categories}adapt::<last_h\"",
            "\"ws-trusted=%{X-WebSafety-Trusted}adapt::<last_h\"",
            "\"ws-level=%{X-WebSafety-Level}adapt::<last_h\"",
            "\"ws-verdict=%{X-WebSafety-Verdict}adapt::<last_h\"",
            "\"ws-policy=%{X-WebSafety-Policy}adapt::<last_h\"",
            "\"ws-member=%{X-WebSafety-Member}adapt::<last_h\"",
            "\"ws-module=%{X-WebSafety-Module}adapt::<last_h\"",
            "\"ws-msgtype=%{X-WebSafety-MsgType}adapt::<last_h\"",
            "\"ws-param1=%{X-WebSafety-Param1}adapt::<last_h\"",
            "\"ws-param2=%{X-WebSafety-Param2}adapt::<last_h\"",
            "\"ws-debug=%{X-WebSafety-Debug}adapt::<last_h\"",
            "\"squid-gt-st=%>st\""
        ]

    def get(self, line):

        # first we split the incoming line into native squid part and web safety added part
        (s, w) = self.split_line(line)

        # create empty entry
        entry = Entry()

        # parse squid and web safety parts differently
        self.parse_squid(s, entry)
        self.parse_safety(w, entry)

        # do additional data enrichment
        Enricher().enrich(entry)

        # anonymize (todo: make anynimizer class)
        self.anonymize(entry)
        
        # and return nicely
        return entry

    def anonymize(self, entry):

        if entry.ws_policy.lower() in self.config.collector.anonymized_policies:

            entry.user_id           = 'anonymous'
            entry.requestor_ip      = 'anonymous'
            entry.requestor_mac     = 'anonymous'
            entry.requestor_name    = 'anonymous'

    def split_line(self, line):

        # we know the web safety part of line start after the ws-iid= part
        pos = line.find("\"ws-iid=")
        if pos == -1:

            # this must be line from squid log, without web safety part
            return (line, "")

        else:

            # this must be line from squid log with websafety part added
            return (line[:pos], line[pos:])

    def parse_safety(self, line, entry):

        # make a copy
        rest = line

        # loop over all fields and parse them
        for field in self.web_safety_fields:

            # parse it out
            rest = self.parse_safety_field(entry, field, rest)

        # warn if anything left unparsed
        if rest:
            logging.warn("Ignoring unparsed part of web safety log format '%s'" % rest)

    def parse_safety_field(self, entry, field, line):

        if "\"ws-iid=%{X-WebSafety-IID}adapt::<last_h\"" == field:

            match = re.match(r"\"ws-iid=(.+?)\"\s(.*)", line, re.M|re.I)
            if not match:
                return line

            v = match.group(1)
            if v == '-':
                v = 0

            entry.ws_iid = int(v)
            
            return match.group(2).strip()

        elif "\"ws-mac=%>eui\"" == field:

            match = re.match(r"\"ws-mac=(.+?)\"\s(.*)", line, re.M|re.I)
            if not match:
                return line

            v = match.group(1)

            entry.requestor_mac = v
            
            return match.group(2).strip()

        elif "\"ws-duration=%{X-WebSafety-Duration}adapt::<last_h\"" == field:

            match = re.match(r"\"ws-duration=(.+?)\"\s(.*)", line, re.M|re.I)
            if not match:
                return line

            v = match.group(1)
            if v == '-':
                v = 0

            entry.ws_duration = int(v)
            
            return match.group(2).strip()

        elif "\"ws-timing=%{X-WebSafety-Timing}adapt::<last_h\"" == field:

            match = re.match(r"\"ws-timing=(.+?)\"\s(.*)", line, re.M|re.I)
            if not match:
                return line

            v = match.group(1)
            if v == '-':
                v = 0

            entry.ws_timing = int(v)
            
            return match.group(2).strip()

        elif "\"ws-mtime=%{X-WebSafety-Mtime}adapt::<last_h\"" == field:

            match = re.match(r"\"ws-mtime=(.+?)\"\s(.*)", line, re.M|re.I)
            if not match:
                return line

            v = match.group(1)
            if v == '-':
                v = 0

            entry.ws_mtime = int(v)
            
            return match.group(2).strip()

        elif "\"ws-scanflags=%{X-WebSafety-ScanFlags}adapt::<last_h\"" == field:

            match = re.match(r"\"ws-scanflags=(.+?)\"\s(.*)", line, re.M|re.I)
            if not match:
                return line

            v = match.group(1)
            if v == '-':
                v = 0

            entry.ws_scanflags = int(v)
            
            return match.group(2).strip()

        elif "\"ws-categories=%{X-WebSafety-Categories}adapt::<last_h\"" == field:

            match = re.match(r"\"ws-categories=(.+?)\"\s(.*)", line, re.M|re.I)
            if not match:
                return line

            v = match.group(1)
            if v == '-':
                v = 0

            entry.ws_categories = int(v)
            
            return match.group(2).strip()

        elif "\"ws-trusted=%{X-WebSafety-Trusted}adapt::<last_h\"" == field:

            match = re.match(r"\"ws-trusted=(.+?)\"\s(.*)", line, re.M|re.I)
            if not match:
                return line

            v = match.group(1)
            if v == '-':
                v = 0

            entry.ws_trusted = int(v)
            
            return match.group(2).strip()

        elif "\"ws-level=%{X-WebSafety-Level}adapt::<last_h\"" == field:

            match = re.match(r"\"ws-level=(.+?)\"\s(.*)", line, re.M|re.I)
            if not match:
                return line

            v = match.group(1)
            if v == '-':
                v = 0

            entry.ws_level = int(v)
            
            return match.group(2).strip()

        elif "\"ws-verdict=%{X-WebSafety-Verdict}adapt::<last_h\"" == field:

            match = re.match(r"\"ws-verdict=(.+?)\"\s(.*)", line, re.M|re.I)
            if not match:
                return line

            v = match.group(1)
            if v == '-':
                v = 4
            else:
                v = int(v)

            # ok see what verdict we have as number and convert it into string
            verdicts = {
                0: "pass",    # pass_unmodified
                1: "adapt",   # pass_adapted
                2: "block",   # generate_artificial_response
                3: "rescan",  # need_more_data_for_analyzis
                4: "skip"     # means the request was not analysed by web safety
            }

            # debug check
            assert(v in verdicts)

            # and assign
            entry.ws_verdict        = verdicts[v]
            entry.ws_verdict_pass   = [0, 1][entry.ws_verdict == "pass"]
            entry.ws_verdict_adapt  = [0, 1][entry.ws_verdict == "adapt"]
            entry.ws_verdict_block  = [0, 1][entry.ws_verdict == "block"]
            entry.ws_verdict_rescan = [0, 1][entry.ws_verdict == "rescan"]
            entry.ws_verdict_skip   = [0, 1][entry.ws_verdict == "skip"]

            # returnt the rest
            return match.group(2).strip()

        elif "\"ws-policy=%{X-WebSafety-Policy}adapt::<last_h\"" == field:

            match = re.match(r"\"ws-policy=(.+?)\"\s(.*)", line, re.M|re.I)
            if not match:
                return line

            entry.ws_policy = match.group(1)
            
            return match.group(2).strip()

        elif "\"ws-member=%{X-WebSafety-Member}adapt::<last_h\"" == field:

            match = re.match(r"\"ws-member=(.+?)\"\s(.*)", line, re.M|re.I)
            if not match:
                return line

            entry.ws_member = match.group(1)
            
            return match.group(2).strip()

        elif "\"ws-module=%{X-WebSafety-Module}adapt::<last_h\"" == field:

            match = re.match(r"\"ws-module=(.+?)\"\s(.*)", line, re.M|re.I)
            if not match:
                return line

            v = match.group(1)
            if v == '-':
                v = 0

            entry.ws_module = int(v)
            
            return match.group(2).strip()

        elif "\"ws-msgtype=%{X-WebSafety-MsgType}adapt::<last_h\"" == field:

            match = re.match(r"\"ws-msgtype=(.+?)\"\s(.*)", line, re.M|re.I)
            if not match:
                return line

            v = match.group(1)
            if v == '-':
                v = 0

            entry.ws_msgtype = int(v)
            
            return match.group(2).strip()

        elif "\"ws-param1=%{X-WebSafety-Param1}adapt::<last_h\"" == field:

            match = re.match(r"\"ws-param1=(.+?)\"\s(.*)", line, re.M|re.I)
            if not match:
                return line

            entry.ws_param1 = match.group(1)
            
            return match.group(2).strip()

        elif "\"ws-param2=%{X-WebSafety-Param2}adapt::<last_h\"" == field:

            match = re.match(r"\"ws-param2=(.+?)\"\s(.*)", line, re.M|re.I)
            if not match:
                return line

            entry.ws_param2 = match.group(1)
            
            return match.group(2).strip()

        elif "\"ws-debug=%{X-WebSafety-Debug}adapt::<last_h\"" == field:

            match = re.match(r"\"ws-debug=(.+?)\"\s?(.*)", line, re.M|re.I)
            if not match:
                return line

            entry.ws_debug = match.group(1)
            
            return match.group(2).strip()

        elif "\"squid-gt-st=%>st\"" ==  field:

            match = re.match(r"\"squid-gt-st=(.+?)\"\s?(.*)", line, re.M | re.I)
            if not match:
                return line

            v = match.group(1)
            if v == '-':
                v = 0

            entry.request_size = int(v)

            return match.group(2).strip()

        else:

            # well we could not parse, stop then
            return ""


    def parse_squid(self, line, entry):

        # make a copy
        rest = line

        # loop over all fields and parse them
        for field in self.squid_fields:
            # parse it out
            rest = self.parse_squid_field(entry, field, rest)

        # warn if anything left unparsed
        if rest:
            logging.warn("Ignoring unparsed part of squid log format '%s'" % rest)

    def parse_squid_field(self, entry, field, line):

        # for now stupid parsing
        if "%ts.%03tu" == field:

            match = re.match(r"(\d+)\.(\d+)\s(.*)", line, re.M|re.I)
            if not match:
                raise Exception("Cannot parse field '%s' in line '%s'" % (field, line))

            ts = match.group(1)  # seconds since epoch
            tu = match.group(2)  # subsecond time (milliseconds)
            r  = match.group(3)  # rest 

            entry.epoch       = int(ts)
            entry.timestamp   = datetime.datetime.fromtimestamp( ((float(entry.epoch)* 1000) + int(tu)) / 1000 )
            entry.date        = entry.timestamp.strftime("%Y-%m-%d")
            entry.hour_of_day = entry.timestamp.hour

            return r.strip()

        elif "%6tr" == field:

            match = re.match(r"(-?\d+)\s(.*)", line, re.M|re.I)
            if not match:
                raise Exception("Cannot parse field '%s' in line '%s'" % (field, line))

            # response time (milliseconds) and rest
            tr = int(match.group(1))
            r  = match.group(2)

            # assign
            entry.response_time = tr

            return r.strip()

        elif "%>a" == field:

            match = re.match(r"(.+?)\s(.*)", line, re.M|re.I)
            if not match:
                raise Exception("Cannot parse field '%s' in line '%s'" % (field, line))

            # client source ip
            v = match.group(1)
            if not v:
                v = "-"

            entry.requestor_ip = v
            
            return match.group(2).strip()

        elif "%Ss/%03>Hs" == field:

            match = re.match(r"(.+?)/(\d+)\s(.*)", line, re.M|re.I)
            if not match:
                raise Exception("Cannot parse field '%s' in line '%s'" % (field, line))
            
            s = match.group(1)      # request status
            c = int(match.group(2)) # status code sent to client
            r = match.group(3)      # rest

            entry.squid_status = s
            entry.status_code  = c

            return r.strip()

        elif "%<st" == field:

            match = re.match(r"(.+?)\s(.*)", line, re.M|re.I)
            if not match:
                raise Exception("Cannot parse field '%s' in line '%s'" % (field, line))

            entry.reply_size = int(match.group(1))  # total size of reply sent to client (after adaptation)
            
            return match.group(2).strip()

        elif "%rm" == field:

            match = re.match(r"(.+?)\s(.*)", line, re.M|re.I)
            if not match:
                raise Exception("Cannot parse field '%s' in line '%s'" % (field, line))

            entry.request_method = match.group(1)
            
            return match.group(2).strip()

        elif "%ru" == field:

            match = re.match(r"(.+?)\s(.*)", line, re.M|re.I)
            if not match:
                raise Exception("Cannot parse field '%s' in line '%s'" % (field, line))

            # get the url and split it into params
            url = match.group(1)
            pos = url.find("?")
            if pos != -1:
                entry.request_url    = url[:pos]
                entry.request_params = url[pos + 1:]
            else:
                entry.request_url    = url
                entry.request_params = ""

            return match.group(2).strip()

        elif "%[un" == field:

            match = re.match(r"(.+?)\s(.*)", line, re.M|re.I)
            if not match:
                raise Exception("Cannot parse field '%s' in line '%s'" % (field, line))

            v = match.group(1)

            entry.requestor_name = v
            
            return match.group(2).strip()

        elif "%Sh/%<a" == field:

            match = re.match(r"(.+?)/(.+?)\s(.*)", line, re.M|re.I)
            if not match:
                raise Exception("Cannot parse field '%s' in line '%s'" % (field, line))
            
            s = match.group(1)      # squid hierarchy status (default_parent etc)
            c = match.group(2)      # server ip address of the last server or peer connection
            r = match.group(3)      # rest

            return r.strip()

        elif "%mt" == field:

            match = re.match(r"(.+?)$", line, re.M|re.I)
            if not match:
                raise Exception("Cannot parse field '%s' in line '%s'" % (field, line))

            entry.mime_type = match.group(1)

            # this is the last field, nothing left
            return ""
        else:

            # well we could not parse, stop then
            return ""