import os
import re
import collections


#
#
#
class HistoryDir:

    def __init__(self, folder):

        # save the folder
        self.folder = folder

    def enumerate(self):

        # this is the return value
        data = collections.OrderedDict()

        # get entries in the folder (note we need them reverse sorted for better look in UI)
        entries = [f.lower() for f in os.listdir(self.folder)]
        entries = reversed(sorted(entries))

        # loop over them
        for entry in entries:

            # construct path
            entry_dir = os.path.join(self.folder, entry)

            # skip non folders
            if not os.path.isdir(entry_dir):
                continue

            # see if it is like YYYY-MM-DD (for example, 2019-04-16)
            match = re.match( r'^(\d\d\d\d)-(\d\d)-(\d\d)$', entry, re.M|re.I)
            if not match:
                continue

            # get parts
            y = match.group(1)
            m = match.group(2)
            d = match.group(3)

            # call the child class to validate the folder
            if not self.is_valid(entry_dir):
                continue

            # fill it
            self.fill_years(data, y, m, d)

        # and return
        return data

    def is_valid(self, folder):

        # the folder must contain the data.csv file
        if os.path.isfile(os.path.join(folder, "data.csv.gz")):
            return True
        else:
            return False

    def fill_years(self, data, year, month, day):

        # convert year to an int
        year = int(year)

        # add it if not already there
        if not year in data:
            data[year] = {}

        # and enumerate it
        self.fill_months(data[year], month, day)

    def fill_months(self, year, month, day):

        # convert month to an int
        month = int(month)

        # add month if it is not already there
        if not month in year:
            year[month] = []

        self.fill_days(year[month], day)

    def fill_days(self, month, day):

        # convert day to an int
        day = int(day)

        # add day if it is not already there
        if not day in month:
            month.append(day)

#
# history database built from the contents of the /opt/dnssafety/var/stats folder
#
class History:

    #
    # constructor/destructor
    #
    def __init__(self, folder):

        # enumerate all entries in a given folder
        self.storage = HistoryDir(folder).enumerate()

    #
    # public methods
    #
    def years(self):

        # return all available years
        return list(self.storage.keys())

    def months(self, year):

        # no year means nothing is returned
        if year not in self.storage:
            return []

        # get the value
        months = self.storage[year]

        # return all available months for a given year
        return list(months.keys())

    def days(self, year, month):

        # no year means nothing is returned
        if year not in self.storage:
            return []

        # get the months
        months = self.storage[year]

        # no month means nothing is returned
        if month not in months:
            return []

        # return all available days for a given month
        days = months[month]

        return days



