import os
import socket
import shutil
import datetime

#
#
#
from daemon.core  import Build, CoreBuild, System, Distrib
from daemon.utils import remake_folder, JsonWriter

#
#
#
from .storage import Storage
from .mailer  import Mailer

#
#
#
class Reporter:

    def __init__(self, stats_dir, reports_dir):

        self.stats_dir   = stats_dir
        self.reports_dir = reports_dir

    def report(self, config):

        # report will live in this folder
        folder = os.path.join(self.reports_dir, config.name)

        # first we build report
        self.build(folder, config)

        # then we mail it
        self.mail(folder, config)

    def build(self, target_dir, config):

        # generate the temporary working folder to render the reports to
        tmp_dir = target_dir + ".new"
        
        # re initialize the working folder
        remake_folder(tmp_dir)

        # ask the child for specific loader class
        loader = self.get_loader()

        # load the data from the statistics storage
        s  = Storage(self.stats_dir, loader)
        df = s.load(config)

        # run the actual reporting on this dataframe into the folder
        self.report_impl(tmp_dir, config, df)

        # ok reporting is finished nicely, make a backup
        bak_dir = target_dir + ".bak"
        if True:

            if os.path.isdir(bak_dir):
                shutil.rmtree(bak_dir)

            if os.path.isdir(target_dir):
                shutil.move(target_dir, bak_dir)

        # save the meta info
        self.report_meta(tmp_dir)

        # and finally publish
        shutil.move(tmp_dir, target_dir)

    def get_meta(self):

        return {
            "generated_on" : datetime.datetime.now().strftime("%a, %d %b %Y %H:%M:%S"),
            "version_core" : CoreBuild.version(),
            "version_ui"   : Build.version(),
            "system"       : System.name(),
            "arch"         : Distrib.arch(),
            "distrib"      : Distrib.name(),
            "hostname"     : socket.gethostname()
        }

    def report_meta(self, folder):

        data = self.get_meta()

        JsonWriter(os.path.join(folder, "meta.json")).write(data)

    def mail(self, folder, config):

        # if no e-mails configured, do nothing
        if not config.send_to_emails:
            return

        # otherwise send
        attached_html = os.path.join(folder, "index.html")
        subject       = "Web Safety report '%s'" % config.name
        recipients    =  ",".join(config.send_to_emails)

        try:
            m = Mailer(config.mailer)
            m.send(recipients, attached_html, subject)
        except Exception as e:
            print(e)

    def get_loader(self):
        raise Exception("The get_loader function MUST be overridden by the child reporter class!")