import os
import numpy  as np
import pandas as pd

#
#
#
from django.template import loader, Context

#
#
#
from daemon.utils  import JsonWriter
from daemon.safety import BuildInCategories, categories_to_strings

#
#
#
from .reporter import Reporter
from .renderer import Renderer
from .loader   import Loader

#
#
#
def categories_to_array(categories_as_flags, count):

    # this is the dict with set categories
    assigned = {}

    # convert the categories to string array
    categories = BuildInCategories.unmap_name(categories_as_flags)

    # loop over the names
    for name in BuildInCategories.CATEGORY_NAMES:
        if name in categories:
            assigned[name] = count
        else:
            assigned[name] = 0

    # if no categories set, mark as none
    if categories_as_flags == 0:
        assigned["none"] = count
    else:
        assigned["none"] = 0

    # and return assigned dict as pandas Series
    result = pd.Series(assigned) 

    # and return nicely
    return result

#
#
#
class DetectedCategories(Reporter):

    template_path = "detected_categories.html"

    def report_impl(self, folder, config, df):

        # generate
        (view, totals) = self.generate(config, df)

        # save view
        view.to_csv(os.path.join(folder, "data.csv"), encoding='utf-8')

        # save totals
        JsonWriter(os.path.join(folder, "data.json")).write(totals)

        # now render them as html
        text = Renderer().render(
            self.template_path, {
                "view"   : view,
                "totals" : totals,
                "config" : config,
                "meta"   : self.get_meta()
            }
        )

        # and save
        with open(os.path.join(folder, "index.html"), "w") as fout:
            fout.write(text)

    def generate(self, config, df):

        # calculate view
        view   = self.calculate_view(df)
        totals = self.calculate_totals(view)

        # and return
        return (view, totals)

    def calculate_view(self, df):

        # we only need one column
        view_row = df[ ["ws_categories"] ]

        # aggregate by that column to make the view smaller
        view = view_row.groupby(
            ["ws_categories"]
        ).agg({ "ws_categories" : "count"}) 

        # rename the column
        view.rename(columns={'ws_categories': 'count'}, inplace=True)

        # index is category flags, so add columns to the view based on it
        view = view.apply(lambda row: categories_to_array(int(row.name), row["count"]), axis=1)
        view.reset_index(inplace=True)
        view.drop('ws_categories', 1, inplace=True)

        # aggregate the view by rows
        view = view.agg(np.sum, axis=0)

        # and transpose it turning colums into rows
        view = view.transpose()

        # view is series we need dataframe
        view1 = pd.DataFrame( {'categories':view.index, 'requests': view.values } )

        # finally sort
        view1 = view1.sort_values("requests", ascending=False)

        # calculate percentage of counts
        if True:

            total_sum = view1["requests"].sum()
            if total_sum > 0:
                view1["requests_percent"] = np.ceil(100 * view1["requests"] / total_sum).astype(int)
            else:
                view1["requests_percent"] = 0

        # and return nicely
        return view1

    def calculate_totals(self, view):

        return {
            'requests_total' : int(view['requests'].sum())
        }



    def get_loader(self):

        parse_dates = []
        usecols     = [
            "ws_categories",
            "fldomain", 
            "user_id",
            "ws_policy"
        ]
        dtype       = {
            "ws_categories" : "uint64"
        }
        
        return Loader(parse_dates, usecols, dtype)


