import os
import datetime
import logging
import pandas as pd

#
#
#
from dateutil.relativedelta import relativedelta

#
#
#
from daemon.utils import file_size_human


#
# iterate by date between two days
#
def daterange(start_date, end_date):

    for n in range(int ((end_date - start_date).days)):
        yield start_date + datetime.timedelta(n)

#
#
#
class Storage:

    def __init__(self, stats_dir, csv_loader):

        self.stats_dir  = stats_dir
        self.csv_loader = csv_loader

    def load(self, config):

        timeframe = config.timeframe
        filters   = config.filters

        # we need to construct the array of days to load from statistics storage
        dates = self.analyze(timeframe)

        # nice, load all those dates
        return self.read(dates, filters)

    def analyze(self, timeframe):

        # this is today, needed almost always, so be it here
        today = datetime.date.today()

        # debug check
        if timeframe.time_type.startswith("last_n_"):
            assert(timeframe.last_value > 0)

        # see what type of time table we have
        if timeframe.time_type == "last_n_days":

            # determine yesterday
            yesterday = today - datetime.timedelta(days=1)

            # check how many days back we have to go
            result = []
            
            if timeframe.last_value > 1:

                for item in daterange(
                    yesterday - datetime.timedelta(days=timeframe.last_value - 1), 
                    yesterday
                ):
                    result.append(item.strftime("%Y-%m-%d"))

            # always append yesterday at the end
            result.append(yesterday.strftime("%Y-%m-%d"))

            # see if we also need to include today
            if timeframe.include_current:

                # append today
                result.append(today.strftime("%Y-%m-%d"))

                # and make sure we get last n days only
                result = result[-timeframe.last_value:]

            # nice
            return result

        elif timeframe.time_type == "last_n_weeks":

            # we get start day by first findign the beginning of our week and then subtracting required weeks from it
            start_week = today - datetime.timedelta(days=today.weekday())

            # this is the number of weeks we have to take
            rewind_weeks = timeframe.last_value

            # if we include current week, rewind must be decreased by one
            if timeframe.include_current:
                if rewind_weeks > 0:
                    rewind_weeks = rewind_weeks - 1

            # now we rewind the start date back for the require number of weeks
            start_date = start_week - datetime.timedelta(weeks=rewind_weeks)
            end_date   = start_date + datetime.timedelta(weeks=rewind_weeks)

            # cool, feel the result array
            result = []
            for item in daterange(start_date, end_date):
                result.append(item.strftime("%Y-%m-%d"))

            # see if we also need to include current week
            if timeframe.include_current:

                # append all days on this current week too
                tomorrow = today + datetime.timedelta(days=1)
                for item in daterange(start_week, tomorrow):
                    result.append(item.strftime("%Y-%m-%d"))

            # and return
            return result

        elif timeframe.time_type == "last_n_months":

            # get the first date of the current month
            start_month = datetime.date.today().replace(day=1)

            # this is the number of weeks we have to take
            rewind_months = timeframe.last_value

            # if we include current month, rewind must be decreased by one
            if timeframe.include_current:
                if rewind_months > 0:
                    rewind_months = rewind_months - 1

            # now we rewind the start date back for the require number of months
            start_date = start_month - relativedelta(months=rewind_months) 
            end_date   = start_date  + relativedelta(months=rewind_months) 

            # cool, feel the result array
            result = []
            for item in daterange(start_date, end_date):
                result.append(item.strftime("%Y-%m-%d"))

            # see if we also need to include current week
            if timeframe.include_current:

                # append all days on this current month too
                tomorrow = today + datetime.timedelta(days=1)
                for item in daterange(start_month, tomorrow):
                    result.append(item.strftime("%Y-%m-%d"))

            # and return
            return result
            
        elif timeframe.time_type == "from_to":

            start_date = datetime.datetime.strptime(timeframe.time_from, "%Y-%m-%d").date()
            end_date   = datetime.datetime.strptime(timeframe.time_to, "%Y-%m-%d").date() + datetime.timedelta(days=1)

            # cool, feel the result array
            result = []
            for item in daterange(start_date, end_date):
                result.append(item.strftime("%Y-%m-%d"))

            # and return
            return result

        else:

            # hm strange
            raise Exception("Unknown time frame type '%s'" % timeframe.time_type)

    def read(self, dates, filters):

        # limit the memory size to 500 Mb?
        max_mem  = 500 * 1024 * 1024
        used_mem = 0

        # list of loaded dataframes
        loaded = []

        # loop over each date
        for date in dates:

            # see if this csv file is present
            data_csv = os.path.join(self.stats_dir, date, "data.csv.gz")
            if not os.path.isfile(data_csv):
                continue

            logging.info("Loading CSV file '%s': %s size" % (data_csv, file_size_human(data_csv)))
            
            # load it
            df = self.load_csv(data_csv)

            # see how much memory it uses
            used_mem += df.memory_usage(index=True).sum()
            if used_mem > max_mem:
                raise Exception("Cannot load CSV file '%s' because total memory usage limit is exceeded (%d > %d)" % (data_csv, used_mem, max_mem))

            # filter it
            df = self.filter(df, filters)

            # and add to the array
            loaded.append(df)

        # finally merge into one dataframe and return
        return pd.concat(loaded, axis=0, ignore_index=True, sort=False)

    def load_csv(self, data_csv):

        # just call the loader instance we got in the constructor, I know it is ugly design
        return self.csv_loader.load_csv(data_csv)

        # instead of loading all
        # return pd.read_csv(data_csv, sep='\t', parse_dates=[1])

        # read all records from the CSV file, but include only some columns
        '''
        df = pd.read_csv(
            data_csv, sep='\t', parse_dates=["timestamp"],
            usecols=[
                "timestamp",
                "date",
                "request_url",
                "request_params",
                "reply_size",
                "user_id",
                "ws_iid",
                "ws_policy",
                "fldomain",
                "ws_verdict_pass",
                "ws_verdict_adapt",
                "ws_verdict_block",
                "ws_verdict_rescan",
                "ws_verdict_skip",
                "ws_offensive",
                "ws_categories"
            ],
            dtype = {
                "ws_verdict_pass"   : "uint8",
                "ws_verdict_adapt"  : "uint8",
                "ws_verdict_block"  : "uint8",
                "ws_verdict_rescan" : "uint8",
                "ws_verdict_skip"   : "uint8",
                "ws_offensive"      : "uint8",
                "ws_categories"     : "uint64"
            }
        )

        # and return
        return df
        '''

    def filter(self, dataframe, filters):

        # for convinience
        df = dataframe

        # first do the exclusions by domains
        if len(filters.exclude_domains) > 0:

            # exclude specific domains
            df = df.loc[
                ~df["fldomain"].isin(filters.exclude_domains)
            ]

            # dump
            logging.info("Excluded domains '%s'" % ", ".join(filters.exclude_domains) )

        # exclude users
        if len(filters.exclude_users) > 0:

            # exclude specific domains
            df = df.loc[
                ~df["user_id"].isin(filters.exclude_users)
            ]

            # dump
            logging.info("Excluded users '%s'" % ", ".join(filters.exclude_users) )

        # exclude policies
        if len(filters.exclude_policies) > 0:

            # exclude specific domains
            df = df.loc[
                ~df["ws_policy"].isin(filters.exclude_policies)
            ]

            # dump
            logging.info("Excluded policies '%s'" % ", ".join(filters.exclude_policies) )


        # now do the inclusions - first by domains
        if len(filters.include_domains) > 0:

            # include specific domains
            df = df.loc[
                df["fldomain"].isin(filters.include_domains)
            ]

            # dump
            logging.info("Only included domains '%s'" % ", ".join(filters.include_domains) )

        # then by users
        if len(filters.include_users) > 0:

            # include specific domains
            df = df.loc[
                df["user_id"].isin(filters.include_users)
            ]

            # dump
            logging.info("Only included users '%s'" % ", ".join(filters.include_users) )

        # finally by policies
        if len(filters.include_policies) > 0:

            # include specific domains
            df = df.loc[
                df["ws_policy"].isin(filters.include_policies)
            ]

            # dump
            logging.info("Only included policies '%s'" % ", ".join(filters.include_policies) )

        # perfect return now
        return df
