import os
import pandas as pd

#
#
#
from daemon.utils import JsonWriter, remake_folder

#
#
#
from .aggregators import RequestorAggregator, DomainAggregator

#
#
#
from .builder_summary import SummaryBuilder
from .builder_hits    import HitsBuilder
from .builder_bytes   import BytesBuilder

#
#
#
class ItemBuilder:

    def __init__(self, builder_type, folder, item):

        # save builder type
        self.builder_type = builder_type

        # the subfolder name cannot contain some characters
        self.item     = item
        self.item_dir = item.replace(":", "_")

        # construct subfolder but do NOT create it for now
        self.folder = os.path.join(folder, self.sub_folder, self.item_dir)

    def build(self, periods):

        # adjust the periods for the specific item type
        periods1 = []
        for period in periods:

            # this is the folder of the user
            period_dir = os.path.join(period, self.sub_folder, self.item_dir)

            # it might not exist because we have limits on the detailed info and it *might* be this item was skipped
            if os.path.isdir(period_dir):
                periods1.append(period_dir)

        # see we have at least some periods to aggregate
        if not len(periods1):
            return

        # ensure exists
        remake_folder(self.folder)

        # aggregate summary
        self.build_summary(periods1)
        self.build_hits(periods1)
        self.build_bytes(periods1)

        # call the child class for other specific aggregations
        self.build_impl(periods1)

    def build_summary(self, periods):

        r = SummaryBuilder(self.folder)
        r.build(periods)

    def build_hits(self, periods):

        r = HitsBuilder(self.builder_type, self.folder)
        r.build(periods)

    def build_bytes(self, periods):

        r = BytesBuilder(self.builder_type, self.folder)
        r.build(periods)

#
#
#
class UserBuilder(ItemBuilder):

    sub_folder = "users"

    def build_impl(self, periods):

        # report domains of different type
        for domain_type in [
            "domains_all", 
            "domains_blocked", 
            "domains_offensive"
        ]:
            self.build_domain_type(domain_type, periods)

        #self.report_time_of_day(g)
        #self.report_activity(g)

    def build_domain_type(self, domain_type, periods):

        # combine all records into one big dataframe
        df = self.load_domains(domain_type, periods)

        # create aggregator
        a = RequestorAggregator()

        # aggregate
        (v, t) = a.domains(df)

        # save the table
        v.to_csv(os.path.join(self.folder, "%s.csv" % domain_type), encoding='utf-8')

        # and save the totals
        JsonWriter(os.path.join(self.folder, "%s.json" % domain_type)).write(t)

    def load_domains(self, domain_type, periods):

        # list of loaded dataframes
        loaded = []

        # load all folders
        for period in periods:

            file = os.path.join(period, "%s.csv" % domain_type)
            if not os.path.isfile(file):
                continue

            loaded.append(pd.read_csv(file))

        # merge into one dataframe
        return pd.concat(loaded, axis=0, ignore_index=True, sort=False)


    '''
    def report_activity(self, g):

        # activity is all records for given label/ip/mac or domain
        d = g.activity()

        d.to_csv(
            os.path.join(self.folder, "activity.csv"), encoding='utf-8', index_label='line'
        )

    def report_time_of_day(self, g):

        # first save distribution of requests per given hour per each domain name
        g.hourly_requests_per_domain().to_csv(
            os.path.join(self.folder, "hourly_requests_per_domain.csv"), encoding='utf-8'
        )

        g.hourly_bandwidth_per_domain().to_csv(
            os.path.join(self.folder, "hourly_bandwidth_per_domain.csv"), encoding='utf-8'
        )
    '''

#
#
#
class DomainBuilder(ItemBuilder):

    #aggregator_class = MacAggregator
    sub_folder       = "domains"

    def build_impl(self, periods):

        # requestors
        self.report_requestors(periods)
        
        # finally
        # self.report_time_of_day(g)
        # self.report_activity(g)

    def report_requestors(self, periods):

        for group_by, csv_name in [
            ("user_id", "users")
        ]:

            # load data frame
            df = self.load_dataframes(periods, "%s.csv" % csv_name)

            # run aggregator
            (v, t) = DomainAggregator().aggregate(df, group_by)

            # and save
            v.to_csv(os.path.join(self.folder, "%s.csv" % csv_name), encoding='utf-8')

            # save the totals
            JsonWriter(os.path.join(self.folder, "%s.json" % csv_name)).write(t)
        
    def load_dataframes(self, periods, name):

        # list of loaded dataframes
        loaded = []

        # load all folders
        for period in periods:

            file = os.path.join(period, name)
            if not os.path.isfile(file):
                continue

            loaded.append(
                pd.read_csv(file)
            )

        # merge into one dataframe
        return pd.concat(loaded, axis=0, ignore_index=True, sort=False)

    

    '''
    def report_activity(self, g):

        # activity is all records for given label/ip/mac or domain
        d = g.activity()

        d.to_csv(
            os.path.join(self.folder, "activity.csv"), encoding='utf-8', index_label='line'
        )

    def report_time_of_day(self, g):

        # dump hits
        g.hourly_hits_per_user().to_csv(
            os.path.join(self.folder, "hourly_hits_per_user.csv"), encoding='utf-8'
        )
        g.hourly_hits_per_ip().to_csv(
            os.path.join(self.folder, "hourly_hits_per_ip.csv"), encoding='utf-8'
        )
        g.hourly_hits_per_mac().to_csv(
            os.path.join(self.folder, "hourly_hits_per_mac.csv"), encoding='utf-8'
        )

        # dump bandwidth
        g.hourly_bandwidth_per_user().to_csv(
            os.path.join(self.folder, "hourly_bandwidth_per_user.csv"), encoding='utf-8'
        )
        g.hourly_bandwidth_per_ip().to_csv(
            os.path.join(self.folder, "hourly_bandwidth_per_ip.csv"), encoding='utf-8'
        )
        g.hourly_bandwidth_per_mac().to_csv(
            os.path.join(self.folder, "hourly_bandwidth_per_mac.csv"), encoding='utf-8'
        )
    '''
