import os
import pandas as pd
import logging

#
#
#
from daemon.utils import JsonWriter, remake_folder

#
#
#
from .generators     import *
from .builder_user   import *
from .builder_domain import *

#
# presentst the limitations for the built information
#
class DetailedInfoLimits:

    def __init__(self):

        self.users_limit   = 50
        self.domains_limit = 100

#
# specific builder builds data based on a single date
#
class Builder:

    def __init__(self, builder_type, folder):

        # make the folder if not yet there
        self.builder_type = builder_type
        self.folder       = folder

    def build(self, file, limits):

        # debug check
        assert(type(limits) == DetailedInfoLimits)

        # read all records from the CSV file, but include only some columns
        df = pd.read_csv(
            file, sep='\t', parse_dates=["timestamp"],
            usecols=[
                "timestamp",
                "date",
                "request_url",
                "request_size",
                "reply_size",
                "user_id",
                "ws_policy",
                "fldomain",
                "ws_verdict_pass",
                "ws_verdict_adapt",
                "ws_verdict_block",
                "ws_verdict_rescan",
                "ws_verdict_skip",
                "ws_offensive",
                "ws_categories"
            ],
            dtype = {
                "ws_verdict_pass"   : "uint8",
                "ws_verdict_adapt"  : "uint8",
                "ws_verdict_block"  : "uint8",
                "ws_verdict_rescan" : "uint8",
                "ws_verdict_skip"   : "uint8",
                "ws_offensive"      : "uint8",
                "ws_categories"     : "uint64"
            }
        )

        # run various builders on this partial dataframe
        self.build_summary(df)
        self.build_hits(df)
        self.build_bytes(df)
        
        # build total users
        if True:

            # built the table of all users available
            users = self.build_users(df)

            # then build detailed info for some users only
            top_n = limits.users_limit
            if top_n == 0:
                logging.info("Detailed info for all %d users:" % len(users))
            else:
                logging.info("Detailed info for the first %d users with most bandwidth consumed of %d total users:" % (top_n, len(users)))

                users = users[:top_n]

            # sort them to look nice when building
            users.sort()

            # loop over and build
            for user in users:
                self.build_user(file, user)

            # finally build files
            self.build_downloaded_files(df)
            self.build_uploaded_files(df)

        # build total domains
        if True:

            # built the table of all domains available
            domains = self.build_domains(df)

            # then build detailed info for some domains only
            top_n = limits.domains_limit
            if top_n == 0:
                logging.info("Detailed info for all %d domains:" % len(domains))
            else:
                logging.info("Detailed info for the first %d domains with most bandwidth consumed of %d total domains:" % (top_n, len(domains)))

                domains = domains[:top_n]

            # sort them to look nice when building
            domains.sort()

            # loop over and build
            for domain in domains:
                self.build_domain(file, domain)
                
    def build_summary(self, df):

        # generate summary for the dataframe
        v = SummaryGenerator().generate(df)

        # and save it as JSON 
        JsonWriter(os.path.join(self.folder, "summary.json")).write(v)

    def build_hits(self, df):

        # how hits shall be generated - switch by type anti-pattern :(
        if self.builder_type == "now":
            v = HitsPerMinuteGenerator().generate(df)
        elif self.builder_type == "day":
            v = HitsPer10MinutesGenerator().generate(df)
        else:
            v = HitsPerHourGenerator().generate(df)

        # save to csv
        v.to_csv(os.path.join(self.folder, "hits.csv"), encoding='utf-8')

    def build_bytes(self, df):

        # how bytes shall be generated - switch by type anti-pattern :(
        if self.builder_type == "now":
            v = BytesPerMinuteGenerator().generate(df)
        elif self.builder_type == "day":
            v = BytesPer10MinutesGenerator().generate(df)
        else:
            v = BytesPerHourGenerator().generate(df)

        # save to csv
        v.to_csv(os.path.join(self.folder, "bytes.csv"), encoding='utf-8')

    def build_downloaded_files(self, df):

        # generate
        v = DownloadedFilesGenerator().generate(df)

        # and save to csv
        v.to_csv(os.path.join(self.folder, "downloaded_files.csv"), encoding='utf-8')

    def build_uploaded_files(self, df):

        # generate
        v = UploadedFilesGenerator().generate(df)

        # and save to csv
        v.to_csv(os.path.join(self.folder, "uploaded_files.csv"), encoding='utf-8')

    def build_users(self, df): #, top_n):

        # construct folder and files paths
        out_dir    = os.path.join(self.folder, "users")
        items_csv  = os.path.join(out_dir, "items.csv")
        items_json = os.path.join(out_dir, "items.json")

        # ensure we have it on disk
        remake_folder(out_dir)

        # generate view and totals
        (view, totals) = UsersGenerator().generate(df)

        # save view
        view.to_csv(items_csv, encoding='utf-8')

        # save totals
        JsonWriter(items_json).write(totals)

        # return all users
        return view.index.tolist()

        '''

        # make a list of it
        users = view.index.tolist()

        # the view has all the users now, but detailed info is generated for only some of them
        if top_n > 0:
            users = users[:top_n]
            users.sort()

        # append a column to the view denoting users with detailed info available
        view['detailed'] = view.apply(lambda row: row.name in users, axis=1)

        # save view
        view.to_csv(items_csv, encoding='utf-8')

        # save totals
        JsonWriter(items_json).write(totals)

        # and return those users who need to be detailed generated
        return users, len(view.index)
        '''

    def build_domains(self, df):

        # construct folder and files paths
        out_dir    = os.path.join(self.folder, "domains")
        items_csv  = os.path.join(out_dir, "items.csv")
        items_json = os.path.join(out_dir, "items.json")

        # ensure we have it on disk
        remake_folder(out_dir)

        # generate view and totals
        (view, totals) = DomainsGenerator().generate(df)

        # save view
        view.to_csv(items_csv, encoding='utf-8')

        # save totals
        JsonWriter(items_json).write(totals)

        # and return those users who need to be detailed generated
        return view.index.tolist()

    def build_user(self, file, user):

        logging.info("\t%s" % user)

        # will append all chunks here
        chunks = []

        # create the chunked CSV reader (by 100000 rows)
        reader = pd.read_csv(file, sep='\t', parse_dates=[1], chunksize=100000)

        # loop over
        for rchunk in reader:

            # filter the raw chunk
            fchunk = rchunk[ rchunk["user_id"] == user ]

            # append it
            chunks.append(fchunk)

        # and concatenate into dataframe
        df = pd.concat(chunks)

        # and build individual report
        return self.build_item(df, "users", UserBuilder, user)

    def build_domain(self, file, domain):

        logging.info("\t%s" % domain)

        # will append all chunks here
        chunks = []

        # create the chunked CSV reader (by 100000 rows)
        reader = pd.read_csv(file, sep='\t', parse_dates=[1], chunksize=100000)

        # loop over
        for rchunk in reader:

            # filter the raw chunk
            fchunk = rchunk[rchunk["fldomain"] == domain]

            # append it
            chunks.append(fchunk)

        # and concatenate into dataframe
        df = pd.concat(chunks)

        # and build individual report
        return self.build_item(df, "domains", DomainBuilder, domain)

    def build_item(self, df, sub_dir, builder_class, item):

        # construct folder
        safe_name = item.replace(":", "_")
        out_dir   = os.path.join(self.folder, sub_dir, safe_name)

        # ensure it exist
        remake_folder(out_dir)

        # and build
        b = builder_class(self.builder_type, out_dir)
        b.build(df)
