import os
import pandas as pd

#
#
#
from daemon.utils import JsonWriter

#
#
#
from .generators import *

#
# individual builder - base class for a give ip/name/mac/domain builders
#
class ItemBuilder:
    
    def __init__(self, builder_type, folder):

        self.builder_type = builder_type
        self.folder       = folder

    def build(self, df):

        # generate common files
        self.build_summary(df)
        self.build_hits(df)
        self.build_bytes(df)

        # call the child class for more specific build steps
        self.build_impl(df)

        # build activity must always come last, because we drop the request_params column here
        self.build_activity(df)

    def build_summary(self, df):

        # generate summary for the dataframe
        v = SummaryGenerator().generate(df)

        # and save it as JSON 
        JsonWriter(os.path.join(self.folder, "summary.json")).write(v)

    def build_hits(self, df):

        # how hits shall be generated - switch by type anti-pattern :(
        if self.builder_type == "now":
            v = HitsPerMinuteGenerator().generate(df)
        elif self.builder_type == "day":
            v = HitsPer10MinutesGenerator().generate(df)
        else:
            v = HitsPerHourGenerator().generate(df)

        # save to csv
        v.to_csv(os.path.join(self.folder, "hits.csv"), encoding='utf-8')

    def build_bytes(self, df):

        # how bytes shall be generated - switch by type anti-pattern :(
        if self.builder_type == "now":
            v = BytesPerMinuteGenerator().generate(df)
        elif self.builder_type == "day":
            v = BytesPer10MinutesGenerator().generate(df)
        else:
            v = BytesPerHourGenerator().generate(df)

        # save to csv
        v.to_csv(os.path.join(self.folder, "bytes.csv"), encoding='utf-8')

    def build_activity(self, df):

        # delete the request_params column
        del df['request_params']

        # activity is all records for given label/ip/mac or domain
        df.to_csv(
            os.path.join(self.folder, "activity.csv"), encoding='utf-8', index_label='line'
        )
