import numpy  as np
import pandas as pd

#
#
#
class DomainGenerator:

    def generate(self, df, group_by):

        view   = self.calculate_view(df, group_by)
        totals = self.calculate_totals(view)

        return (view, totals)

    def calculate_view(self, df, group_by):

        # group records by requestor, then calculate hits and offensive hits per group
        view = df.groupby(
            [group_by]
        ).agg(
            { 
                "request_url"       : 'count',
                "reply_size"        : 'sum',
                "request_size"      : 'sum',
                "ws_verdict_pass"   : 'sum',
                "ws_verdict_adapt"  : 'sum',
                "ws_verdict_block"  : 'sum',
                "ws_verdict_rescan" : 'sum',
                "ws_verdict_skip"   : 'sum',
                "ws_offensive"      : 'sum'
            }
        )

        # calculate percentage of requests
        if True:

            total_sum = view["request_url"].sum()
            if total_sum > 0:
                view["requests_percent"] = np.ceil(100 * view["request_url"] / total_sum).astype(int)
            else:
                view["requests_percent"] = 0

        # calculate percentage of downloads
        if True:

            total_sum = view["reply_size"].sum()
            if total_sum > 0:
                view["bandwidth_percent"] = np.ceil(100 * view["reply_size"] / total_sum).astype(int)
            else:
                view["bandwidth_percent"] = 0

        # calculate percentage of uploads
        if True:

            total_sum = view["request_size"].sum()
            if total_sum > 0:
                view["request_size_percent"] = np.ceil(100 * view["request_size"] / total_sum).astype(int)
            else:
                view["request_size_percent"] = 0

        # rename columns
        # view.rename(columns={group_by:'requests'}, inplace=True)

        # sort them
        view = view.sort_values('reply_size', ascending=False)

        # and return nicely
        return view

    def calculate_totals(self, view):

        return {
            'total_request_url'       : int(view['request_url'].sum()),
            'total_reply_size'        : int(view['reply_size'].sum()),
            'total_request_size'      : int(view['request_size'].sum()),
            'total_ws_verdict_pass'   : int(view['ws_verdict_pass'].sum()),
            'total_ws_verdict_adapt'  : int(view['ws_verdict_adapt'].sum()),
            'total_ws_verdict_block'  : int(view['ws_verdict_block'].sum()),
            'total_ws_verdict_rescan' : int(view['ws_verdict_rescan'].sum()),
            'total_ws_verdict_skip'   : int(view['ws_verdict_skip'].sum()),
            'total_ws_offensive'      : int(view['ws_offensive'].sum())
        }

    def hourly_hits_per_user(self, df):
        return self.hourly_hits_per(df, 'user_id')

    def hourly_hits_per(self, df, requestor):
        return self.hourly_per(df, 'request_url', 'count', requestor)

    def hourly_bandwidth_per_user(self, df):
        return self.hourly_bandwidth_per(df, 'user_id')

    def hourly_bandwidth_per(self, df, requestor):
        return self.hourly_per(df, 'reply_size', 'sum', requestor)

    def hourly_uploads_per_user(self, df):
        return self.hourly_uploads_per(df, 'user_id')

    def hourly_uploads_per(self, df, requestor):
        return self.hourly_per(df, 'request_size', 'sum', requestor)

    def hourly_per(self, df, value_field, aggregation_func, requestor):

        # group records by requestor, then by hour of day, and calculate how many records are per hour by day
        view = df.groupby(
            [requestor, 'hour_of_day']
        ).agg(
            { value_field: aggregation_func }
        )

        #
        # pivot the table; see for the reasoning the article at
        # https://stackoverflow.com/questions/31975139/reshaping-pandas-groupby-data-row-values-into-column-headers
        #
        view = view.reset_index().pivot(requestor, 'hour_of_day', value_field)

        # make a new dataframe with columns corresponding to hours of a day
        view1 = pd.DataFrame(view, columns=list(range(0, 24)), index=view.index)

        # set all NaN into zeros to look better
        view1.fillna(0, inplace=True)

        # calculate totals per row
        view1['per_day'] = \
            view1[0] + view1[1] + view1[2] + view1[3] + view1[4] + view1[5] + view1[6] + view1[7] + view1[8] + \
            view1[9] + view1[10] + view1[11] + view1[12] + view1[13] + view1[14] + view1[15] + view1[16] + view1[17] + \
            view1[18] + view1[19] + view1[20] + view1[21] + view1[22] + view1[23]

        # after pivot the values are float, we need ints
        view1 = view1.astype('int64')

        # then percents
        totals_per_day = view1["per_day"].sum()
        if totals_per_day > 0:
            view1["per_day_percent"] = np.ceil(100 * view1["per_day"] / totals_per_day).astype(int)
        else:
            view1["per_day_percent"] = 0

        # finally sort descending
        view1 = view1.sort_values('per_day', ascending=False)

        # and return
        return view1
