import os
import requests
import numpy  as np
import pandas as pd
import urllib.parse

#
#
#
from daemon.core  import DaemonPath
from daemon.utils import JsonReader
#
#
#
def is_youtube(row):

    request_url = row['request_url']

    if request_url.endswith(".youtube.com/watch"):
        return True

    return False

#
#
#
class VideosGenerator:

    def __init__(self):

        # we will need the youtube api key to resolve title and description of the videos
        api_key = ""
        try:
            data    = JsonReader(os.path.join(DaemonPath.etc_dir(), "safety", "youtube.json")).read_object()
            api_key = data.get("APIKey", "")
        except Exception as e:
            pass

        # and assign possibly empty api key
        self.api_key = api_key

    def generate(self, dataframe):

        # select only youtube videos
        df = dataframe[
            dataframe.apply(is_youtube, axis=1)
        ]

        # leave only needed columns
        df = df[ ["ws_iid", "timestamp", "user_id", "request_url", "request_params"] ]

        # set all NaN into empty string to look better
        df.fillna("", inplace=True)

        # enrich the dataframe
        df = self.enrich(df)

        # and return
        return df

    def enrich(self, dataframe):

        # loop over all records in the dataframe and parse out the video ids
        data = {
            'video_id'    : [],
            'video_title' : [],
            'video_desc'  : []
        }

        for index, row in dataframe.iterrows():

            # parse the query string
            video_id    = self.parse_video(row["request_params"])
            title, desc = self.lookup_video(video_id)

            # append it to data
            data['video_id'].append(video_id)
            data['video_title'].append(title)
            data['video_desc'].append(desc)

        # create new dataframe
        df1 = pd.DataFrame(data) 

        # merge columns from df1 into df, not optimal?
        dataframe[ 'video_id' ]    = data['video_id']
        dataframe[ 'video_title' ] = data['video_title']
        dataframe[ 'video_desc' ]  = data['video_desc']

        # and return
        return dataframe

    def parse_video(self, request_params):

        # any error in parsing means video id is empty
        video_id = ""

        try:
            qs  = urllib.parse.parse_qs(request_params)
            ids = qs.get("v", [""])

            video_id = ids[0]
        except Exception as e:
            video_id = ""

        return video_id

    def lookup_video(self, video_id):

        # any error in lookup is handled
        title = ""
        desc  = ""

        try:

            if not video_id:
                raise Exception("Empty video id")
            if not self.api_key:
                raise Exception("YouTube API key is empty")

            url    = 'https://www.googleapis.com/youtube/v3/videos'
            params = dict(part='snippet', id=video_id, key=self.api_key)
            resp   = requests.get(url=url, params=params)
            data   = resp.json()

            if data["kind"] != "youtube#videoListResponse":
                raise Exception("Wrong data kind %s" % data["kind"])

            items = data["items"]
            if len(items) != 1:
                raise Exception("Wrong count of items %d instead of 1" % len(items))

            # get first snippet
            item = items[0]

            # and assign
            title = item["snippet"]["title"]
            desc  = item["snippet"]["description"]

        except Exception as e:

            title = "unable to lookup"
            desc  = str(e)

        # sanitize
        title = title.replace("\n", " ")
        title = title.replace("\r", " ")

        desc = desc.replace("\n", " ")
        desc = desc.replace("\r", " ")

        # and return nicely (we do not need large)
        return (title[:32], desc[:64])



