import os
import shutil
import logging
import datetime

#
#
#
from daemon.utils           import Timer, file_size_human
from daemon.stats.collector import History


#
#
#
from daemon.stats.reporter.builders._impl.specific import DetailedInfoLimits

#
#
#
from .builders.daily import Builder
from .history        import is_valid_report_dir

#
#
#
class DayReporter:

    def __init__(self, config):

        # debug check dirs are fine
        assert(os.path.isdir(config.stats_dir))
        assert(os.path.isdir(config.reports_dir))

        # save
        self.config      = config
        self.stats_dir   = config.stats_dir
        self.reports_dir = config.reports_dir

    def report(self, force):

        logging.info("Building daily reports:")

        # see the history
        history = History(self.stats_dir)

        # loop over each year
        for year in history.years():

            # then over each month in a year
            for month in history.months(year):

                # and finally over each day in a month
                for day in history.days(year, month):

                    # construct the prefix
                    date = "%04d-%02d-%02d" % (year, month, day)

                    # see if admin insists
                    if force:

                        # oh yes just do it
                        self.report_date(date)

                    else:

                        # see if the report is already there
                        if not is_valid_report_dir(os.path.join(self.reports_dir, date)):

                            # no such report, generate then
                            self.report_date(date)

                        else:

                            # well report is there but we always regenerate yesterday report
                            today = datetime.date.today()
                            start = today - datetime.timedelta(days=1)

                            if datetime.date(year, month, day) < start:

                                # log and skip
                                logging.info("%s daily report skipped (already generated)" % date)

                            else:

                                # do regenerate indeed
                                self.report_date(date)

    def report_date(self, date):

        # create timer
        timer = Timer()

        # construct the folders for this generation run
        out_dir = os.path.join(self.reports_dir, date)
        tmp_dir = out_dir + ".temp"

        # clear the temp folder
        if os.path.exists(tmp_dir):
            shutil.rmtree(tmp_dir)

        # create it
        os.mkdir(tmp_dir)

        # construct the input data path
        data_csv = os.path.join(self.stats_dir, date, "data.csv.gz")

        # log it
        logging.info("%s starting daily report (CSV size is %s)..." % (date, file_size_human(data_csv)))

        # create limits for the builder
        limits = DetailedInfoLimits()
        if True:
            limits.users_limit   = self.config.reporter.max_detailed_users
            limits.domains_limit = self.config.reporter.max_detailed_domains
        
        # and build
        r = Builder(tmp_dir)
        r.build(data_csv, limits)

        # remove the output dir just in case
        if os.path.exists(out_dir):
            shutil.rmtree(out_dir)

        # and rename temp
        os.rename(tmp_dir, out_dir)

        # ok completed, dump the log
        logging.info("%s daily report completed." % date)
        logging.info("Statistics:")
        logging.info("  Elapsed time    : %d seconds" % timer.elapsed())
        logging.info("  Processed bytes : %d bytes" % os.path.getsize(data_csv))
        logging.info("  CSV file        : %s" % data_csv)

        
