#!/usr/bin/env python3
#
# (c) Diladele B.V. 2020, Web Safety
#  
import os
import json
import socket
import logging
import argparse
import datetime
import pandas as pd

from daemon.core import DaemonPath, UiPath, SquidPath

#
# helper functions
#
def is_valid_ipv4_address(address):

    try:
        socket.inet_pton(socket.AF_INET, address)
    except AttributeError:  # no inet_pton (like on Windows)
        try:
            socket.inet_aton(address)
        except socket.error:
            return False
        return address.count('.') == 3
    except socket.error:
        return False

    return True

def is_valid_ipv6_address(address):

    try:
        socket.inet_pton(socket.AF_INET6, address)
    except socket.error:
        return False
    return True



#
#
#
def split_df(users):

    names = []
    ips   = []
    for user in users:

        if is_valid_ipv4_address(user) or is_valid_ipv6_address(user):
            ips.append(user)
        else:
            if user == "-":
                continue
            names.append(user)

    # make unique
    return ( list(set(names)), list(set(ips)) )


#
#
#
def run_stats(stats_dir):

    # we look at two days before (not yesterday)
    folder   = datetime.datetime.strftime(datetime.datetime.now() - datetime.timedelta(2), '%Y-%m-%d')
    data_csv = os.path.join(stats_dir, folder, "data.csv.gz")

    # default values
    report_data = {
        "users": 0,
        "ips"  : 0
    }

    # calculate total number of filtered users/ips
    if os.path.isfile(data_csv):

        logging.info("Loading statistics from file at %s" % data_csv)

        # read data
        df = pd.read_csv(data_csv, sep='\t', parse_dates=[1]) 
        if True:

            # we have only filtered records where ws_iid attribute is NOT zero
            df = df.loc[ df["ws_iid"] != 0 ]
            
            # we never count localhost
            df = df.loc[  df["user_id"] != "127.0.0.1" ]

        # now group records by user_id
        users = df.groupby(
            ["user_id"]
        ).agg(
            { "request_url" : "count" }
        ).sort_values("request_url", ascending=False)

        # split into users/ips
        (names, ips) = split_df(users.index.tolist())

        # dump
        logging.info("Detected the following unique user names:")
        for value in names:
            logging.info("\t%s", value)

        logging.info("Detected the following unique IP addresses:")
        for value in ips:
            logging.info("\t%s", value)

        # and save
        report_data["users"] = len(names)
        report_data["ips"]   = len(ips)

    else:

        logging.info("Statistics file was not found, assuming no filtering was done.")

    # save the report
    report_json = os.path.join(stats_dir, "licensing.json")
    with open(report_json, 'w') as fout:
        json.dump(report_data, fout)

    # dump
    logging.info("Successfully written report_data to '%s'." % report_json)


#
# 
#
def run(args):

    logging.info("Running License Reporter with the following parameters:")
    logging.info("\tStats folder: %s" % args.stats)

    run_stats(args.stats)
    
    logging.info("License Reporter finished successfully!")

#
# helper function for default values
#
def get_default_args():

    l = SquidPath.log_dir()
    s = os.path.join(UiPath.var_dir(), "stats")
    r = os.path.join(UiPath.var_dir(), "reports")
    
    return (l, s, r)

#
# entry point
#
def main():

    # define default values
    (default_logs, default_stats, default_reports) = get_default_args()
    
    # define arguments
    parser = argparse.ArgumentParser(
        description='''
            License Reporter. Calculates total number of filtered users/IP addresses 
            based on information stored in the statistics folder. Usually run daily 
            by cron.
        '''
    )

    parser.add_argument("--stats", default=default_stats, help="path to folder with statistics")
    parser.add_argument("--verbose", help="print additional debug information", action="store_true")

    # parse them
    args = parser.parse_args()

    # init logger
    level = logging.INFO
    if args.verbose:
        level = logging.DEBUG

    logging.basicConfig(format='%(asctime)s %(message)s', level=level)   

    # make the process consume less cpu
    try:
        os.nice(19)
    except Exception as e:
        logging.error("Cannot renice process, error %s" % str(e))

    # build the reports
    run(args)

if __name__ == '__main__':

    main()