#!/usr/bin/env python3
#
# (c) Diladele B.V. 2020, Web Safety
#  
import os
import json
import logging
import argparse

#
#
#
from daemon.stats.periodic import Config, Factory
from daemon.core           import DaemonPath, UiPath, SquidPath

#
#
#
def run_report(config, item):

    logging.info("Building report '%s' ..." % item.name)

    # good, create the specific reporter based on type
    r = Factory.create(config.stats_dir, config.reports_dir, item.report_type)

    # build it
    r.report(item)

    # log
    logging.info("Report '%s' built successfully." % item.name)


#
#
#
def run_single(config, report_name):

    # find the report with this name
    for item in config.periodic.reports:

        if item.name == report_name:

            # log it
            logging.info("Report '%s' found in the configuration, starting build..." % report_name)

            # and run
            run_report(config, item)
#
#
#
def run_all(config):

    # loop over all reports in the config and run each
    for item in config.periodic.reports:
        run_report(config, item)

#
#
#
def is_cluster_slave():

    path = os.path.join(DaemonPath.etc_dir(), "node", "cluster_client.json")
    if not os.path.exists(path):
        return False

    # assume not slave
    enabled = False

    # open json and see
    with open(path) as fin:
        data    = json.load(fin)
        enabled = data.get("enabled", False)

    # and return
    return enabled

#
# 
#
def run(args):

    # the slave nodes upload squid logs to the master server, so no point in running
    if is_cluster_slave():

        # dump to log
        logging.info("Running Periodic Report Builder on cluster slave nodes is not needed, periodic reports will only run on cluster master.")

        # and exit
        return

    # ok not slave, good
    logging.info("Running Periodic Report Builder with the following parameters:")

    # create the configuration
    config = Config(args.stats, args.reports)
    config.dump()

    logging.info("\tReport to build : %s" % args.report)

    if args.report == "all":
        run_all(config)
    else:
        run_single(config, args.report)

    logging.info("Reporter finished successfully!")

#
# entry point
#
def main():

    # define arguments
    parser = argparse.ArgumentParser(
        description='''
            Periodic Report Builder. Builds periodic reports as required by the administrator. 
            Usually run at specified time intervals by cron file /etc/cron.d/websafety_periodic
        '''
    )

    parser.add_argument("--stats",   default="", help="path to folder with statistics storage")
    parser.add_argument("--reports", default="", help="path to folder to put generated reports to")
    parser.add_argument("--report",  default="all", help="name of the report to build")
    parser.add_argument("--verbose", help="print additional debug information", action="store_true")

    # parse them
    args = parser.parse_args()

    # init logger
    level = logging.INFO
    if args.verbose:
        level = logging.DEBUG

    logging.basicConfig(format='%(asctime)s %(message)s', level=level)   

    # make the process consume less cpu
    try:
        os.nice(19)
    except Exception as e:
        logging.error("Cannot renice process, error %s" % str(e))

    # build the reports
    run(args)

if __name__ == '__main__':

    main()