#!/usr/bin/env python3
#
# (c) Diladele B.V. 2020, Web Safety
#  
import os
import json
import time
import logging
import datetime
import argparse
import traceback
import dateutil.parser

#
#
#
from daemon.stats.collector import NowCollector, Config
from daemon.stats.reporter  import NowReporter

#
# 
#
def run_now(config):

    # first get the last N records from the access log into reports dir
    c = NowCollector(config)
    c.collect(config.realtime.last_line_count)

    # now generate various CSV and JSON files out of raw data
    r = NowReporter(config)
    r.report()

    # finally save the timestamp of last generation
    with open(os.path.join(config.reports_dir, "now", "generated.json"), mode='w') as fout:
        json.dump( { "generated" : datetime.datetime.now().isoformat() }, fout)

# 
#
#
def when_generated(config):

    # assume generated 365 days ago (to always trigger generation in case of error)
    result = datetime.datetime.now() - datetime.timedelta(days=365)

    # open the json file and try to load actual info from it
    try:

        with open(os.path.join(config.reports_dir, "now", "generated.json")) as fin:
            data = json.load(fin)

        result = dateutil.parser.parse(data["generated"])
    
    except Exception as e:      
        print(str(e))
        pass

    return result


#
#
#
def shall_run(config):

    # get the time of the last generation
    now  = datetime.datetime.now()
    then = when_generated(config)

    # see if N seconds elapsed since the last generation
    elapsed  = (now - then).total_seconds()
    interval = config.realtime.regeneration_interval
    if elapsed > interval:
        return True

    # dump
    logging.debug(
        "Then %s, now %s, elapsed %d seconds of max %d, no need to generate." % (then, now, elapsed, interval)
    )

    return False

#
#
#
def run_if(args):

    # create the config object
    config = Config(args.logs, args.stats, args.reports)
    
    # if time has not come yet, do nothing
    if not shall_run(config):
        return

    # ok we shall run now
    logging.info("Realtime traffic information reporter is starting...")

    # dump the config
    config.dump()

    # and run
    run_now(config)

    # and dump again
    logging.info("Realtime traffic information reporter finished successfully!")

#
#
#
def run_once(args):

    # regenerate if the time has come
    try:

        # may throw
        run_if(args)

    except Exception as e:

        # dump the exception to log and continue
        logging.error("Panic: unexpected exception: %s" % str(e))
        logging.error(str(traceback.format_exc()))


#
#
#
def run_loop(args):

    # as a daemon we loop forever
    while True:

        # 
        run_once(args)

        # sleep for 5 seconds
        time.sleep(5)



#
#
#
def run(args):

    if args.once:
        run_once(args)
    else:
        run_loop(args)

#
# entry point
#
def main():

    # define arguments
    parser = argparse.ArgumentParser(
        description='''
            Realtime Report Builder. Builds reports for UI/Traffic/Realtime out of Squid's access_log file. 
            Usually runs as systemd daemon by /etc/systemd/system/wsmond.service script.
        '''
    )

    parser.add_argument("--logs", default="", help="path to folder with logs (optional)")
    parser.add_argument("--stats", default="", help="path to folder with statistics parsed from logs (optional)")
    parser.add_argument("--reports", default="", help="path to folder to put generated reports to")
    parser.add_argument("--verbose", help="print additional debug information", action="store_true")
    parser.add_argument("--renice", help="try to consume less CPU", action="store_true")
    parser.add_argument("--once", help="run once not as a daemon", action="store_true")

    # parse them
    args = parser.parse_args()

    # init logger
    level = logging.INFO
    if args.verbose:
        level = logging.DEBUG

    logging.basicConfig(format='%(asctime)s %(message)s', level=level)   

    # make the process consume less cpu
    try:
        if args.renice:
            os.nice(19)
    except Exception as e:
        logging.error("Cannot renice process, error %s" % str(e))

    # and run forever
    run(args)

#
#
#
if __name__ == '__main__':

    main()