#!/usr/bin/env python3
#
# (c) Diladele B.V. 2020, Web Safety
#  
import os
import shutil
import logging
import argparse

#
#
#
from daemon.stats.collector import Config
from daemon.stats.reporter  import CronReporter, CronPurger
from daemon.core            import UiPath, SquidPath

#
#
#
def run_cron(config, force):

    # purge the reports older than N days
    purger = CronPurger(config)
    purger.purge()

    # now enumerate the statistics storage and incrementally build reports
    r = CronReporter(config)

    # admin may force complete report rebuilt if needed
    r.report(force)

#
# 
#
def run(args):

    # create config
    logging.info("Report builder script is starting...")

    # create the config object
    config = Config(args.logs, args.stats, args.reports)
    config.dump()

    # we never generate reports if disk space is low
    usage   = shutil.disk_usage(config.stats_dir)
    free_gb = usage.free / 1024 / 1024 / 1024
    if free_gb < 5:

        # dump to log
        logging.info("History reporting requires at least 5GB of free space on disk (you have %dGB), exiting." % free_gb)

        # and exit
        return

    if not config.collector.enable:

        # dump to log
        logging.info("History collection and reporting is disabled, nothing to do.")

    else:

        # run it
        run_cron(config, args.force)

    # and 
    logging.info("Reporter finished successfully!")

#
# entry point
#
def main():

    # define arguments
    parser = argparse.ArgumentParser(
        description='''
            Report Builder. Builds reports out of Squid's access_log files. 
            Note: assumes the records in access log are monotonously increasing 
            from day to day. Usually run daily by cron.
        '''
    )

    parser.add_argument("--logs", default="", help="path to folder with logs (optional)")
    parser.add_argument("--stats", default="", help="path to folder with statistics parsed from logs (optional)")
    parser.add_argument("--reports", default="", help="path to folder to put generated reports to")
    parser.add_argument("--force", required=False, default=False, action='store_true', help="forcefully regenerate all reports")
    parser.add_argument("--verbose", help="print additional debug information", action="store_true")

    # parse them
    args = parser.parse_args()

    # init logger
    level = logging.INFO
    if args.verbose:
        level = logging.DEBUG

    logging.basicConfig(format='%(asctime)s %(message)s', level=level)   

    # make the process consume less cpu
    try:
        os.nice(19)
    except Exception as e:
        logging.error("Cannot renice process, error %s" % str(e))

    # build the reports
    run(args)

if __name__ == '__main__':

    main()