#!/usr/bin/env python3
#
# (c) Diladele B.V. 2019, Web Safety
#  
import os
import json
import shutil
import socket
import logging
import requests
import argparse
import datetime
import pandas as pd

#
#
#
from daemon.core  import DaemonPath, UiPath, SquidPath, Build
from daemon.utils import JsonReader

#
#
#
class Storage:

    def __init__(self, stats_dir):

        # take a look at two days before
        folder   = datetime.datetime.strftime(datetime.datetime.now() - datetime.timedelta(2), '%Y-%m-%d')
        data_csv = os.path.join(stats_dir, folder, "data.csv.gz")

        # load the dataframe
        self.df  = pd.read_csv(data_csv, sep='\t', parse_dates=[1]) 


#
#
#
class RecategorizationGenerator:

    def generate(self, target_dir):

        # dump to log
        logging.info("Running recategorized domains generator...")

        file = os.path.join(DaemonPath.etc_dir(), "safety", "recategorized_domains.json")
        if not os.path.isfile(file):
            return

        # no need to upload if data is empty
        data = JsonReader(file).read_array()
        if len(data) == 0:
            return

        # good, copy then
        shutil.copyfile(file, os.path.join(target_dir, "recategorized_domains.json"))

#
#
#
class BasicGenerator:

    def __init__(self, stats_dir):

        # dump to log
        logging.info("Running basic telemetry generator...")

        # save stats dir
        self.stats_dir = stats_dir

    def generate(self, target_dir):

        # prepare data
        data = self.get_data()

        # and save
        data.to_csv(os.path.join(target_dir, "non_categorized_domains.csv"), encoding='utf-8')

    def get_data(self):

        # load the dataframe
        df = Storage(self.stats_dir).df

        # leave only those records which were filtered
        df = df.loc[ df["ws_iid"] != 0 ]

        # never take localhost into account
        df = df.loc[  df["user_id"] != "127.0.0.1" ]

        # only leave unknown categories
        df = df.loc[ df["ws_categories"] == 0 ]

        # DEBUG print( df[ ["user_id", "domain", "ws_iid", "ws_categories"] ] )

        # now group by access count to find out most visited domains
        view = df.groupby(
            ["domain"]
        ).agg(
            { "request_url" : "count" }
        ).sort_values("request_url", ascending=False)

        # rename columns
        view.rename(columns={"request_url":"hits"}, inplace=True)

        # good, return
        return view


#
#
#
class ExtendedGenerator:

    def generate(self, target_dir):
        pass

#
#
#
class Archiver:

    def __init__(self, archive_dir):

        self.archive_dir = archive_dir

    def archive(self, target_dir):

        name = os.path.join(self.archive_dir, "telemetry")

        shutil.make_archive(
            name, 'zip', target_dir
        )

        return name + ".zip"

#
#
#
class Uploader:

    def upload(self, archive):

        # construct where to upload to
        url = "https://defs.diladele.com/websafety/telemetry/v2/upload/"
        
        # dump to log
        logging.info("Uploading local file '%s' to URL '%s' using POST request..." % (archive, url))

        # and use request library to do the heavy lifting
        files  = { 'file' : open(archive,'rb') }
        values = { 'type' : 'websafety', 'version' : Build.version() }

        # ok
        r = requests.post(url, files=files, data=values)

#
#
#
class Config:

    def __init__(self):

        # read sharing settings and intialize members
        file = os.path.join(DaemonPath.etc_dir(), "node", "sharing.json")
        data = JsonReader(file).read_object()
        if True:
            self.upload_recategorization   = data.get("upload_recategorization", False)
            self.upload_telemetry_basic    = data.get("upload_telemetry_basic", False)
            self.upload_telemetry_extended = data.get("upload_telemetry_extended", False)

        # dump
        logging.info("Read the following upload settings from '%s'" % file)
        logging.info("\tupload_recategorization   = %s" % self.upload_recategorization)
        logging.info("\tupload_telemetry_basic    = %s" % self.upload_telemetry_basic)
        logging.info("\tupload_telemetry_extended = %s" % self.upload_telemetry_extended)

        # for DEBUG
        self.upload_recategorization = True
        self.upload_telemetry_basic  = True
        

#
#
#
class TelemetryUploader:

    def __init__(self):

        # create temporary folder
        self.temp_dir = os.path.join(UiPath.var_dir(), "temp", "telemetry")
        if os.path.isdir(self.temp_dir):
            shutil.rmtree(self.temp_dir)

        os.makedirs(self.temp_dir)

    def upload(self, stats_dir):

        # create config
        config = Config()

        # will store all telemetry here
        target_dir = self.temp_dir

        # first copy all to telemetry folder
        if config.upload_recategorization:

            r = RecategorizationGenerator()
            r.generate(target_dir)

            logging.info("Recategorized domains generator finished successfully")

        if config.upload_telemetry_basic:

            b = BasicGenerator(stats_dir)
            b.generate(target_dir)

            logging.info("Basic telemetry generator finished successfully")

        if config.upload_telemetry_extended:

            e = ExtendedGenerator()
            e.generate(target_dir)

        # pack it now in a separate folder
        archive_dir = os.path.join(UiPath.var_dir(), "temp", "telemetry.archived")
        if os.path.isdir(archive_dir):
            shutil.rmtree(archive_dir)

        os.makedirs(archive_dir)

        # pack indeed
        archive_zip = Archiver(archive_dir).archive(target_dir)

        # and upload
        Uploader().upload(archive_zip)



#
# 
#
def run(args):

    logging.info("Running Telemetry Uploader with the following parameters:")
    logging.info("\tStats folder: %s" % args.stats)

    u = TelemetryUploader()
    u.upload(args.stats)

    logging.info("Telemetry Uploader finished successfully!")

#
# helper function for default values
#
def get_default_args():

    l = SquidPath.log_dir()
    s = os.path.join(UiPath.var_dir(), "stats")
    r = os.path.join(UiPath.var_dir(), "reports")
    
    return (l, s, r)

#
# entry point
#
def main():

    # define default values
    (default_logs, default_stats, default_reports) = get_default_args()
    
    # define arguments
    parser = argparse.ArgumentParser(
        description='''
            Anonymous telemetry collector for Web Safety project. 

                Collects anonymous telemetry data out of /opt/websafety-ui/var/stats files. 
                Usually runs from cron using /etc/cron.daily/websafety_telemetry.
                
                Telemetry is important for us because it allows to make the 
                product better. Telemetry is always used ONLY for product improvements!             
                Please allow us to use the telemetry data by setting the checkbox at
                UI / Dashboard / Tools / Data Choices.

                Thanks!
        '''
    )

    parser.add_argument("--stats", default=default_stats, help="path to folder with statistics")
    parser.add_argument("--verbose", help="print additional debug information", action="store_true")

    # parse them
    args = parser.parse_args()

    # init logger
    level = logging.INFO
    if args.verbose:
        level = logging.DEBUG

    logging.basicConfig(format='%(asctime)s %(message)s', level=level)   

    # make the process consume less cpu
    try:
        os.nice(19)
    except Exception as e:
        logging.error("Cannot renice process, error %s" % str(e))

    # build the reports
    run(args)

if __name__ == '__main__':

    main()
