#!/usr/bin/env python3
import os
import sys
import time
import pytz
import logging
import datetime
import argparse

#
#
#
def perform_action_linux(distrib, timezone):

    # additional step for debian based distributions
    if distrib in ["ubuntu20", "debian10"]:

        # write it into the time zone file
        file = '/etc/timezone'

        logging.debug("Opening the '%s' for writing..." % file)
        with open(file, 'w') as fout:
            fout.write(timezone)
            fout.write('\n')

        logging.debug("Time zone '%s' is written to file '%s' successfully" % (timezone, file))

    # update localtime link
    localtime = '/etc/localtime'
    zoneinfo  = os.path.join('/usr/share/zoneinfo/', timezone)

    if os.path.exists(localtime):
        logging.debug("Removing already existing localtime link '%s'..." % localtime)    
        os.unlink(localtime) 

    logging.debug("Linking time zone info file '%s' to localtime link '%s'..." % (zoneinfo, localtime))
    os.symlink(zoneinfo, localtime)


#
#
#
def perform_action(system_name, distrib, timezone):

    if len(timezone) == 0:
        raise Exception("Time zone cannot be empty!")

    logging.debug("System: %s, distrib: %s, setting time zone to '%s' ... " % (system_name, distrib, timezone))

    # check user tries to set a valid time zone
    if timezone not in list(pytz.all_timezones):
        raise Exception("Provided time zone value '%s' cannot be found in the list of available time zones!" % timezone)

    if system_name == "linux":
        perform_action_linux(distrib, timezone)
    else:
        raise Exception("Provided system name '%s' is not valid!" % system_name)

    # if we got here everything is fine
    logging.debug("Time zone information updated successfully!")


#
# main
#
def main():

    verbose = False
    start   = time.time()
    
    # parse command line args
    parser = argparse.ArgumentParser(description='Time zone manipulation script.')
    parser.add_argument("--verbose", help="print additional debug information", action="store_true")
    parser.add_argument("--timezone", help="sets system time zone to this value", required=True)
    parser.add_argument("--system", help="linux", required=True)
    parser.add_argument("--distrib", help="debian10|ubuntu20", required=True)
    
    args  = parser.parse_args()       
    level = logging.INFO
    if args.verbose:
        level = logging.DEBUG
        
    logging.basicConfig(format='%(asctime)s %(message)s', level=level)
            
    logging.debug("Time zone manipulation script is started at %s" % datetime.datetime.now().strftime("%Y-%m-%d, %H:%M:%S"))    
    perform_action(args.system, args.distrib, args.timezone)    
    logging.debug("Time zone manipulation script is finished SUCCESSFULLY in %d seconds!" % (time.time() - start))

if __name__ == "__main__":

    # call the entry code
    main()

    # if we got here everything is fine
    sys.exit(0)
