import os
import json

#
#
#
from daemon.core  import DaemonPath, UiPath
from daemon.utils import FileWriter, JsonDumper, ElevatedCommand

#
#
#
class BaseObject:

    def load_json(self, path):

        if os.path.exists(path):
            with open(path) as fin:    
                return json.load(fin)

        return {}

    def save_json(self, target_file, data):

        # get base name of the path and construct temp 
        temp_name = os.path.basename(target_file) 
        temp_dir  = os.path.join(UiPath.var_dir(), "temp")

        # write it into a temporary folder        
        w = FileWriter(temp_dir)
        d = JsonDumper()
        w.write(temp_name, d.dumps(data))

        # and publish it
        self.publish( os.path.join(temp_dir, temp_name), target_file)

    def publish(self, source_file, target_file):

        exe  = os.path.join(UiPath.bin_dir(), "fsmgr.py")
        arg1 = "--source-file=%s" % source_file
        arg2 = "--target-file=%s" % target_file
        args = [exe, arg1, arg2]

        (exit_code, stdout, stderr) = ElevatedCommand.run(args)

        # fail if failed
        if exit_code != 0:

            raise Exception(
                "Cannot copy source file from '%s' into '%s', exit code: %d, stdout: %s, stderr: %s" % (
                    source_file, target_file, exit_code, stdout, stderr
                )
            )

#
#
#
class BaseObjectFactory:

    def construct(self):

        # construct empty object
        obj = self.model_class()

        # and load it
        obj.load()

        # return nicely
        return obj

    def construct_from(self, file):

        # construct empty object
        obj = self.model_class()

        # and load it
        obj.load_from(file)

        # return nicely
        return obj



#
# 
#
class Sharing(BaseObject):

    def __init__(self):

        self.upload_recategorization   = False
        self.upload_telemetry_basic    = False
        self.upload_telemetry_extended = False
        self.storage_file              = os.path.join(DaemonPath.etc_dir(), "node", "sharing.json")

    def as_data(self):

        return {
            "upload_recategorization"   : self.upload_recategorization, 
            "upload_telemetry_basic"    : self.upload_telemetry_basic,
            "upload_telemetry_extended" : self.upload_telemetry_extended
        }

    def load(self):
        self.load_from(self.storage_file)

    def load_from(self, file_name):

        data = self.load_json(file_name)

        self.upload_recategorization   = data.get("upload_recategorization", False)
        self.upload_telemetry_basic    = data.get("upload_telemetry_basic", False)
        self.upload_telemetry_extended = data.get("upload_telemetry_extended", False)

    def save(self):

        self.save_json(
            self.storage_file, 
            self.as_data()
        )

#
#
#
class SharingFactory(BaseObjectFactory):

    model_class = Sharing

#
#
#
class ClusterClient(BaseObject):

    def __init__(self):

        self.enabled           = False
        self.server_address    = ""
        self.server_port       = 18999
        self.sync_interval     = 15
        self.sync_logs         = False
        self.log_sync_interval = 60
        self.storage_file      = os.path.join(DaemonPath.etc_dir(), "node", "cluster_client.json")

    def as_data(self):

        return {
            "enabled"           : self.enabled, 
            "server_address"    : self.server_address,
            "server_port"       : self.server_port,
            "sync_interval"     : self.sync_interval,
            "sync_logs"         : self.sync_logs,
            "log_sync_interval" : self.log_sync_interval
        }

    def load(self):
        self.load_from(self.storage_file)

    def load_from(self, file_name):

        data = self.load_json(file_name)

        self.enabled           = data.get("enabled", False)
        self.server_address    = data.get("server_address", "")
        self.server_port       = data.get("server_port", 18999)
        self.sync_interval     = data.get("sync_interval", 15)
        self.sync_logs         = data.get("sync_logs", False)
        self.log_sync_interval = data.get("log_sync_interval", 60)
        
    def save(self):

        self.save_json(
            self.storage_file, 
            self.as_data()
        )

#
#
#
class ClusterClientFactory(BaseObjectFactory):

    model_class = ClusterClient


#
#
#
class ClusterServer(BaseObject):

    def __init__(self):

        self.enabled        = False
        self.listen_address = ""
        self.listen_port    = 18999
        self.storage_file   = os.path.join(DaemonPath.etc_dir(), "node", "cluster_server.json")

    def as_data(self):

        return {
            "enabled"        : self.enabled, 
            "listen_address" : self.listen_address,
            "listen_port"    : self.listen_port
        }

    def load(self):
        self.load_from(self.storage_file)

    def load_from(self, file_name):

        data = self.load_json(file_name)

        self.enabled        = data.get("enabled", False)
        self.listen_address = data.get("listen_address", "")
        self.listen_port    = data.get("listen_port", 18999)
        
    def save(self):

        self.save_json(
            self.storage_file, 
            self.as_data()
        )


#
#
#
class ClusterServerFactory(BaseObjectFactory):

    model_class = ClusterServer


#
#
#
class ClusterSettings(BaseObject):

    SYNC_MODE_HTTP           = 0
    SYNC_MODE_HTTPS_NOVERIFY = 1
    SYNC_MODE_HTTPS_SERVER   = 2
    SYNC_MODE_HTTPS_MUTUAL   = 3
    SYNC_MODE_CHOICES  = (
        (SYNC_MODE_HTTP,            "HTTP"),
        (SYNC_MODE_HTTPS_NOVERIFY,  "HTTPS / No Verification"),
        (SYNC_MODE_HTTPS_SERVER,    "HTTPS"),
        (SYNC_MODE_HTTPS_MUTUAL,    "HTTPS / Mutual Authentication")
    )


    def __init__(self):

        self.sync_mode    = ClusterSettings.SYNC_MODE_HTTPS_MUTUAL
        self.verbose_log  = False
        self.storage_file = os.path.join(DaemonPath.etc_dir(), "node", "cluster_settings.json")

    def as_data(self):

        return {
            "sync_mode"   : self.sync_mode,
            "verbose_log" : self.verbose_log
        }

    def load(self):
        self.load_from(self.storage_file)

    def load_from(self, file_name):

        data = self.load_json(file_name)

        self.sync_mode   = data.get("sync_mode", ClusterSettings.SYNC_MODE_HTTPS_MUTUAL)
        self.verbose_log = data.get("verbose_log", False)
        
    def save(self):

        self.save_json(
            self.storage_file, 
            self.as_data()
        )

#
#
#
class ClusterSettingsFactory(BaseObjectFactory):

    model_class = ClusterSettings