import os

from .nettypes import ActualNetDevice, CONFIG_TYPE_DHCP, CONFIG_TYPE_NONE, CONFIG_TYPE_STATIC
from .netaddr_netplan import Parser as NetPlanParser
from .netaddr_debian import Parser as DebianParser


#
#
#
class CentOsIpParser:

    def fill(self, device):
        raise Exception("CentOsIpParser::fill - not implemented!")

#
#
#
class NetPlanIpParser(object):

    def fill(self, device):

        # assume the file name as default in Ubuntu 20
        filename = "/etc/netplan/00-installer-config.yaml"
        if not os.path.isfile(filename):

            # maybe it is Ubuntu 18
            filename = "/etc/netplan/50-cloud-init.yaml"
            if not os.path.isfile(filename):

                # ok default file is not found, fire for now, in future look at directory contents?
                raise Exception("Cannot find default netplan file in '%s'" % filename)

        # parse the file
        nics = NetPlanParser().parse(filename)

        # and set its properties
        self.configure(nics, device)

    def configure(self, nics, device):

        # see if we have such a device in netplan file
        nic = nics.get(device.name, None)
        if nic is not None:
            self.configure_ipv4(nic, device.ip4)
            self.configure_ipv6(nic, device.ip6)

    def configure_ipv4(self, nic, ipv4):
        
        # assume none
        ipv4.config_type = CONFIG_TYPE_NONE

        # set addressing method
        if nic.dchp4:

            # it is DHCP
            ipv4.config_type = CONFIG_TYPE_DHCP

        else:

            # it is NOT DHCP, see assign all present
            if len(nic.addresses) > 0:

                # always take first address only
                addr, mask = nic.addresses[0]

                # assign
                ipv4.address = addr
                ipv4.netmask = mask

            # set gateway
            if nic.gateway4:
                ipv4.gateway  = nic.gateway4

            # set nameservers
            (search, dnses) = nic.nameservers
            if len(search) > 0:

                # again take only first
                ipv4.dns_search = search[0]

            if len(dnses) > 0:
                ipv4.dns_srv1 = dnses[0]
            if len(dnses) > 1:
                ipv4.dns_srv2 = dnses[1]

            # good, now see if BOTH addresses and gateway were configured
            if len(nic.addresses) > 0 and len(nic.gateway4) > 0:

                # ok it is normal static configuration
                ipv4.config_type = CONFIG_TYPE_STATIC

            else:

                # some info for static is missing, assume none
                ipv4.config_type = CONFIG_TYPE_NONE

    def configure_ipv6(self, nic, ipv6):

        # assume none
        ipv6.config_type = CONFIG_TYPE_NONE

        # TODO: for now parsing from IPv6 settings in /etc/netplan is not implemented
        return

        # set addressing method
        if nic.dhcp6:

            # it is DHCP
            ipv6.config_type = CONFIG_TYPE_DHCP

        else:

            # it is NOT DHCP, see assign all present
            if len(nic.addresses) > 0:

                # always take first address only
                addr, mask = nic.addresses[0]

                # assign
                ipv4.address = addr
                ipv4.netmask = mask

            # set gateway
            if nic.gateway4:
                ipv4.gateway  = nic.gateway4

            # set nameservers
            (search, dnses) = n.nameservers
            if len(search) > 0:

                # again take only first
                ipv4.dns_search = search[0]

            if len(dnses) > 0:
                ipv4.dns_srv1 = dnses[0]
            if len(dnses) > 1:
                ipv4.dns_srv2 = dnses[1]

            # good, now see if BOTH addresses and gateway were configured
            if len(nic.addresses) > 0 and len(nic.gateway4) > 0:

                # ok it is normal static configuration
                ipv4.config_type = CONFIG_TYPE_STATIC

            else:

                # some info for static is missing, assume none
                ipv4.config_type = CONFIG_TYPE_NONE




#
#
#
class DebianIpParser:

    def fill(self, device):

        # parse existing file in /etc/network/interfaces
        path = "/etc/network/interfaces"

        # parse the file
        file = DebianParser().parse(path)

        # and set its properties
        self.configure(file, device)
        
    def configure(self, file, device):

        for iface in file.ifaces:
            if iface.logical_name == device.name:
                if iface.get_addr_family() == "inet":
                    self.configure_ipv4(iface, device.ip4)
                if iface.get_addr_family() == "inet6":
                    self.configure_ipv6(iface, device.ip6)

    def configure_ipv4(self, iface, ipv4):
        
        # assume none
        ipv4.config_type = CONFIG_TYPE_NONE

        # set method
        method = iface.get_config_method()
        if method == "static":
            ipv4.config_type = CONFIG_TYPE_STATIC
        elif method == "dhcp":
            ipv4.config_type = CONFIG_TYPE_DHCP
        elif method == "loopback":
            pass            
        else:
            pass

        ipv4.address     = iface.get_address()
        ipv4.netmask     = iface.get_netmask()
        ipv4.gateway     = iface.get_gateway()
        ipv4.dns_search  = iface.get_dns_search()

        dnses = iface.get_dns_nameservers()
        if len(dnses) > 0:
            ipv4.dns_srv1 = dnses[0]
        if len(dnses) > 1:
            ipv4.dns_srv2 = dnses[1]

    def configure_ipv6(self, iface, ipv6):
        
        # assume none
        ipv6.config_type = CONFIG_TYPE_NONE

        # set method
        method = iface.get_config_method()
        if method == "static":
            ipv6.config_type = CONFIG_TYPE_STATIC
        elif method == "dhcp":
            ipv6.config_type = CONFIG_TYPE_DHCP
        elif method == "loopback":
            pass            
        else:
            pass

        ipv6.address     = iface.get_address()
        ipv6.subnet      = iface.get_netmask()
        ipv6.gateway     = iface.get_gateway()
        ipv6.dns_search  = iface.get_dns_search()

        dnses = iface.get_dns_nameservers()
        if len(dnses) > 0:
            ipv6.dns_srv1 = dnses[0]
        if len(dnses) > 1:
            ipv6.dns_srv2 = dnses[1]


if __name__ == "__main__":

    
    d = ActualNetDevice()
    d.name = "ens160"
    
    # test for debian 9 / ubuntu 16 style parser
    # p = DebianIpParser().fill(d)    
    # test for ubuntu 18 style parser
    p = NetPlanIpParser().fill(d)

    
    print ("Device: %s" % d.name)
    print ("\tMAC: %s" % d.mac)
    print ("\tState: %s" % d.state)

    print ("\tIP4:")
    print ("\t\t%s" % d.ip4.config_type)
    print ("\t\t%s" % d.ip4.address)
    print ("\t\t%s" % d.ip4.netmask)
    print ("\t\t%s" % d.ip4.gateway)
    print ("\t\t%s" % d.ip4.dns_srv1)
    print ("\t\t%s" % d.ip4.dns_srv2)
    print ("\t\t%s" % d.ip4.dns_search)

    print ("\tIP6:")
    print ("\t\t%s" % d.ip6.config_type)
    print ("\t\t%s" % d.ip6.address)
    print ("\t\t%s" % d.ip6.subnet)
    print ("\t\t%s" % d.ip6.gateway)
    print ("\t\t%s" % d.ip6.dns_srv1)
    print ("\t\t%s" % d.ip6.dns_srv2)
    print ("\t\t%s" % d.ip6.dns_search)