#
#
#
import os
import yaml

#
#
#
class YamlReader(object):

    @staticmethod
    def read(file):

        # note we always read one doc from file
        with open(file, "r") as fin:
            docs = yaml.load_all(fin, Loader=yaml.FullLoader)
            for doc in docs:
                return doc

        return {}

#
#
#
class Nic(object):

    def __init__(self):
        
        self.name        = ""         # enp3s0
        self.dhcp4       = True       # true or false
        self.gateway4    = None       # none by default, usually something like 192.168.1.1
        self.nameservers = None       # a tuple with ([], []) - the list of search domains and the list of DNS server IPs
        self.addresses   = []         # empty list by default, usually something like [ 
                                      #     (192.168.1.100, 255.255.255.0)
                                      # ] note, currently we skip any IPv6-like looking address!

#
#
#
class Parser:

    def parse(self, file):

        # read the file
        doc = YamlReader.read(file)

        # get the config
        network = doc['network']
        version = network['version']
        if 2 != version:
            raise Exception("Cannot find version 2 in YAML netplan file '%s'" % file)

        renderer = network.get('renderer', 'networkd')
        if renderer != 'networkd':
            raise Exception("Cannot find 'networkd' renderer in YAML netplan file '%s'" % file)

        # this is the value to return
        nics = {}

        # ok look for ethernets
        ethernets = network.get('ethernets', [])
        for ethernet in ethernets:

            # create
            n = Nic()
            n.name        = ethernet

            # parse ipv4
            n.dchp4       = ethernets[ethernet].get('dhcp4', True)
            n.gateway4    = ethernets[ethernet].get('gateway4', "")
            n.addresses   = self.parse_addresses(ethernets[ethernet].get('addresses'))            
            n.nameservers = self.parse_nameservers(ethernets[ethernet].get('nameservers'))

            # and save in the dict
            nics[n.name] = n

        # and returl all nics detected
        return nics

    def parse_addresses(self, addresses):

        # result
        result = []

        # nothing in nothing out
        if not addresses:
            return result

        # the addresses must be a list though
        assert(type(addresses) == list)

        # loop over each
        for address in addresses:

            # and split into mask and add
            addr, mask = address.strip().lower().split("/")

            # if address contains : then it is IPv6, skip it for now
            if ':' in addr:
                continue

            assert(addr)
            assert(mask)

            result.append(
                (addr, self.parse_mask(mask))
            )
            
        return result

    def parse_nameservers(self, nameservers):

        # this is the return result
        search    = []
        addresses = []

        # nothing in nothing out
        if not nameservers:
            return (search, addresses)

        # parse search
        if True:

            # assign
            search = nameservers.get('search', [])

            # and check type
            assert(type(search) == list)

        # parse DNS addresses
        if True:

            # assign
            addresses = nameservers.get('addresses', [])

            # check type
            assert(type(addresses) == list)

            # check each type
            for address in addresses:
                
                assert(type(address) == str)
                assert(address.find("/") == -1)

        # return both
        return (search, addresses)

    def parse_mask(self, mask):

        assert(int(mask) > 0)
        assert(int(mask) < 32)

        values = {
            
            30 : "255.255.255.252",
            29 : "255.255.255.248",
            28 : "255.255.255.240",
            27 : "255.255.255.224",
            26 : "255.255.255.192",
            25 : "255.255.255.128",
            24 : "255.255.255.0",
            23 : "255.255.254.0",
            22 : "255.255.252.0",
            21 : "255.255.248.0",
            20 : "255.255.240.0",
            19 : "255.255.224.0",
            18 : "255.255.192.0",
            17 : "255.255.128.0",
            16 : "255.255.0.0",
        }

        value = values.get(int(mask), None)
        if value is None:
            raise Exception("Cannot parse mask '%s'" % mask)

        return value

if __name__ == "__main__":

    for file in [
        "test/03_netplan_addlist.yml",
        "test/02_netplan_static.yml",
        "test/01_netplan_dhcp.yml",
        "test/50-cloud-init.yaml"
    ]:

        f = Parser().parse(file)
        print (f)        
    