import base64
from OpenSSL import crypto

#
#
#
class Certificate:

    def __init__(self, pem, is_missing=False):

        # check pem and its type
        assert(pem)
        assert(type(pem) == str)

        # save members
        self.pem  = pem
        self.x509 = crypto.load_certificate(crypto.FILETYPE_PEM, pem)

        # parse out extensions
        self.extensions = self.collect_extensions(self.x509)
        self.subject    = str(self.x509.get_subject())
        self.issuer     = str(self.x509.get_issuer())

        # the flag indicating we have additionally downloaded this certificate
        self.missing    = is_missing

    def is_missing(self):
        return self.missing

    def common_name(self):

        # get value
        v = self.x509.get_subject().CN
        
        # assert it is a string
        assert(type(v) == str)

        # and return it
        return v

    def valid_from(self):

        # get value
        v = self.x509.get_notBefore()

        # assert it is bytes
        assert(isinstance(v, bytes))

        # and return it as string
        return v.decode("utf-8")

    def valid_until(self):

        # get value
        v = self.x509.get_notAfter()

        # assert it is bytes
        assert(isinstance(v, bytes))
        
        # and return it as string
        return v.decode("utf-8")

    def as_data_uri(self):

        prefix  = 'data:text/plain;base64,'    
        payload = base64.b64encode(self.pem.encode())
        
        return prefix + payload.decode("utf-8")

    def hex_sn(self):

        hex_sn = '{0:X}'.format(int(self.x509.get_serial_number()))
        hex_sn = ':'.join(hex_sn[i:i+2] for i in range(0, len(hex_sn), 2))

        return hex_sn

    def is_ca(self):

        # extension key is bytes!
        if b'basicConstraints' not in self.extensions:
            return False

        name  = b'basicConstraints'
        value = self.extensions[name]
        
        if value.find("CA:TRUE") != -1:
            return  True

        return False

    def subjectAltNames(self):

        if b'subjectAltName' not in self.extensions:
            return ""

        name  = b'subjectAltName'
        value = self.extensions[name]

        return str(value)


    def authorityInfoAccess(self):

        if "authorityInfoAccess" not in self.extensions:
            return ("", "")

        name  = "authorityInfoAccess"
        value = self.extensions[name]

        # parse the return values
        ocsp       = ""
        ca_issuers = ""

        # split the value into
        items = value.split("\n")
        for item in items:
            if item.startswith("OCSP"):
                pos = item.find("URI:")
                if pos != -1:
                    temp = item[pos + len("URI:"):]
                    ocsp = temp.strip()

            if item.startswith("CA Issuers"):
                pos = item.find("URI:")
                if pos != -1:
                    temp       = item[pos + len("URI:"):]
                    ca_issuers = temp.strip()

        return (ocsp, ca_issuers)

    def collect_extensions(self, x509):

        extensions = {}

        index = 0
        count = x509.get_extension_count()

        while index < count:

            extension = x509.get_extension(index)
            index    += 1

            # ignore all errors and hope for the best
            try:
                name  = extension.get_short_name()
                if name:
                    extensions[name] = str(extension)
            except Exception as e:                
                pass

        return extensions