import re
import ldap

#
# domain
#
from daemon.core import System
from _domain.utils import Command

#
# ours
#
from .binary_klist import BinaryKlist
from .binary_kinit import BinaryKinit
from .ad_search    import LdapSearch, LdapConnectionFactory

#
# 
#
class KeyTabDumper:

    def __init__(self):

        self.exe = BinaryKlist.full_path()

    def dump(self, keytab_path):

        if System.name() == System.WS_WINDOWS:
            return "Showing KEYTAB %s contents on windows ALWAYS SUCCEEDS with this message." % keytab_path
            
        return self.run_linux(keytab_path)

    def run_linux(self, keytab_path):

        args = [self.exe, "-k", "-K", "-t", "-e", keytab_path]

        (exit_code, stdout, stderr) = Command().run(args) 

        if exit_code != 0:
            raise Exception("Cannot run command %s, error:\n%s" % (" ".join(args), stdout + stderr))

        return stdout + "\n" + stderr
    

#
#
#
class KeyTabInitializer:

    def __init__(self):
        self.exe = BinaryKinit.full_path()

    def initialize(self, keytab_path, spn):
        
        try:
            if System.name() == System.WS_WINDOWS:
                return (True, "Trying actual key tab %s on Windows with SPN %s ALWAYS SUCCEEDS" % (keytab_path, spn))

            return (True, self.run_linux(keytab_path, spn))

        except Exception as e:
            return (False, str(e))


    def run_linux(self, keytab_path, spn):

        # now we should also call the actual kinit to see if it can connect to the domain
        args = [self.exe, "-V", "-k", "-t", keytab_path, spn]

        (exit_code, stdout, stderr) = Command().run(args) 
        if exit_code != 0:
            raise Exception("Cannot run command %s, error:\n%s" % (" ".join(args), stdout + stderr))

        return stdout + "\n" + stderr
    

#
# looks up some attributes in the AD and makes sure these are fine and as expected
#
class KerberosChecker:

    def __init__(self, endpoint):

        self.endpoint = endpoint
        
    def check(self, spn_value, keytab_path):

        # spn stored in the configuration looks like HTTP/proxy.example.lan@EXAMPLE.LAN, we need to get rid of realm part
        spn = spn_value

        # construct it
        if True:
            pos = spn.find("@")
            if pos != -1:
                spn = spn[:pos]

            # debug check
            if not spn:
                raise Exception("Cannot get SPN from config value %s" % spn_value)

        # find all entries in the AD with matching SPN
        entries = self.find_entries(spn)

        # it may be SPN is not mapped at all
        if len(entries) == 0:
            raise Exception("SPN '%s' is NOT mapped to any user in Active Directory" % spn)

        # check if we have only 1
        if len(entries) != 1:
            raise Exception("SPN '%s' is mapped to SEVERAL users in Active Directory. Additional info: %s" % (spn, str(entries)))

        # good debug check
        assert(1 == len(entries))

        # unpack the values (oh LDAP)
        (s, u, v) = entries[0]

        # and return
        s1 = s[0]       # spn
        u1 = u[0]       # user name that this spn mapped to
        v1 = v[0]       # key version number in the AD

        # note these are byte arrays in Python3 we now recode them to simple strings
        s1 = s1.decode("utf-8")
        u1 = u1.decode("utf-8")
        v1 = v1.decode("utf-8")

        # debug check
        assert(s1 == spn)

        # compare kvno
        v2 = self.get_keytab_kvno(keytab_path, spn_value)
        if v1 != v2:

            # all fails
            raise Exception("Kerberos authentication for SPN '%s' (mapped to user '%s') may NOT work correctly. Error: KVNO NOT EQUAL ('%s' in LDAP != '%s' in keytab file '%s')" 
                % (s1, u1, v1, v2, keytab_path)
            )

        # good then
        return (s1, u1, v1, v2)

    def get_keytab_kvno(self, keytab_path, keytab_spn):

        records = []
        dumped  = KeyTabDumper().dump(keytab_path).split('\n')

        for line in dumped:

            # this is for linux            
            smatch = re.match( r'^\s*(\d+)\s(\d\d[\/|-]\d\d[\/|-]\d\d\d?\d?)\s+(\d\d:\d\d:\d\d)\s+(HTTP\/\S+@\S+)\s+.*$', line, re.M|re.I)
            if smatch:
                kvno = smatch.group(1)
                date = smatch.group(2)
                time = smatch.group(3)
                spn  = smatch.group(4)

                if spn == keytab_spn:
                    return kvno

            # this is for freebsd
            smatch = re.match( r'^\s*(\d+)\s*arcfour-hmac-md5\s+(HTTP\/\S+@\S+)\s+(\d\d\d\d-\d\d-\d\d)\s*$', line, re.M|re.I)
            if smatch:

                kvno = smatch.group(1)
                date = smatch.group(3)
                time = "00:00:00"
                spn  = smatch.group(2)

                if spn == keytab_spn:
                    return kvno

        return ""

    def find_entries(self, value_spn):

        # this is the list to return
        found = []

        # create the connection
        conn = LdapConnectionFactory.connect(self.endpoint)

        # construct search filter and name of the attributes
        filter = "(servicePrincipalName=%s)" % value_spn
        attrs  = [ "objectClass", "distinguishedName", "servicePrincipalName", "samAccountName", "msDS-KeyVersionNumber"]

        # and run the search
        msgid = conn.search_ext(self.endpoint.creds.base_dn, ldap.SCOPE_SUBTREE, filter, attrs, sizelimit=150)

        while True:

            (res_type, res_data) = conn.result(msgid, all=0)

            # have we reached the end
            if len(res_data) == 0:
                break;

            if res_type == ldap.RES_SEARCH_ENTRY:

                # as we are passing all=0, res_data contains exactly one element
                (dn, values) = res_data[0]

                # add the found entry
                found.append( (dn, values) )

        # good now we have the list of found SPNs in the found array
        if not found:
            raise Exception("Cannot find any object with servicePrincipalName equal to '%s' in Active Directory." % value_spn)

        # build human readable list of user names for the admin
        entries = []

        for (dn, attrs) in found:
            for attr in attrs:
                if attr == "servicePrincipalName":
                    entries.append(
                        (attrs["servicePrincipalName"], attrs["sAMAccountName"], attrs["msDS-KeyVersionNumber"])
                    )

        # and return 
        return entries