#
# system
#
import os
import json
import errno
import shutil
import tempfile

#
# utils
#
from daemon.core import DaemonPath, UiPath
from _domain.utils import Command, CommandElevated

#
#
#
from .binary_certmgr import BinaryCertMgr

#
#
#
class SquidCertDumper:

    def __init__(self):

        self.exe = BinaryCertMgr.full_path()

    def as_json(self, pem):

        arg1 = "--action=verify-root-certificate"
        arg2 = "--input=" + pem
        args = [self.exe, arg1, arg2]

        # get the certificate info        
        (exit_code, stdout, stderr) = Command().run(args)
        if exit_code == 0:
            return json.loads(stdout)

        # if we got here everything is bad
        raise Exception(
            "Command %s failed.\n\tExit Code: %d\n\tSTDOUT: %s\n\tSTDERR: %s" % (" ".join(args), exit_code, stdout, stderr)
        )

#
#
#
class SquidCertConverter:

    def __init__(self):

        self.exe = BinaryCertMgr.full_path()

    def to_der(self, pem, der):

        # run the certificate convert tool
        arg1 = "--action=convert-root-certificate"
        arg2 = "--input=" + pem
        arg3 = "--output=" + der
        args = [self.exe, arg1, arg2, arg3]

        (exit_code, stdout, stderr) = Command().run(args)
        if exit_code == 0:
            return

        # if we got here everything is bad
        raise Exception(
            "Command %s failed.\n\tExit Code: %d\n\tSTDOUT: %s\n\tSTDERR: %s" % (" ".join(args), exit_code, stdout, stderr)
        )

#
# 
#
class SquidCertDbInitializer:

    def __init__(self):

        self.exe = BinaryCertMgr.full_path()
    
    def initialize(self):   

        arg  = "--action=regenerate-certificate-storage"
        args = [self.exe, arg]

        (exit_code, stdout, stderr) = CommandElevated().run(args)
        if exit_code == 0:
            return

        # if we got here everything is bad
        raise Exception(
            "Command %s failed.\n\tExit Code: %d\n\tSTDOUT: %s\n\tSTDERR: %s" % (" ".join(args), exit_code, stdout, stderr)
        )

#
# 
#
class SquidCertGenerator:

    def __init__(self):

        self.exe = BinaryCertMgr.full_path()

    def generate(self, path_to_cert_pem, country, state, city, organization, ou, email, cn, days):   

        # create arguments array
        data = {
            'country'             : country,
            'province'            : state,
            'city'                : city,
            'organization'        : organization,
            'organizational-unit' : ou,
            'common-name'         : cn,
            'e-mail'              : email,
            'lifetime'            : days,
            'output'              : path_to_cert_pem
        }
        args = [self.exe, "--action=create-root-certificate"]
        for key in data.keys():
            args.append("--%s=%s" % (key, str(data[key])))

        (exit_code, stdout, stderr) = Command().run(args)
        if exit_code == 0:
            return

        # if we got here everything is bad
        raise Exception(
            "Command %s failed.\n\tExit Code: %d\n\tSTDOUT: %s\n\tSTDERR: %s" % (" ".join(args), exit_code, stdout, stderr)
        )

#
#
#
class SquidCert:

    def __init__(self):

        self.pem_path = os.path.join(DaemonPath.etc_dir(), "myca.pem")
        self.der_path = os.path.join(DaemonPath.etc_dir(), "myca.der")

    def get(self):

        return SquidCertDumper().as_json(self.pem_path)

    def upload(self, data):

        # first write the file into the temporary folder
        temp_dir = ""
        if True:

            with tempfile.NamedTemporaryFile(prefix="myca_upload_", suffix=".folder", dir=os.path.join(UiPath.var_dir(), "temp"), delete=True) as temp:
                temp_dir = temp.name

        # make it on disk
        if not os.path.isdir(temp_dir):
            os.mkdir(temp_dir)

        # make names
        new_pem = os.path.join(temp_dir, "myca.pem")
        new_der = os.path.join(temp_dir, "myca.der")

        # write the new pem file 
        with open(new_pem, 'wb') as fout:
            for chunk in data.chunks():
                fout.write(chunk)

        # dump the new pem and thus verify it
        cert = SquidCertDumper().as_json(new_pem)

        # see if this certificate IS root ca
        if not cert['IsCA']:
            raise Exception("The file '%s' does not contain a Root CA certificate, Squid cannot used such PEM file for SSL decryption" % new_pem)

        # ok file is good; now we need to convert it to DER
        SquidCertConverter().to_der(new_pem, new_der)
        
        # then move into place using elevation
        self.publish(new_pem, new_der)

        # reinitialize the SSL storage (ensure you have the right to write into squid folder!)
        SquidCertDbInitializer().initialize()

        # and finally drop the temp dir
        shutil.rmtree(temp_dir)

    def generate(self, country, state, city, organization, ou, email, cn, days):

        # will first generate in the temporary folder
        temp_dir = ""
        if True:

            with tempfile.NamedTemporaryFile(prefix="myca_generate_", suffix=".folder", dir=os.path.join(UiPath.var_dir(), "temp"), delete=True) as temp:
                temp_dir = temp.name

        # make it on disk
        if not os.path.isdir(temp_dir):
            os.mkdir(temp_dir)

        # make names
        new_pem = os.path.join(temp_dir, "myca.pem")
        new_der = os.path.join(temp_dir, "myca.der")

        # generate the new pem file by calling special command
        SquidCertGenerator().generate(new_pem, country, state, city, organization, ou, email, cn, days)
        
        # ok file is good; now we need to convert it to DER
        SquidCertConverter().to_der(new_pem, new_der)

        # replace the new files 
        self.publish(new_pem, new_der)

        # reinitialize the SSL storage (ensure you have the right to write into squid folder!)
        SquidCertDbInitializer().initialize()

        # and finally drop the temp dir
        shutil.rmtree(temp_dir)
    
    def publish(self, new_pem, new_der):

        self.publish_file(new_pem, self.pem_path)
        self.publish_file(new_der, self.der_path)

    def publish_file(self, source_file, target_file):

        # publish means move into /opt/websafety/etc using elevation
        exe  = os.path.join(UiPath.bin_dir(), "fsmgr.py")
        arg1 = "--source-file=%s" % source_file
        arg2 = "--target-file=%s" % target_file
        args = [exe, arg1, arg2]

        (exit_code, stdout, stderr) = CommandElevated().run(args)

        # fail if failed
        if exit_code != 0:

            raise Exception(
                "Cannot copy file '%s' into '%s', exit code %d, stdout %s, stderr %s" % (
                    source_file, target_file, exit_code, stdout, stderr
                )
            )
