import json
import urllib
import requests
import dateutil

#
#
#
from daemon.core import System

#
#
#
class Endpoint(object):

    def __init__(self, server="127.0.0.1", port=18888):

        self.server   = server
        self.port     = port
        
    def as_url(self):        
        return "http://%s:%d/monitoring/analytics/icap/getlastitems" % (self.server, self.port)

#
#
#
class OrderBy(object):

    def __init__(self, column_name="timestamp", ascending=False):

        self.column_name = column_name
        self.ascending   = ascending

#
#
#
class Options(object):

    def __init__(self, limit=10000, order=OrderBy(), where=""):

        self.limit = limit
        self.order = order
        self.where = where

    def as_params(self):
        return urllib.parse.urlencode(self.as_dict())

    def as_dict(self):
        args = {
            "limit" : self.limit
        }

        if self.where:

            # please note in UI where supports * and in wsmgrd it must be % (sql like)            
            original = self.where
            original = original.replace("*", "%")
            original = original.replace("?", "_")

            # and assign
            args["where"] = original

        if self.order.column_name:

            args["order_by"] = self.order.column_name
            if self.order.ascending:
                args["asc"] = "ASC"
            else:
                args["asc"] = "DESC"

        return args


#
# this hook is used to parse the ISO serialized date in the JSON into python's date time object
#

def timestamp_hook(json_dict):

    for (key, value) in json_dict.items():

        if key == "timestamp":
            json_dict["timestamp"] = dateutil.parser.parse(value, ignoretz=True)

    return json_dict



#
# downloads the raw JSON array of events from wsmgrd through the configured options
#
class Provider(object):

    def __init__(self, endpoint=Endpoint()):

        self.endpoint=endpoint

    def retrieve(self, options=Options()):

        # these are the entries to return
        entries = None

        # assume debug
        if System.name() == System.WS_WINDOWS:

            entries = self._debug_entries(options)

        else:
            
            # wrap into except handler to provide better readable exception
            try:
                entries = self._entries(options)            
            except Exception as e:
                raise Exception(
                    "Cannot retrieve Real Time data from wsmgrd. Check if it is running and functional. " +
                    "More information may be available in the wsmrgd log. Error: %s" % str(e)
                )

        # good, now adjust the entries by constructing urls
        for entry in entries:

            # url
            if entry["method"] == "CONNECT":
                entry["url"] = "https://%s" % entry["host"]
            else:
                entry["url"] = "%s://%s%s?%s" % (entry["scheme"], entry["host"], entry["path"], entry["query"])

            # categories as array of strings
            # item["categories"] = ", ".join(BuildInCategories.unmap_name(item["categoryflags"]))

            # scan flags
            # item["scanflags_as_string"] = ScanFlagsParser.unmap(item["scanflags"])

        # and return
        return entries

    def _debug_entries(self, options):

        records = []
        with open("m:/websafety/tools/test-wsmgr/input.json", "r") as fin:
            records = json.load(fin, object_hook=timestamp_hook)

        # debug entries support only limiting option
        if len(records) > options.limit:
            records = records[:options.limit]

        # imitate where only when user has typed something as a filter
        where = options.where
        
        if where:

            # filtered records
            filtered = []

            # construct the attributes to look for
            attributes = ["iid", "host", "user_name", "user_ip", "policy", "module", "verdict"]

            # now loop over records and include only those that match
            for record in records:

                for attribute in attributes:

                    v = record.get(attribute, '')
                    if not v:
                        continue

                    if str(v).find(where) != -1:
                        filtered.append(record)

            # and reset the records
            records = filtered

        # TODO: support options.order

        # and return nicely
        return records

    def _entries(self, options):

        # response does not support with, hence this awkward finally construction
        resp = None
        try:

            # download
            url  = "%s?%s" % (self.endpoint.as_url(), options.as_params())            
            resp = requests.get(url)
            data = resp.content.decode('utf-8')

            # convert to dict
            return json.loads(data, object_hook=timestamp_hook)

        finally:
            if resp is not None:
                resp.close()
        
        