import os

#
#
#
from daemon.core  import DaemonPath, UiPath, SystemLogDir
from daemon.utils import Tailer

#
# base class for system and application logs
#
class BaseLogFile:

    def __init__(self, log_dir):

        # save the path to log directory
        self.dir = log_dir

        # generate the actual path to log file
        self.log = os.path.join(self.dir, self.log_name)

    def get_path(self):

        return self.log

    def get_size(self):

        # assume zero
        size = 0

        # ask the system (remain zero in case of any failure)        
        if os.path.exists(self.log):
            try:
                size = os.path.getsize(self.log)
            except Exception as e:
                print(str(e))

        # and return nicely
        return size

    def get_contents(self, line_count, include, exclude):

        try:

            # get contents as string
            contents = self.get_contents_impl(line_count, include, exclude)

            # hack: in a log was written using rsyslog then the tab was converted into #011 we re-convert it back now
            if self.shall_retab():
                contents = contents.replace("#011", "\t")

            # and return normally
            return contents

        except Exception as e:
            return "Cannot read log file %s, error: %s" % (self.log, str(e))

    def get_contents_impl(self, line_count, include, exclude):

        # include and exclude are just strings
        assert(isinstance(include, str))
        assert(isinstance(exclude, str))

        # get n lines from the system
        tailer = Tailer(self.log)
        lines  = tailer.lines(line_count)

        # got output, do exclusions
        if len(exclude) > 0:

            # append only those which do NOT have exclude string
            items = []
            for line in lines:
                if line.find(exclude) == -1:
                    items.append(line)

            # and replace
            lines = items

        # do inclusions
        if len(include) > 0:
            
            items = []
            for line in lines:
                if line.find(include) != -1:
                    items.append(line)

            # and replace
            lines = items

        # good finally make a big string
        stdout = '\n'.join(lines)

        # and return
        return stdout

    def shall_retab(self):
        return False


#
# base class for all system logs we have (wsicapd, wsgsbd, wsytgd etc)
#
class VarLogFile(BaseLogFile):

    def __init__(self):

        # call the base class
        super(VarLogFile, self).__init__(
            SystemLogDir.path()
        )

    def shall_retab(self):

        # the log files in /var/log/ws*.log are written using rsyslog redirection with systemd
        # for some reasons the rsyslog daemon replaces all \t into #011, we need to revert it 
        # back - note this overrides the default implementation in the BaseLogFile class
        return True

    
#
# base class for all daemon logs we have (update.log, etc)
#
class DaemonLogFile(BaseLogFile):

    def __init__(self):

        # call the base class
        super(DaemonLogFile, self).__init__(
            os.path.join(DaemonPath.var_dir(), "log")
        )


#
# base class for all ui logs we have (cron_report.log, etc)
#
class UiLogFile(BaseLogFile):

    def __init__(self):

        # call the base class
        super(UiLogFile, self).__init__(
            os.path.join(UiPath.var_dir(), "log")
        )

