import os
import time
import shutil
import tempfile

#
#
#
from django.db        import connection, transaction

#
#
#
from daemon.core  import UiPath, DaemonPath
from daemon.utils import remake_folder, ElevatedCommand

#
#
#
from frame.models import WebSafetyCore, WebSafetyUi

#
# generates the JSON files for websafety-core in a temporary folder
#
class CoreGenerator:

    @staticmethod
    def generate():

        # we will generate all JSON files in a temporary folder
        temp_dir = ""
        if True:

            with tempfile.NamedTemporaryFile(prefix="opt_websafety_etc_", suffix=".folder", dir=os.path.join(UiPath.var_dir(), "temp"), delete=True) as temp:
                temp_dir = temp.name

        # make it on disk
        remake_folder(temp_dir)

        # update root websafety core object in the database marking the time of generation
        core           = WebSafetyCore.objects.first()
        core.version   = core.version + 1
        core.timestamp = int(round(time.time() * 1000))
        core.save()

        # generate it into temp folder
        core.generate(temp_dir)

        # also dump the copy of the database to /opt/websafety/var/db
        core.dump_db()

        # and return the folder with generated data
        return temp_dir

#
# prepares ZIP that we will distribute to our sync cluster nodes when/if then ask
#
class CoreArchiver:

    @staticmethod
    def archive(folder):

        # copy several files from current /opt/websafety/etc, needed for cluster sync!
        for item in [
            "version.json", 
            "blocked_image.gif", 
            "blocked_page.html", 
            "license.pem", 
            "bypass_partial.html", 
            "blocked_safe_browsing.html",
            "krb5.keytab",
            "krb5.conf"
        ] :
            source = os.path.join(DaemonPath.etc_dir(), item)
            target = os.path.join(folder, item)

            if os.path.isfile(source):
                shutil.copyfile(source, target)

        # copy the /opt/websafety-ui/var/db/config.sqlite so that all cluster nodes have the same UI database
        # note, this database is not part of websafety-core but it is good to have it on the clusters I guess?
        if True:

            source = os.path.join(UiPath.var_dir(), "db", "config.sqlite")
            target = os.path.join(folder, "config.sqlite")
        
            try: 
                c = connection.cursor()
                c.execute("BEGIN IMMEDIATE")
                shutil.copyfile(source, target)
            finally:
                c.close()
                connection.rollback()

        # generate the name of the archive based on the current websafety-core object
        arc_name = "%s-%s" % (
            str(WebSafetyCore.objects.first().timestamp), 
            str(WebSafetyCore.objects.first().version)
        )
        arc_path = os.path.join(UiPath.var_dir(), "temp", arc_name)

        # pack it
        shutil.make_archive(arc_path, 'zip', folder)

        # and return
        return arc_path + ".zip"

#
# publishes the JSON files for websafety-core from temporary folder into actual /opt/websafety/etc
#
class CorePublisher:

    @staticmethod
    def publish():

        # generate into temporary folder
        temp_dir = CoreGenerator.generate()

        # call the sudo command and copy the generated data from this temp folder into actual daemon folder
        if True:

            exe  = os.path.join(UiPath.bin_dir(), "confmgr.py")
            arg1 = "--config=%s"  % temp_dir
            args = [exe, arg1]

            (exit_code, stdout, stderr) = ElevatedCommand.run(args)

            # fail if failed
            if exit_code != 0:

                raise Exception(
                    "Cannot copy generated configuration from folder '%s', exit code %d, stdout %s, stderr %s" % (
                        temp_dir, exit_code, stdout, stderr
                    )
                )

        # now pack the temp_dir into a zip for cluster sync
        archive_zip = CoreArchiver.archive(temp_dir)

        # call the sudo command and copy the archive into /opt/websafety/var/cluster daemon folder
        if True:

            source_file = archive_zip
            target_file = os.path.join(DaemonPath.var_dir(), "cluster", os.path.basename(archive_zip))

            exe  = os.path.join(UiPath.bin_dir(), "fsmgr.py")
            arg1 = "--source-file=%s" % source_file
            arg2 = "--target-file=%s" % target_file
            args = [exe, arg1, arg2]

            (exit_code, stdout, stderr) = ElevatedCommand.run(args)

            # fail if failed
            if exit_code != 0:

                raise Exception(
                    "Cannot copy packed configuration file from '%s' into '%s', exit code: %d, stdout: %s, stderr: %s" % (
                            source_file, target_file, exit_code, stdout, stderr
                        )
                    )

            # do not forget to remove
            os.unlink(source_file)

        # cool, erase the temp folder as it is not needed any more
        shutil.rmtree(temp_dir)


#
# generates JSON files for websafety-ui directly into /opt/websafety-ui/etc
#
class UiGenerator:

    @staticmethod
    def generate():

        # update root websafety ui object in the database marking the time of generation
        ui           = WebSafetyUi.objects.first()
        ui.version   = ui.version + 1
        ui.timestamp = int(round(time.time() * 1000))
        ui.save()

        # and generate it
        ui.generate(UiPath.etc_dir())
        
#
# publishes the JSON files for websafety-ui into /opt/websafety-ui/etc (used by reporter scripts)
#
class UiPublisher:

    @staticmethod
    def publish():

        # just call the generator
        UiGenerator.generate()
