import os
import shutil

from django.template import loader, Context

#
# business logic
#
from  daemon.core import UiPath, System, Distrib
from _domain.node import NetDeviceEnumerator, DebianNetworkSettingsGenerator, EtcResolvConfGenerator, NetPlanNetworkSettingsGenerator

#
#
#
from node.models import NetDevice, NetAdvanced

#
# helper object for /etc/network/interfaces style generation
#
class NetworkGeneratorInterfaces:

    def __init__(self):

        # verify it is linux
        assert(System.name() in [System.WS_LINUX, System.WS_WINDOWS])

        # and debian 10
        assert(Distrib.name() in [Distrib.WS_DEBIAN11, Distrib.WS_WINDOWSX])

    def generate(self, devices, advanced):

        # always generate manual settings
        self.generate_manual(advanced.manual_text)

        # note in debian 10 we save DNS servers into /etc/resolv.conf
        # but in Ubuntu 16 we are saving these as dns-* options into the
        # /etc/network/interfaces file itself, so we need to know exactly
        # if this is debian or not

        # and raise the flag
        is_resolvconf_installed = self.is_resolvconf_installed()

        # generate all interfaces
        contents = ""
        for device in devices:

            t = loader.get_template("node/network/etc_network_interface")
            c = { "device": device, "is_resolvconf_installed" : is_resolvconf_installed }

            contents += t.render(c)

        # generate final file
        t = loader.get_template("node/network/etc_network_interfaces")
        c = { "contents" : contents, "manual" : advanced.manual_text }

        # this is what needs to be written to /etc/network/interfaces
        data = t.render(c)

        # and generate it
        DebianNetworkSettingsGenerator().generate(data)

        # now if resolv conf is not installed, we also need to write to /etc/resolv.conf file
        if not is_resolvconf_installed:

            nameservers = []
            search      = []

            for device in devices:

                nameservers.extend(self.nameserver_addresses(device))
                search.extend(self.nameserver_search(device))

            # make unique
            nameservers = list(set(nameservers))
            search      = list(set(search))

            # render
            t = loader.get_template("node/network/etc_resolv_conf")
            c = {"nameservers" : nameservers, "search" : search}

            # and write
            EtcResolvConfGenerator().generate(t.render(c))

    def nameserver_search(self, device):

        # we glue together searches
        data = [
            device.ip4_dns_search.strip(),
            device.ip6_dns_search.strip()
        ]

        # throw away the empty strings
        data = list(filter(None, data))

        # make unique
        data = list(set(data))

        # and return
        return data

    def nameserver_addresses(self, device):

        # glue together
        data = [
            device.ip4_dns_srv1.strip(),
            device.ip4_dns_srv2.strip(),
            device.ip6_dns_srv1.strip(),
            device.ip6_dns_srv2.strip()
        ]

        # throw away the empty strings
        data = list(filter(None, data))

        # make unique
        data = list(set(data))

        # and return
        return data

    def is_resolvconf_installed(self):

        # ideally we need to look into dpkg --list output for now let it be it
        if not os.path.isfile("/etc/os-release"):
            return False

        with open("/etc/os-release", "r") as fin:
            data = fin.read() 
            if data.find("ubuntu") == -1:
                return False

        return True

    def generate_manual(self, manual):

        # construct file names
        cur_file = os.path.join(UiPath.etc_dir(), "node", "etc_network_interfaces.manual")
        bak_file = cur_file + ".bak"
        new_file = cur_file + ".new"

        # clear the environment
        if os.path.exists(bak_file):
            os.unlink(bak_file)
        if os.path.exists(new_file):
            os.unlink(new_file)

        # always generate manual file (even if manual is empty)
        t = loader.get_template("node/network/etc_network_interfaces.manual")
        m = manual.replace("\r\n", "\n")
        c = { "manual": m }

        # the contents
        contents = t.render(c)

        # write data into new file
        with open(new_file, "w") as fout:
            fout.write(contents)
            fout.flush()

        # move files around
        if os.path.exists(cur_file):
            shutil.move(cur_file, bak_file)
        shutil.move(new_file, cur_file)

#
# helper object for /etc/netplan/ style generation
#
class NetworkGeneratorNetplan:

    def __init__(self):

        # verify it is linux
        assert(System.name() in [System.WS_LINUX, System.WS_WINDOWS])

        # and ubuntu 20
        assert(Distrib.name() in [Distrib.WS_UBUNTU20, Distrib.WS_WINDOWSX])

    def generate(self, devices, advanced):

        # bail out directly if admin typed anything into manual field
        if advanced.manual_text:

            # not supported yet
            raise Exception("Additional manual network settings are not supported when using Netplan")

        # fill the yaml for netplan
        data = {
            "network" : {
                "version"   : 2,
                "renderer"  : "networkd",
                "ethernets" : self.ethernets(devices)
            }
        }

        # and generate the actual netplan file 
        NetPlanNetworkSettingsGenerator().generate(data)

    def ethernets(self, devices):

        # resulting dict
        data = {}

        # loop over all devices
        for device in devices:

            # and save device settings
            data[device.name] = self.ethernet(device)

        # return nicely
        return data

    def nameserver_search(self, device):

        # we glue together searches
        data = [
            device.ip4_dns_search.strip(),
            device.ip6_dns_search.strip()
        ]

        # throw away the empty strings
        data = list(filter(None, data))

        # make unique
        data = list(set(data))

        # and return
        return data

    def nameserver_addresses(self, device):

        # glue together
        data = [
            device.ip4_dns_srv1.strip(),
            device.ip4_dns_srv2.strip(),
            device.ip6_dns_srv1.strip(),
            device.ip6_dns_srv2.strip()
        ]

        # throw away the empty strings
        data = list(filter(None, data))

        # make unique
        data = list(set(data))

        # and return
        return data

    def nameservers(self, device):

        return {
            "search"    : self.nameserver_search(device),
            "addresses" : self.nameserver_addresses(device)
        }

    def configure_ipv4(self, data, device):

        # always pop first
        data.pop("dhcp4", None)
        data.pop("gateway4", None)

        # see what we need to do
        if device.ip4_config == NetDevice.CONFIG_TYPE_NONE:
            pass
        
        elif device.ip4_config == NetDevice.CONFIG_TYPE_DHCP:
            data["dhcp4"] = True

        else:

            # set gateway
            data["gateway4"] = device.ip4_gateway

            # add addresses if needed
            if 'addresses' not in data:
                data['addresses'] = []

            # extend addresses
            data["addresses"].extend(
                [ "%s/%d" % (device.ip4_address, device.ip4_netmask_as_int()) ]
            )

        # and return nicely
        return data

    def configure_ipv6(self, data, device):

        # always pop first
        data.pop("dhcp6", None)
        data.pop("gateway6", None)

        # see what we need to do
        if device.ip6_config == NetDevice.CONFIG_TYPE_NONE:
            pass
        
        elif device.ip6_config == NetDevice.CONFIG_TYPE_DHCP:
            data["dhcp6"] = True

        else:

            # set gateway
            data["gateway6"] = device.ip6_gateway

            # add addresses if needed
            if 'addresses' not in data:
                data['addresses'] = []

            # extend addresses
            data["addresses"].extend(
                [ "%s/%d" % (device.ip6_address, device.ip6_subnet) ]
            )

        # and return nicely
        return data

    def ethernet(self, device):

        # empty data
        data = {}

        # configure ipv4 and ipv6
        data = self.configure_ipv4(data, device)
        data = self.configure_ipv6(data, device)

        # we configure nameservers ONLY if IPv4 or IPv6 are set to static
        if device.ip4_config == NetDevice.CONFIG_TYPE_STATIC or device.ip6_config == NetDevice.CONFIG_TYPE_STATIC:

            # set
            data["nameservers"] = self.nameservers(device)

        # and return nicely
        return data

#
# this is actually a business logic class, it is responsible for network settings generator
#
class NetworkGenerator:

    def generate(self, devices, advanced):

        # debug check
        assert(type(advanced) == NetAdvanced)

        # see what type of generation is in use
        if advanced.network_type == NetAdvanced.NETWORK_TYPE_NONE:

            # no need to generate anything
            pass

        elif advanced.network_type == NetAdvanced.NETWORK_TYPE_INTERFACES:

            # generate
            NetworkGeneratorInterfaces().generate(devices, advanced)

        elif advanced.network_type == NetAdvanced.NETWORK_TYPE_NETPLAN:

            # generate
            NetworkGeneratorNetplan().generate(devices, advanced)

        else:

            # unexpected condition
            raise Exception("Network settings generator is not implemented for system: '%s', distrib '%s'" % (System.name(), Distrib.name()))


#
#
#
class NetworkDeviceSyncer:

    def reread_from_system(self, force_sync=False):

        # get all devices from the system
        actual = NetDeviceEnumerator().enumerate()

        # remove stale devices from the database
        self.remove_stale_devices(actual)

        # now add or update each actual device
        for key in actual.keys():
            self.add_or_update(actual[key], force_sync)

        # ok now we have synced our database with actual devices from the system

    def remove_stale_devices(self, actual):
        
        # calcuate those to remove
        to_remove = []
        for device in NetDevice.objects.all():
            if device.name not in actual.keys():
                to_remove.append(device)

        # remove all
        for device in to_remove:
            device.delete()

    def add_or_update(self, actual, force_sync):

        device  = None
        devices = NetDevice.objects.filter(name=actual.name)

        # assume we do not need to update the IP address read from system
        update_addr = False
        
        if not devices:
            
            # ok we do not have such a device, add one
            device = NetDevice()
            device.name = actual.name
            device.save()

            # and raise the update address flag
            update_addr = True

        else:

            # we do have a device, get it
            device = devices[0]

            # raise the update the flag if user 'forces' us to read from system
            if force_sync:
                update_addr = True

        # update its settings
        device.state = actual.state
        device.mac   = actual.mac

        # update IP addresses ONLY if we need to; we need to update the IP addresses of the device when this device is
        # new for our database - so we read its settings from system; or when user specifically told us to force 
        # reading from the system - this can happen if he manually changed the system settings from root console for example
        # and need the system to throw away settings in the database and replace them with the settings from the system
        if update_addr:
            
            self.update_ip4(device, actual.ip4)
            self.update_ip6(device, actual.ip6)

        # save
        device.save()


    def update_ip4(self, device, addr):

        device.ip4_config     = addr.config_type
        device.ip4_address    = addr.address
        device.ip4_netmask    = addr.netmask
        device.ip4_gateway    = addr.gateway
        device.ip4_dns_srv1   = addr.dns_srv1
        device.ip4_dns_srv2   = addr.dns_srv2
        device.ip4_dns_search = addr.dns_search
        
    def update_ip6(self, device, addr):

        # TODO set IPv6 attributes
        pass


