import os
import json

#
# business logic
#
from daemon.core   import DaemonPath, UiPath
from utility.views import ViewBaseLog
from _domain.node  import WsSyncdLog, ClusterServerNodesList, ClusterServerFactory, ClusterClientFactory, ClusterSettings, ClusterSettingsFactory


#
#
#
from django import forms
from django.views import generic
from django.contrib import messages
from django.shortcuts import render
from django.http import HttpResponseRedirect
from django.urls import reverse_lazy
from django.contrib.messages.views import SuccessMessageMixin

#
# cluster
#
class ClusterClientForm(forms.Form):

    enabled           = forms.BooleanField(required=False)
    server_address    = forms.CharField(max_length=254, required=False)
    server_port       = forms.IntegerField(widget=forms.TextInput())
    sync_interval     = forms.IntegerField(widget=forms.TextInput())
    sync_logs         = forms.BooleanField(required=False)
    log_sync_interval = forms.IntegerField(widget=forms.TextInput())
    
    def clean(self):

        # get cleaned data
        cleaned_data = super(ClusterClientForm, self).clean()

        # we cannot enable if server address is empty
        enabled = cleaned_data.get('enabled', False)
        if enabled:
            server_address = cleaned_data.get('server_address')
            if len(server_address) == 0:
                self.errors['__all__'] = self.error_class(["Cannot activate configuration client functionality because server address is not set."])            
                raise forms.ValidationError("")

        return cleaned_data 

    
    def clean_enabled(self):

        enabled = self.cleaned_data['enabled']
        if enabled:
            
            # ok admin wants to enable client, let's see we are NOT server already
            obj = ClusterServerFactory().construct()
            if obj.enabled:
                self.errors['__all__'] = self.error_class(["Cannot activate configuration client functionality because this node is already activated as configuration server."])            
                raise forms.ValidationError("")

        return enabled

    def clean_sync_interval(self):

        value = self.cleaned_data['sync_interval']
        if value < 1:
            raise forms.ValidationError("Must be >= 1 second; recommended value is 15. Smaller value will lead to higher CPU usage.")
        return value

    def clean_log_sync_interval(self):

        value = self.cleaned_data['log_sync_interval']
        if value < 1:
            raise forms.ValidationError("Must be >= 1 second; recommended value is 60. Smaller value will lead to higher CPU usage.")
        return value

    def clean_server_port(self):

        value = self.cleaned_data['server_port']
        if value < 1024 or value > 65534:
            raise forms.ValidationError("Must be within (1024:65535) interval; default value is 18999.")
        return value

#
#
#
class ViewClusterClient(SuccessMessageMixin, generic.View):

    template_name   = "node/cluster/client_form.html"
    success_message = "need_squid_restart"
    form_class      = ClusterClientForm

    def get_success_url(self):
        return reverse_lazy('node:ViewClusterClient')

    def get(self, request, *args, **kwargs):

        obj  = ClusterClientFactory().construct()
        form = ClusterClientForm(initial = obj.as_data())

        return render(request, self.template_name, {'form': form })

    def post(self, request, *args, **kwargs):

        form  = ClusterClientForm(request.POST)
        if form.is_valid():

            # ok form is valid, fill the data
            obj = ClusterClientFactory().construct()
            obj.enabled           = form.cleaned_data['enabled']
            obj.server_port       = form.cleaned_data['server_port']
            obj.server_address    = form.cleaned_data['server_address']
            obj.sync_interval     = form.cleaned_data['sync_interval']
            obj.sync_logs         = form.cleaned_data['sync_logs']
            obj.log_sync_interval = form.cleaned_data['log_sync_interval']
            obj.save()

            # mark as needing relstart
            messages.info(request, "need_squid_restart")

            return HttpResponseRedirect(self.get_success_url()) 

        return render(request, self.template_name, {'form': form })

#
#
#
class ClusterServerForm(forms.Form):

    enabled        = forms.BooleanField(required=False)
    listen_address = forms.CharField(max_length=254, required=False)
    listen_port    = forms.IntegerField(widget=forms.TextInput())
    
    def clean_enabled(self):

        enabled = self.cleaned_data['enabled']
        if enabled:
            
            # ok admin wants to enable server, let's see we are NOT client already
            obj = ClusterClientFactory().construct()            
            if obj.enabled:
                self.errors['__all__'] = self.error_class(["Cannot activate configuration server functionality because this node is already activated as configuration client."])            
                raise forms.ValidationError("")

        return enabled

    def clean_listen_port(self):

        value = self.cleaned_data['listen_port']
        if value < 1024 or value > 65534:
            raise forms.ValidationError("Must be within (1024:65535) interval; default value is 18999.")
        return value

#
#
#
class ViewClusterServer(SuccessMessageMixin, generic.View):

    template_name   = "node/cluster/server_form.html"
    success_message = "need_squid_restart"
    form_class      = ClusterClientForm

    def get_success_url(self):
        return reverse_lazy('node:ViewClusterServer')

    def get(self, request, *args, **kwargs):

        obj  = ClusterServerFactory().construct()
        form = ClusterServerForm(initial=obj.as_data())

        return render(request, self.template_name, {'form': form })

    def post(self, request, *args, **kwargs):

        form  = ClusterServerForm(request.POST)
        if form.is_valid():

            # ok form is valid, fill the data
            obj = ClusterServerFactory().construct() 
            obj.enabled        = form.cleaned_data['enabled']
            obj.listen_address = form.cleaned_data['listen_address']
            obj.listen_port    = form.cleaned_data['listen_port']
            obj.save()

            # mark as needing relstart
            messages.info(request, "need_squid_restart")

            return HttpResponseRedirect(self.get_success_url()) 

        return render(request, self.template_name, {'form': form })

#
#
#
class ViewClusterLog(ViewBaseLog):

    template_name = "node/cluster/sync_log.html"
    log_class     = WsSyncdLog


#
#
#
'''
class ViewClusterServerNodesList(generic.TemplateView):

    template_name = "node/cluster/nodes.html"

    def get_context_data(self, **kwargs):
        
        context = super(ViewClusterServerNodesList, self).get_context_data(**kwargs)

        # allocate default response
        data = { 
            'error' : False,  
            'desc'  : '',  
            'items' : {} 
        }
        
        try:
            # get the wssyncd port for the REST service
            wsport = 18889
            
            name = os.path.join(DaemonPath.etc_dir(), "node", "wssyncd.json")
            with open(name) as fin:    
                wssyncd = json.load(fin)
                wsport  = wssyncd['rest_service']['port']

            # see if server is enabled
            name = os.path.join(DaemonPath.etc_dir(), "node", "cluster_server.json")
            with open(name) as fin:    
                cluster = json.load(fin)

                if cluster['enabled']:
                    data['items'] = ClusterServerNodesList().run(wsport)
            
        except Exception as e:
            data['error'] = True
            data['desc']  = str(e)
        
        context['nodes'] = data

        return context
'''



#
# cluster
#
class ClusterSettingsForm(forms.Form):

    sync_mode   = forms.IntegerField(widget=forms.RadioSelect(choices=ClusterSettings.SYNC_MODE_CHOICES))
    verbose_log = forms.BooleanField(required=False)

    def clean_sync_mode(self):

        value = self.cleaned_data['sync_mode']
        if value not in [
            ClusterSettings.SYNC_MODE_HTTP, 
            ClusterSettings.SYNC_MODE_HTTPS_NOVERIFY,
            ClusterSettings.SYNC_MODE_HTTPS_SERVER, 
            ClusterSettings.SYNC_MODE_HTTPS_MUTUAL
        ]:
            value = ClusterSettings.SYNC_MODE_HTTPS_MUTUAL

        return value
    

#
#
#
class ViewClusterSettings(SuccessMessageMixin, generic.View):

    template_name   = "node/cluster/settings.html"
    success_message = "need_squid_restart"
    form_class      = ClusterSettingsForm

    def get_success_url(self):
        return reverse_lazy('node:ViewClusterSettings')

    def get(self, request, *args, **kwargs):

        obj  = ClusterSettingsFactory().construct()
        form = ClusterSettingsForm(initial=obj.as_data())

        return render(request, self.template_name, {'form': form })

    def post(self, request, *args, **kwargs):

        form  = ClusterSettingsForm(request.POST)
        if form.is_valid():

            # ok form is valid, fill the data
            obj = ClusterSettingsFactory().construct()
            obj.sync_mode   = form.cleaned_data['sync_mode']
            obj.verbose_log = form.cleaned_data['verbose_log']
            obj.save()

            # mark as needing relstart
            messages.info(request, "need_squid_restart")

            return HttpResponseRedirect(self.get_success_url()) 

        return render(request, self.template_name, {'form': form })