import os
import traceback
import shutil

#
# business logic
#
from daemon.core import System, Distrib
from _domain.node import NetDeviceEnumerator, NetDeviceDumper

#
# django
#
from django import forms
from django.views import generic
from django.contrib import messages
from django.contrib.messages.views import SuccessMessageMixin
from django.urls import reverse_lazy, reverse

#
# ours
#
from node.models import NetDevice, NetAdvanced
from node.blogic import NetworkGenerator, NetworkDeviceSyncer


#
#
#
class NetAdvancedForm(forms.ModelForm):

    class Meta:
        model  = NetAdvanced       
        fields = ['network_type', 'manual_text']

    network_type = forms.IntegerField(widget=forms.RadioSelect(choices=NetAdvanced.NETWORK_TYPE_CHOICES))
    manual_text  = forms.CharField(widget=forms.Textarea(attrs={'size':'90', 'rows':'10', 'class':'input-block-level'}),required=False)

    def clean_manual_text(self):

        value = self.cleaned_data['manual_text']
        if len(value) > 0:

            # we only accept ascii
            try:
                value.encode('ascii')
            except UnicodeEncodeError:
                raise forms.ValidationError("Value can only contain ASCII symbols")

        return value


#
#
#
class ViewNetworkAdvanced(SuccessMessageMixin, generic.edit.UpdateView):
    
    model           = NetAdvanced
    form_class      = NetAdvancedForm
    template_name   = "node/network/advanced_form.html"
    
    def get_success_url(self):
        return reverse_lazy('node:ViewNetworkAdvanced')

    def get_object(self):
        return NetAdvanced.objects.first()

    def form_valid(self, form):

        try:

            # check we use only tested management styles
            network_type = form.cleaned_data['network_type']

            if network_type == NetAdvanced.NETWORK_TYPE_INTERFACES:
                if Distrib.name() not in [Distrib.WS_DEBIAN11, Distrib.WS_WINDOWSX]:
                    raise Exception("This network management type is only supported on Debian 11 (/etc/network/interfaces). Please select a different management type.")

            if network_type == NetAdvanced.NETWORK_TYPE_NETPLAN:
                if Distrib.name() not in [Distrib.WS_UBUNTU20, Distrib.WS_WINDOWSX]:
                    raise Exception("This network management type is only supported on Ubuntu 20 (netplan). Please select a different management type.")

            # save the form into the database
            result = super(ViewNetworkAdvanced, self).form_valid(form)

            # get the node and generate all devices
            NetworkGenerator().generate(
                NetDevice.objects.all(),
                NetAdvanced.objects.first()
            )

            # tell the user
            messages.info(self.request, "need_reboot")

            # and return successfully
            return result

        except Exception as e:        

            form.errors['__all__']   = form.error_class(["%s" % str(e)])
            form.errors['traceback'] = "%s" % traceback.format_exc()

            # failure
            return super(ViewNetworkAdvanced, self).form_invalid(form)

#
#
#
class ViewNetworkDeviceList(SuccessMessageMixin, generic.ListView):

    template_name = "node/network/device_list.html"

    def get_queryset(self):
        
        # see if we need to force sync - i.e. read from system
        reread = self.request.GET.get('reread', '')
        if reread in ["1", "on", "yes", "true"]:
            force_sync = True
        else:
            force_sync = False

        # resync devices from system
        NetworkDeviceSyncer().reread_from_system(force_sync)
        
        # and return
        return NetDevice.objects.all()

    def get_context_data(self, **kwargs):
        context = super(ViewNetworkDeviceList, self).get_context_data(**kwargs)
        context['devices_dump'] = NetDeviceDumper().dump_devices()
        context['netadvanced']  = NetAdvanced.objects.first()
        return context


#
#
#
class ViewNetworkDeviceIpv4Update(SuccessMessageMixin, generic.edit.UpdateView):

    model           = NetDevice
    success_message = "need_reboot"
    template_name   = "node/network/device_form_ipv4.html"
    fields          = ['ip4_config', 'ip4_address', 'ip4_netmask', 'ip4_gateway', 'ip4_dns_srv1', 'ip4_dns_srv2', 'ip4_dns_search']

    def get_success_url(self):
        return reverse_lazy('node:ViewNetworkDeviceIpv4Update', kwargs = {'pk' : self.kwargs['pk'] })

    def get_context_data(self, **kwargs):
        context = super(ViewNetworkDeviceIpv4Update, self).get_context_data(**kwargs)
        context['device_dump'] = NetDeviceDumper().dump_device(self.object.name)
        return context

    def form_valid(self, form):

        try:
            # save the form into the database
            result = super(ViewNetworkDeviceIpv4Update, self).form_valid(form)

            # get the node and generate all devices
            NetworkGenerator().generate(
                NetDevice.objects.all(),
                NetAdvanced.objects.first()
            )

            # and return successfully
            return result

        except Exception as e:      

            form.errors['__all__']   = form.error_class(["%s\n%s" % (str(e), traceback.format_exc())])
            form.errors['traceback'] = "%s" % traceback.format_exc()

            # failure
            return super(ViewNetworkDeviceIpv4Update, self).form_invalid(form)
   

#
#
#
class ViewNetworkDeviceIpv6Update(SuccessMessageMixin, generic.edit.UpdateView):

    model           = NetDevice
    template_name   = "node/network/device_form_ipv6.html"
    fields          = ['ip6_config', 'ip6_address', 'ip6_subnet', 'ip6_gateway', 'ip6_dns_srv1', 'ip6_dns_srv2', 'ip6_dns_search']
    success_message = "need_reboot"
    
    def get_success_url(self):
        return reverse_lazy('node:ViewNetworkDeviceIpv6Update', kwargs = {'pk' : self.kwargs['pk'] })

    def get_context_data(self, **kwargs):
        context = super(ViewNetworkDeviceIpv6Update, self).get_context_data(**kwargs)
        context['device_dump'] = NetDeviceDumper().dump_device(self.object.name)
        return context

    def form_valid(self, form):

        try:
            # save the form into the database
            result = super(ViewNetworkDeviceIpv6Update, self).form_valid(form)

            # get the node and generate all devices
            NetworkGenerator().generate(
                NetDevice.objects.all(),
                NetAdvanced.objects.first()
            )

            # and return successfully
            return result

        except Exception as e:        
            form.errors['__all__']   = form.error_class(["%s\n%s" % (str(e), traceback.format_exc())])
            form.errors['traceback'] = "%s" % traceback.format_exc()

            # failure
            return super(ViewNetworkDeviceIpv6Update, self).form_invalid(form)
    