import os
import shutil
import tempfile
import traceback
#
#
#
from django import forms
from django.http import HttpResponse, HttpResponseRedirect
from django.views import generic
from django.contrib import messages
from django.contrib.messages.views import SuccessMessageMixin
from django.urls import reverse

#
#
#
from daemon.core   import DaemonPath, UiPath
from daemon.utils  import ElevatedCommand
from _domain.squid import SquidCacheInitializer

#
#
#
from squid.models import MemoryCache, DiskCache
from squid.generator import Generator

#
#
#
class ViewCacheMemory(SuccessMessageMixin, generic.edit.UpdateView):

    model           = MemoryCache
    fields          = ['cache_mem', 'maximum_object_size_in_memory', 'memory_replacement_policy']
    template_name   = 'squid/cache/memory.html'
    success_message = 'need_squid_restart'

    def get_object(self):
        return MemoryCache.objects.first()
    
    def get_success_url(self): 
        return reverse("squid:ViewCacheMemory")


#
#
#
class ViewCacheDisk(SuccessMessageMixin, generic.edit.UpdateView):

    model           = DiskCache
    fields          = ['cache_replacement_policy', 'minimum_object_size', 'maximum_object_size', 'cache_type', 'ufs_mb']
    template_name   = 'squid/cache/disk.html'
    success_message = 'need_squid_restart'

    def get_object(self):
        return DiskCache.objects.first()
    
    def get_success_url(self): 
        return reverse("squid:ViewCacheDisk")

#
#
#
class ViewCacheDiskEnable(generic.View):

    success_message = "need_squid_restart"
    
    def post(self, request, *args, **kwargs):

        diskcache = DiskCache.objects.first()
        value     = request.POST.get('enabled', '0')
        if value == '1':
            diskcache.enabled = True
        else:
            diskcache.enabled = False
        
        diskcache.save()
        
        messages.info(self.request, self.success_message)   

        return HttpResponseRedirect(reverse("squid:ViewCacheDisk"))


#
#
#
class DiskResetForm(forms.Form):
    confirm = forms.CharField()
    
    def clean_confirm(self):

        value = self.cleaned_data['confirm']
        value = value.strip()
        value = value.upper()

        if value != "I CONFIRM":
            raise forms.ValidationError("Please type 'I confirm' in this field if you agree to erase and re-initialize the cache dir.")

        return value

#
#
#
class ViewCacheDiskReset(SuccessMessageMixin, generic.edit.FormView):
    
    template_name   = 'squid/cache/disk_reset.html'
    form_class      = DiskResetForm
    success_message = 'need_squid_restart'

    
    def get_success_url(self): 
        return reverse("squid:ViewCacheDisk")

    def get_context_data(self, **kwargs):
        
        context = super(ViewCacheDiskReset, self).get_context_data(**kwargs)    
        context['disk_cache'] = DiskCache.objects.first()
        return context

    def form_valid(self, form):

        try:

            # ok the user wants us to reset the cache
            disk_cache = DiskCache.objects.first()

            # first we must enable the cache in the database
            disk_cache.enabled = True
            disk_cache.save()

            # generate the squid configuration on disk
            self.generate_conf()            

            # and call special sudoing binary that will stop squid, reset the cache and start squid again
            SquidCacheInitializer().initialize()

            # perfect now we have the cache enabled and reinitialized, return nicely
            return super(ViewCacheDiskReset, self).form_valid(form)
            
        except Exception as e:        
            form.errors['__all__'] = form.error_class([
                "%s\n%s" % (str(e), traceback.format_exc()) 
            ])

            # failure
            return super(ViewCacheDiskReset, self).form_invalid(form)

    def generate_conf(self):

        # first crete temporary folder
        temp_dir = ""
        if True:

            with tempfile.NamedTemporaryFile(prefix="cache_reset", suffix=".folder", dir=os.path.join(UiPath.var_dir(), "temp"), delete=True) as temp:
                temp_dir = temp.name

            temp_dir = os.path.join(temp_dir, "squid")

        # make it on disk
        if not os.path.isdir(temp_dir):
            os.makedirs(temp_dir)

        # run generator into it
        g = Generator(temp_dir)
        g.generate()

        # then move into place using elevation
        self.publish_dir(
            temp_dir, 
            os.path.join(DaemonPath.etc_dir(), "squid")
        )

        # and finally drop the temp dir
        shutil.rmtree(temp_dir)

    def publish_dir(self, source_dir, target_dir):

        # publish means move into /opt/websafety/etc using elevation
        exe  = os.path.join(UiPath.bin_dir(), "fsmgr.py")
        arg1 = "--source-folder=%s" % source_dir
        arg2 = "--target-folder=%s" % target_dir
        args = [exe, arg1, arg2]

        (exit_code, stdout, stderr) = ElevatedCommand.run(args)

        # fail if failed
        if exit_code != 0:

            raise Exception(
                "Cannot copy folder '%s' into '%s', exit code %d, stdout %s, stderr %s" % (
                    source_dir, target_dir, exit_code, stdout, stderr
                )
            )
