import os
import re
import errno
import traceback

#
# domain logic (non django)
#
from daemon.core  import DaemonPath
from daemon.tools import HtpasswdExe

#
# django
#
from django.contrib import messages
from django import forms
from django.views import generic
from django.shortcuts import render, get_object_or_404
from django.urls import reverse
from django.http import HttpResponse, HttpResponseRedirect
from django.contrib.messages.views import SuccessMessageMixin

#
#
#
from squid.models      import AuthLocalDbUser
from utility.validator import *

#
# list of users
#
class ViewAuthLocalDbUsersList(generic.ListView):

    model         = AuthLocalDbUser
    template_name = "squid/auth/localdb_user_list.html"
    
    def get_success_url(self): 
        return reverse("squid:ViewAuthLocalDbUsersList")

    def get_context_data(self, **kwargs):

        # see if it exists
        if os.path.isfile(os.path.join(DaemonPath.etc_dir(), "squid", "users.htpasswd")):
            exists = True
        else:
            exists = False

        # save to context to enable export button
        context = super(ViewAuthLocalDbUsersList, self).get_context_data(**kwargs)        
        context['htpasswd_exists'] = exists

        # and return
        return context

    def post(self, request, *args, **kwargs):

        for selected_id in request.POST.getlist('selected_id'):   
            item = get_object_or_404(AuthLocalDbUser, pk=selected_id)
            item.delete()         
            
        # tell the user squid needs to be reloaded or restarted            
        messages.info(self.request, "need_squid_restart")

        # and redirect to success url
        return HttpResponseRedirect(self.get_success_url())

#
# create a user
#
class AuthLocalDbUserForm(forms.ModelForm):

    def clean_comment(self):
        return AsciiOnly.clean(self.cleaned_data['comment'])

class AuthLocalDbUserCreateForm(AuthLocalDbUserForm):

    class Meta:
        model  = AuthLocalDbUser
        fields = ['user_name', 'user_pass', 'comment']

    def clean_user_name(self):

        # general checks
        value = self.cleaned_data['user_name']
        value = Strip.clean(value)
        value = AsciiOnly.clean(value)
        value = NoWhiteSpaces.clean(value)
        value = NoWildcards.clean(value)

        # specific checks
        matched = re.match(r"^[\w\.]+$", value, re.M|re.I)
        if not matched:
            raise forms.ValidationError("User name can only be alpha-numeric (including dot).")

        # we only support lower cased user names because of bug 1687
        if value.lower() != value:
            raise forms.ValidationError("Sorry for now only lower cased user names are supported.")

        return value

    def clean_user_pass(self):

        # get the password
        value = self.cleaned_data['user_pass']
        value = Strip.clean(value)
        value = AsciiOnly.clean(value)
        value = NoWhiteSpaces.clean(value)

        # ok run the htpasswd command and encode the password
        try:
            value = HtpasswdExe().encode(value)
        except Exception as e:
            raise forms.ValidationError("Cannot encode password, error: %s." % str(e))

        return value

class ViewAuthLocalDbUsersCreate(SuccessMessageMixin, generic.edit.CreateView):

    model           = AuthLocalDbUserForm
    form_class      = AuthLocalDbUserCreateForm
    template_name   = "squid/auth/localdb_user_create.html"
    success_message = "need_squid_restart"    
    
    def get_success_url(self): 
        return reverse("squid:ViewAuthLocalDbUsersList")

#
# update a user - only comment can change, if admin needs to reset the user password he needs to remove/create 
#
class AuthLocalDbUserUpdateForm(AuthLocalDbUserForm):

    class Meta:
        model  = AuthLocalDbUser
        fields = ['comment']

class ViewAuthLocalDbUsersUpdate(SuccessMessageMixin, generic.edit.UpdateView):

    model           = AuthLocalDbUser
    form_class      = AuthLocalDbUserUpdateForm
    template_name   = "squid/auth/localdb_user_update.html"
    success_message = "need_squid_restart"    
    
    def get_success_url(self): 
        return reverse("squid:ViewAuthLocalDbUsersList")

#
#
#
class ViewAuthLocalDbExport(generic.View):

    def get(self, request, *args, **kwargs):            

        path = os.path.join(DaemonPath.etc_dir(), "squid", "users.htpasswd")
        with open(path, "rb") as fin:
            response = HttpResponse(fin.read(), content_type="application/octet-stream")
            response['Content-Disposition'] = "attachment; filename=\"%s\"" % "users.htpasswd"
            return response

#
#
#
class AuthLocalDbImportForm(forms.Form):
    file = forms.FileField()

#
#
#
class ViewAuthLocalDbImport(SuccessMessageMixin, generic.edit.FormView):

    form_class      = AuthLocalDbImportForm
    template_name   = 'squid/auth/localdb_users_import.html'
    success_message = 'need_squid_restart'
    
    def get_success_url(self):
        return reverse("squid:ViewAuthLocalDbUsersList")

    def form_valid(self, form):

        try:

            # combine contents
            contents = ""

            if True:
                data = self.request.FILES['file']
                for chunk in data.chunks():
                    contents += chunk.decode()

            # get the lines
            lines = contents.splitlines()
            for line in lines:

                # strip
                line = line.rstrip()
                pos  = line.find(":")
                if pos == -1:
                    continue

                user_name = line[:pos]
                user_pass = line[pos + 1:]

                if not user_name:
                    continue
                if not user_pass:
                    continue

                # add to the database
                try:

                    v, c = AuthLocalDbUser.objects.get_or_create(user_name=user_name)
                    v.user_pass = user_pass
                    v.comment   = ""

                    v.save()

                except Exception as e:
                    pass
                    
            # ok if we got here everything is fine
            return super(ViewAuthLocalDbImport, self).form_valid(form)
            
        except Exception as e:        
            form.errors['__all__'] = form.error_class(["%s" % str(e)])

            # failure
            return super(ViewAuthLocalDbImport, self).form_invalid(form)