import os
import shutil
import traceback

#
# domain logic (non django)
#
from daemon.core import DaemonPath, Distrib
from _domain.squid import SquidCert, SquidCertDbInitializer


#
#
#
from django import forms
from django.http import HttpResponse, HttpResponseRedirect
from django.views import generic
from django.contrib import messages
from django.urls import reverse
from django.contrib.messages.views import SuccessMessageMixin

#
#
#
from squid.models import BumpMode
from utility.validator import *


class BumpModeForm(forms.ModelForm):

    class Meta:
        model  = BumpMode       
        fields = ['value', 'download_missing_intermediate_certs_automatically']

    value = forms.IntegerField(widget=forms.RadioSelect(choices=BumpMode.BUMPMODE_CHOICES))

#
#
#
class ViewSslInfo(SuccessMessageMixin, generic.edit.UpdateView):

    model           = BumpMode
    form_class      = BumpModeForm    
    success_message = 'need_squid_restart'
    template_name   = 'squid/ssl/mode.html'

    def get_success_url(self):
        return reverse('squid:ViewSslInfo')

    def get_object(self):
        return BumpMode.objects.first()

    def form_valid(self, form):

        try:

            # for debugging, windows does not init the ssl storage (all other systems do)

            if Distrib.name() not in [Distrib.WS_WINDOWSX]:

                # always reinitialize the SSL storage first - this is important as user *may* have uploaded a different
                # root certficate - squid needs to get rid of all those already cached in the ssl storage
                SquidCertDbInitializer().initialize()

            # save the form into the database
            result = super(ViewSslInfo, self).form_valid(form)

            # and return successfully
            return result

        except Exception as e:        

            form.errors['__all__']   = form.error_class(["%s" % str(e)])
            form.errors['traceback'] = "%s" % traceback.format_exc()

            # failure
            return super(ViewSslInfo, self).form_invalid(form)

    def get_context_data(self, **kwargs):
        
        context = super(ViewSslInfo, self).get_context_data(**kwargs)    

        result = False 
        cert   = None
        error  = ""

        try:
            cert   = SquidCert().get()
            result = True

        except Exception as e:
            result = False
            error  = "%s\nStack:%s" % (str(e), traceback.format_exc())

        context['cert']     = cert
        context['result']   = result
        context['error']    = error
        
        return context
#
#
#
class ViewSslRootCa(generic.base.TemplateView):

    template_name='squid/ssl/cert.html'

    def get_context_data(self, **kwargs):
        
        context = super(ViewSslRootCa, self).get_context_data(**kwargs)    

        result = False 
        cert   = None
        error  = ""

        try:
            cert   = SquidCert().get()
            result = True

        except Exception as e:
            result = False
            error  = "%s\nStack:%s" % (str(e), traceback.format_exc())

        context['cert']     = cert
        context['result']   = result
        context['error']    = error
        context['bumpmode'] = BumpMode.objects.first()
        

        return context

#
#
#
class ViewSslDer(generic.View):

    def get(self, request, *args, **kwargs):  

        path = os.path.join(DaemonPath.etc_dir(), "myca.der")
        with open(path, "rb") as fin:
            response = HttpResponse(fin.read(), content_type="application/x-x509-ca-cert")
            response['Content-Disposition'] = "attachment; filename=\"%s\"" % "myca.der"
            return response

#
#
#
class ViewSslPemBackUp(generic.View):

    def get(self, request, *args, **kwargs):            

        path = os.path.join(DaemonPath.etc_dir(), "myca.pem")
        with open(path, "rb") as fin:
            response = HttpResponse(fin.read(), content_type="application/x-x509-ca-cert")
            response['Content-Disposition'] = "attachment; filename=\"%s\"" % "myca.pem"
            return response

#
#
#
class PemUploadForm(forms.Form):
    file = forms.FileField()

#
#
#
class ViewSslPemUpload(SuccessMessageMixin, generic.edit.FormView):

    form_class      = PemUploadForm
    template_name   = 'squid/ssl/pem_upload.html'
    success_message = 'need_squid_restart'
    
    def get_success_url(self): 
        return reverse('squid:ViewSslInfo')

    def form_valid(self, form):

        try:
            # get data
            data = self.request.FILES['file']

            # upload it
            cert = SquidCert()
            cert.upload(data)
            
            # ok if we got here everything is fine
            return super(ViewSslPemUpload, self).form_valid(form)
            
        except Exception as e:        

            # convert error to string
            error = "%s%s" % (str(e), traceback.format_exc())

            # and add it to collection of form errors
            form.errors['__all__'] = form.error_class([error])

            # failure
            return super(ViewSslPemUpload, self).form_invalid(form)
#
#
#
class PemGenerateForm(forms.Form):

    newCertSupportEMail = forms.CharField(max_length=64)
    newCertCommonName   = forms.CharField(max_length=64)
    newCertLifeTime     = forms.IntegerField(widget=forms.TextInput())
    newCertOrganization = forms.CharField(max_length=64)
    newCertOU           = forms.CharField(max_length=64)
    newCertCountry      = forms.CharField(max_length=2)
    newCertState        = forms.CharField(max_length=64)
    newCertCity         = forms.CharField(max_length=64)

    def clean_newCertSupportEMail(self):
        return AsciiOnly.clean(self.cleaned_data['newCertSupportEMail'])

    def clean_newCertCommonName(self):
        return AsciiOnly.clean(self.cleaned_data['newCertCommonName'])

    def clean_newCertOrganization(self):
        return AsciiOnly.clean(self.cleaned_data['newCertOrganization'])

    def clean_newCertOU(self):
        return AsciiOnly.clean(self.cleaned_data['newCertOU'])

    def clean_newCertCountry(self):
        return AsciiOnly.clean(self.cleaned_data['newCertCountry'])

    def clean_newCertState(self):
        return AsciiOnly.clean(self.cleaned_data['newCertState'])

    def clean_newCertCity(self):
        return AsciiOnly.clean(self.cleaned_data['newCertCity'])


class ViewSslPemGenerate(SuccessMessageMixin, generic.edit.FormView):

    form_class      = PemGenerateForm
    template_name   = 'squid/ssl/pem_generate.html'
    success_message = 'need_squid_restart'
    
    def get_success_url(self): 
        return reverse('squid:ViewSslInfo')

    def form_valid(self, form):

        try:

            # generate it
            cert = SquidCert()
            cert.generate(
                form.cleaned_data['newCertCountry'],
                form.cleaned_data['newCertState'],
                form.cleaned_data['newCertCity'],
                form.cleaned_data['newCertOrganization'],
                form.cleaned_data['newCertOU'],
                form.cleaned_data['newCertSupportEMail'],
                form.cleaned_data['newCertCommonName'],
                form.cleaned_data['newCertLifeTime']
            )

            # ok if we got here everything is fine
            return super(ViewSslPemGenerate, self).form_valid(form)

        except Exception as e:        

            # convert error to string
            error = "%s%s" % (str(e), traceback.format_exc())

            # and add it to collection of form errors
            form.errors['__all__'] = form.error_class([error])

            # failure
            return super(ViewSslPemGenerate, self).form_invalid(form)