import os
import re
import json
import errno
import shutil
import traceback

#
# domain logic (non django)
#
from daemon.core  import DaemonPath, UiPath
from daemon.utils import JsonDumper, SudoPublisher
from _domain.utils import TempFolder
from _domain.squid import \
    AdDetector, \
    LdapTester, \
    LdapsDetector, \
    LdapCreds, LdapEndpoint, KeyTabDumper, KerberosChecker, KeyTabInitializer, \
    PseudoAuthDumper

#
#
#
from django import forms
from django.views import generic
from django.contrib.messages.views import SuccessMessageMixin
from django.urls import reverse
from django.http import HttpResponse, HttpResponseRedirect
from django.contrib import messages
from django.shortcuts import get_object_or_404
from django.template import loader, Context
from django.forms.utils import ErrorList

#
#
#
from squid.generator   import Generator
from squid.models      import *
from utility.validator import *


#
# auth - user labelling
#
class ViewAuthLabelEdit(SuccessMessageMixin, generic.edit.UpdateView):

    template_name   ='squid/auth/label_general.html'
    success_message = "need_squid_restart"
    model           = AuthLabel
    fields          = ['enable', 'helper_verbose', 'helper_total', 'helper_idle', 'helper_startup', 'resolve_ip_as_user_name']

    def get_object(self):   
        return AuthLabel.objects.first()

    def get_success_url(self): 
        return reverse("squid:ViewAuthLabelEdit")

#
# list of user labels
#
class ViewAuthLabelList(generic.ListView):

    template_name ='squid/auth/label_users.html'
    model         = AuthLabelUsers
    
    def get_success_url(self):
        return reverse("squid:ViewAuthLabelList")

    def post(self, request, *args, **kwargs):

        for selected_id in request.POST.getlist('selected_id'):   
            item = get_object_or_404(AuthLabelUsers, pk=selected_id)
            item.delete()         
            
        # tell the user squid needs to be reloaded or restarted            
        messages.info(self.request, "need_squid_restart")

        # and redirect to success url
        return HttpResponseRedirect(self.get_success_url())

#
#
#
class AuthLabelUsersForm(forms.ModelForm):

    class Meta:
        model  = AuthLabelUsers
        fields = ['user_ip', 'user_name', 'user_mac', 'comment']
    
    def clean_user_name(self):

        # general checks
        value = self.cleaned_data['user_name']
        value = Strip.clean(value)
        value = AsciiOnly.clean(value)
        value = NoWhiteSpaces.clean(value)
        value = NoWildcards.clean(value)

        # specific checks
        matched = re.match(r"^[\w\.]+$", value, re.M|re.I)
        if not matched:
            raise forms.ValidationError("User name can only be alpha-numeric (including dot).")

        # we only support lower cased user names because of bug 1687
        if value.lower() != value:
            raise forms.ValidationError("Sorry for now only lower cased user names are supported.")

        return value

    def clean_comment(self):
        return AsciiOnly.clean(self.cleaned_data['comment'])

#
#
#
class ViewAuthLabelCreate(SuccessMessageMixin, generic.edit.CreateView):

    template_name   ='squid/auth/label_users_form.html'
    success_message = "need_squid_restart"    
    model           = AuthLabelUsers
    form_class      = AuthLabelUsersForm
    
    def get_success_url(self):
        return reverse("squid:ViewAuthLabelList")

    def form_valid(self, form):

        try:
            if 'user_ip' in form.cleaned_data:
                if 'user_mac' in form.cleaned_data:

                    user_ip  = form.cleaned_data['user_ip'].strip()
                    user_mac = form.cleaned_data['user_mac'].strip()

                    if len(user_ip) == 0 and len(user_mac) == 0:
                        raise Exception("Please specify either IP or MAC address.")

            return super(ViewAuthLabelCreate, self).form_valid(form)
            
        except Exception as e:        
            form.errors['__all__'] = form.error_class(["%s" % str(e)])

            # failure
            return super(ViewAuthLabelCreate, self).form_invalid(form)
    
#
#
#
class ViewAuthLabelUpdate(SuccessMessageMixin, generic.edit.UpdateView):

    template_name   ='squid/auth/label_users_form.html'
    success_message = "need_squid_restart"    
    model           = AuthLabelUsers
    form_class      = AuthLabelUsersForm

    def get_success_url(self):
        return reverse("squid:ViewAuthLabelList")

    def form_valid(self, form):

        try:
            if 'user_ip' in form.cleaned_data:
                if 'user_mac' in form.cleaned_data:

                    user_ip  = form.cleaned_data['user_ip'].strip()
                    user_mac = form.cleaned_data['user_mac'].strip()

                    if len(user_ip) == 0 and len(user_mac) == 0:
                        raise Exception("Please specify either IP or MAC address.")

            return super(ViewAuthLabelUpdate, self).form_valid(form)
            
        except Exception as e:        
            form.errors['__all__'] = form.error_class(["%s" % str(e)])

            # failure
            return super(ViewAuthLabelUpdate, self).form_invalid(form)
