#
# system
#

#
# django
#
from django import forms

#
# our
#
from traffic.models    import PeriodicReport
from utility.validator import AsciiOnly

#
#
#
class PeriodicReportForm(forms.ModelForm):

    class Meta:
        model  = PeriodicReport
        fields = '__all__'

    timeframe_from = forms.DateField(required=False, widget=forms.DateInput(attrs={ 'class':'datepicker' }))
    timeframe_to   = forms.DateField(required=False, widget=forms.DateInput(attrs={ 'class':'datepicker' }))
    
    def clean(self):

        # we need to validate schedule - it must NOT be 'all *'
        if 'schedule_min' in self.cleaned_data:
            if 'schedule_hour' in self.cleaned_data:
                if 'schedule_dom' in self.cleaned_data:
                    if 'schedule_month' in self.cleaned_data:
                        if 'schedule_dow' in self.cleaned_data:

                            # ok all schedule fields are valid, get them
                            s_min   = self.cleaned_data['schedule_min'].strip()
                            s_hour  = self.cleaned_data['schedule_hour'].strip()
                            s_dom   = self.cleaned_data['schedule_dom'].strip()
                            s_month = self.cleaned_data['schedule_month'].strip()
                            s_dow   = self.cleaned_data['schedule_dow'].strip()

                            # and check
                            if s_min in ["", "*"]:
                                if s_hour in ["", "*"]:
                                    if s_dom in ["", "*"]:
                                        if s_month in ["", "*"]:
                                            if s_dow in ["", "*"]:
                                                raise forms.ValidationError("empty schedule is not allowed, set at least minutes and/or hours.")

        # validate time_type
        timeframe_type = self.cleaned_data['timeframe_type']
        if timeframe_type == PeriodicReport.TIMEFRAME_FROMTO:
            timeframe_from = self.cleaned_data['timeframe_from']
            timeframe_to   = self.cleaned_data['timeframe_to']
            if timeframe_from is None or timeframe_to is None:
                raise forms.ValidationError("specify values for boths date-from and date-to fields")
            if timeframe_from > timeframe_to:
                raise forms.ValidationError("date-from shall be earlier that date-to field")

        return self.cleaned_data


    def is_ascii(self, s):
        return all(ord(c) < 128 for c in s)

    def clean_name(self):

        name = self.cleaned_data['name']
        name = AsciiOnly.clean(name)
        
        # we need to case insensitive compare names in the database, it means we must manually walk it
        for item in PeriodicReport.objects.all():
            if item.pk == self.instance.pk:
                continue
            if item.name.lower() == name.lower():
                raise forms.ValidationError("Periodic report '%s' already exists." % name)

        # check for banned symbols
        banned = ["&", "/", "\\", "(", ")", ".", ",", "*"]
        for c in banned:
            if name.find(c) != -1:
                raise forms.ValidationError("Report name cannot contain characters %s" % (" ".join(banned)))

        return name

    '''
    def clean_include_domains(self):

        value = self.cleaned_data['include_domains']
        ttype = self.cleaned_data['template_type']

        if ttype == 'icap-domains-detailed-domain-info':
            if len(value.strip()) == 0:
                raise forms.ValidationError("Domain name must be filled")

        return value
    '''

    def clean_schedule_hour(self):

        value = self.cleaned_data['schedule_hour'].strip()
        if value == '*':
            return value
        try:
            n = int(value)
            if n < 0 or n > 23:
                raise Exception("invalid hour specified (%s)" % value)
        except Exception as e:
            raise forms.ValidationError("hours must be either * or within interval [0:23]. Error: %s" % str(e))

        return value

    def clean_schedule_min(self):

        value = self.cleaned_data['schedule_min'].strip()
        if value == '*':
            return value
        try:
            n = int(value)
            if n < 0 or n > 59:
                raise Exception("invalid minute specified (%s)" % value)
        except Exception as e:
            raise forms.ValidationError("minutes must be either * or within interval [0:59]. Error: %s" % str(e))

        return value

    def clean_schedule_dom(self):

        value = self.cleaned_data['schedule_dom'].strip()
        if value == '*':
            return value
        try:
            n = int(value)
            if n < 0 or n > 31:
                raise Exception("invalid day of month specified (%s)" % value)
        except Exception as e:
            raise forms.ValidationError("Day of month must be either * or within interval [0:31]. Error: %s" % str(e))

        return value

    def clean_schedule_month(self):

        value = self.cleaned_data['schedule_month'].strip()
        if value == '*':
            return value
        try:
            n = int(value)
            if n < 1 or n > 12:
                raise Exception("invalid month specified (%s)" % value)
        except Exception as e:
            raise forms.ValidationError("Month must be either * or within interval [1:12]. Error: %s" % str(e))

        return value

    def clean_schedule_dow(self):

        value = self.cleaned_data['schedule_dow'].strip()
        if value == '*':
            return value
        try:
            n = int(value)
            if n < 0 or n > 6:
                raise Exception("invalid day of week specified (%s)" % value)
        except Exception as e:
            raise forms.ValidationError("Day of week must be either * or within interval [0:6]. Error: %s" % str(e))

        return value
