import os
import re
import shutil

#
#
#
from django.template import loader

#
# domain
#
from daemon.core   import System, UiPath
from daemon.utils  import FileWriter, JsonDumper, ElevatedCommand

#
# ours
#
from utility.generator import BaseGenerator
from traffic.models    import *
from squid.models      import Network, TroubleShooting
from safety.models     import Policy
from console.settings  import DATABASES

#
#
#
class Generator(BaseGenerator):

    #
    #
    #
    def __init__(self, root_dir):

        # call the base class
        super(Generator, self).__init__(root_dir)

    def generate_config(self, target_dir):

        # create writer and dumper
        w = FileWriter(target_dir)
        d = JsonDumper()

        # and generate (for now nothing as we have new reports)
        self.generate_traffic(w, d)
        self.generate_reports(w, d)
        self.generate_crontab()    
        self.generate_logrotate()    
        
    #
    #
    #
    def generate_traffic(self, writer, dumper):

        o = Traffic.objects.first()
        m = Mailer.objects.first()
        t = TroubleShooting.objects.first()
        d = { 
            "realtime" : {
                "regeneration_interval" : o.realtime_regeneration_interval,
                "last_line_count"       : o.realtime_last_line_count
            },
            "collector": {
                "enable"                       : o.collector_enable,
                "ignore_status_non_aborted"    : o.collector_ignore_status_non_aborted,
                "ignore_status_407"            : o.collector_ignore_status_407,
                "ignore_error_invalid_request" : o.collector_ignore_error_invalid_request,                
                "ignore_error_transaction_end" : o.collector_ignore_error_transaction_end,

                "ignore_module_ads"              : o.collector_ignore_module_ads,
                "ignore_module_privacy"          : o.collector_ignore_module_privacy,
                "ignore_module_sslbump"          : o.collector_ignore_module_sslbump,
                "ignore_module_clean"            : o.collector_ignore_module_clean,
                "ignore_module_adult_heuristics" : o.collector_ignore_module_adult_heuristics,
                "ignore_module_adult_safesearch" : o.collector_ignore_module_adult_safesearch,
                "ignore_module_adult_youtube"    : o.collector_ignore_module_adult_youtube,
                "ignore_module_adult_phrases"    : o.collector_ignore_module_adult_phrases,
                "ignore_module_adult_image"      : o.collector_ignore_module_adult_image,
                "ignore_module_apps"             : o.collector_ignore_module_apps,
                "anonymized_policies"            : self.generate_anonymized_policies(),
                "folders"                        : self.generate_list(o.collector_folders),
                "cluster"                        : o.collector_cluster,
                "log_retention_days"             : t.log_retension_days      # note this comes from Squid's Troubleshooting model
            },
            "reporter" : {
                "max_detailed_users"   : o.reporter_max_detailed_users,
                "max_detailed_domains" : o.reporter_max_detailed_domains
            },
            "mailer" : {
                'server'   : m.smtp_server,
                'port'     : m.smtp_port,
                'username' : m.smtp_username,
                'password' : m.smtp_password,
                'use_auth' : m.smtp_use_auth
            }
        }
        writer.write('traffic.json', dumper.dumps(d))

    def generate_anonymized_policies(self):

        policies = []
        for policy in Policy.objects.all():         
            if policy.advanced.hide_history:
                policies.append(policy.name.lower())

        return policies



    #
    #
    #
    def generate_reports(self, writer, dumper):

        # save reporter
        mailer = Mailer.objects.first()
        data   = []

        # save only enabled periodic reports
        for item in PeriodicReport.objects.all():
                
            if not item.enabled:
                continue

            data.append(
                self.generate_report(item)
            )

        # and write file
        writer.write('periodic_reports.json', dumper.dumps(data))

    def generate_logrotate(self):

        # render the file contents
        data = ""
        if True:
            data = loader.get_template(
                os.path.join("squid/conf/etc_logrotate_d_squid")
            ).render(context={"troubleshooting" : TroubleShooting.objects.first()})

        # now generate the logrotate file in websafety-ui folder
        target_dir  = os.path.join(UiPath.etc_dir(), "squid")
        logrotate_d = os.path.join(target_dir, "etc_logrotate_d_squid")

        # write it
        with open(logrotate_d, 'w') as fout:
            fout.write(data)

        # do nothing on windows
        if System.name() == System.WS_WINDOWS:
            return

        # publish the logrotate file
        exe  = os.path.join(UiPath.bin_dir(), "logrotatemgr.py")
        args = [exe]

        (exit_code, stdout, stderr) = ElevatedCommand.run(args)

        # mgr failed, shall we fail?
        if exit_code != 0:

            raise Exception(
                "Cannot publish squid's logrotate file at /etc/logrotate.d/squid, exit code: %d, stdout: %s, stderr: %s" % (
                        exit_code, stdout, stderr
                    )
                )

    def generate_crontab(self):

        # note: crontab will be directly generated in websafety-ui folder, not websafety-core
        target_dir = os.path.join(UiPath.etc_dir(), "traffic")

        # generate cron tab lines
        lines = []
        for report in PeriodicReport.objects.all():
            
            if not report.enabled:
                continue

            line  = "%s %s %s %s %s " % (report.schedule_min, report.schedule_hour, report.schedule_dom, report.schedule_month, report.schedule_dow)
            line += "websafety "
            line += "/usr/bin/python3 /opt/websafety-ui/bin/periodic.py --report=\"%s\" " % report.name
            line += "> \"/opt/websafety-ui/var/reports/periodic/%s.log\" 2>&1" % report.name

            lines.append(line)

        # write them into a file on disk
        our_periodic = os.path.join(target_dir, "websafety_periodic")
        with open(our_periodic, 'w') as fout:
            for line in lines:
                fout.write("%s\n" % line)

        # publish cron file and tell cron to restart
        if System.name() != System.WS_WINDOWS:

            exe  = os.path.join(UiPath.bin_dir(), "cronmgr.py")
            args = [exe]

            (exit_code, stdout, stderr) = ElevatedCommand.run(args)

            # cron failed, shall we fail?
            if exit_code != 0:

                raise Exception(
                    "Cannot publish periodic reports cron file, exit code: %d, stdout: %s, stderr: %s" % (
                            exit_code, stdout, stderr
                        )
                    )


    #
    #
    #
    def generate_report(self, report):

        data = {
            'name'     : report.name,
            'comments' : report.comments,
            'type'     : report.rep_type,
            'schedule' : {
                'type'  : "automatic",
                'min'   : report.schedule_min,
                'hour'  : report.schedule_hour,
                'dom'   : report.schedule_dom,
                'month' : report.schedule_month,
                'dow'   : report.schedule_dow,
            },
            'timeframe' : {
                'type'            : report.timeframe_type,
                'value'           : report.timeframe_value,
                'from'            : self.to_isoformat(report.timeframe_from),
                'to'              : self.to_isoformat(report.timeframe_to),
                'include_current' : report.include_current,
            },
            'filters' : {
                'include_domains'    : self.generate_list(report.include_domains),
                'include_users'      : self.generate_list(report.include_users),
                'include_policies'   : self.generate_list(report.include_policies),
                #'include_categories' : self.generate_list(report.include_categories),
                'exclude_domains'    : self.generate_list(report.exclude_domains),
                'exclude_users'      : self.generate_list(report.exclude_users),
                'exclude_policies'   : self.generate_list(report.exclude_policies),
                #'exclude_categories' : self.generate_list(report.exclude_categories),
                'limit_n_entries'    : report.limit_n_entries,
                'limit_n_drilldown'  : report.limit_n_drilldown,
            },
            'actions' : {
                'send_to_emails' : self.generate_list(report.send_to_emails),
            }
        }
        return data

    def generate_list(self, value_str):

        array = []
        for item in re.split('[\r\n,]', value_str):
            item = item.strip()
            if len(item) == 0:
                continue

            if item not in array:
                array.append(item)
        return array

    def to_isoformat(self, date):
        if not date:
            return None
        else:
            return date.isoformat()

