#
# system
#
import os
import json
import shutil

#
#
#
from daemon.core  import UiPath, DaemonPath
from daemon.utils import JsonReader

#
# django
#
from django.db import models
from django.conf import settings

#
# various settings for the traffic monitor
#
class Traffic(models.Model):

    realtime_regeneration_interval = models.IntegerField(default=60)     # how often to regenerate now from access.log, default 60 seconds
    realtime_last_line_count       = models.IntegerField(default=1000)   # use last N lines from the squid access.log

    collector_enable               = models.BooleanField(default=True)   # enable traffic history reporting note 

    # some settings for exclusion of non important entries from the access log
    collector_ignore_status_non_aborted    = models.BooleanField(default=True)
    collector_ignore_status_407            = models.BooleanField(default=True)
    collector_ignore_error_invalid_request = models.BooleanField(default=True)
    collector_ignore_error_transaction_end = models.BooleanField(default=True)

    # settings for ignoring modules
    collector_ignore_module_ads              = models.BooleanField(default=True)
    collector_ignore_module_privacy          = models.BooleanField(default=True)
    collector_ignore_module_sslbump          = models.BooleanField(default=True)
    collector_ignore_module_clean            = models.BooleanField(default=False)
    collector_ignore_module_adult_heuristics = models.BooleanField(default=False)
    collector_ignore_module_adult_safesearch = models.BooleanField(default=True)
    collector_ignore_module_adult_youtube    = models.BooleanField(default=True)
    collector_ignore_module_adult_phrases    = models.BooleanField(default=False)
    collector_ignore_module_adult_image      = models.BooleanField(default=True)
    collector_ignore_module_apps             = models.BooleanField(default=True)

    # list of folders to look at when collecting squid logs and cluster logging
    collector_folders = models.TextField(blank=True)
    collector_cluster = models.BooleanField(default=True)

    reporter_max_detailed_users   = models.IntegerField(default=50)    # give detailed information for up to 50 users
    reporter_max_detailed_domains = models.IntegerField(default=100)   # give detailed information for up to 100 domains
 
#
# 
#
class Mailer(models.Model):

    # smtp connection settings
    smtp_username   = models.CharField(max_length=254, blank=True)
    smtp_password   = models.CharField(max_length=254, blank=True)
    smtp_server     = models.CharField(max_length=254, blank=True)
    smtp_port       = models.IntegerField(default=587)
    smtp_use_auth   = models.BooleanField(default=True)
    smtp_test_email = models.CharField(max_length=254, blank=True)  # used for tests only

#
# custom report generation task with various parameters
#
class PeriodicReport(models.Model):

    TYPE_TOP_SITES_BY_SIZE   = 'top_sites_by_size'
    TYPE_TOP_SITES_BY_HITS   = 'top_sites_by_hits'
    TYPE_BLOCKED_SITES       = 'blocked_sites'
    TYPE_OFFENSIVE_SITES     = 'offensive_sites'    
    TYPE_TOP_USERS_BY_SIZE   = 'top_users_by_size'
    TYPE_TOP_USERS_BY_HITS   = 'top_users_by_hits'
    TYPE_BLOCKED_USERS       = 'blocked_users'
    TYPE_OFFENSIVE_USERS     = 'offensive_users'    
    TYPE_SEARCH_QUERIES      = 'search_queries'    
    TYPE_YOUTUBE_VIDEOS      = 'youtube_videos'    
    TYPE_DETECTED_CATEGORIES = 'detected_categories'
    TYPE_TRIGGERED_POLICIES  = 'triggered_policies'
    

    TYPE_CHOICES = (
        (TYPE_TOP_SITES_BY_SIZE,   "Top Sites By Size"),
        (TYPE_TOP_SITES_BY_HITS,   "Top Sites By Hits"),
        (TYPE_BLOCKED_SITES,       "Blocked Sites"),
        (TYPE_OFFENSIVE_SITES,     "Offensive Sites"),
        (TYPE_TOP_USERS_BY_SIZE,   "Top Users By Size"),
        (TYPE_TOP_USERS_BY_HITS,   "Top Users By Hits"),
        (TYPE_BLOCKED_USERS,       "Blocked Users"),
        (TYPE_OFFENSIVE_USERS,     "Offensive Users"),    
        (TYPE_SEARCH_QUERIES,      "Search Queries"),
        (TYPE_YOUTUBE_VIDEOS,      "Youtube Videos"),
        (TYPE_DETECTED_CATEGORIES, "Detected Categories"),
        (TYPE_TRIGGERED_POLICIES,  "Triggered Policies")
    )

    #
    # general settings
    #
    name     = models.CharField(max_length=254, unique=True)    # unique name of report
    comments = models.TextField(blank=True)                     # need for the admin only
    enabled  = models.BooleanField(default=True)                # if disabled report is not active *but* still visible in Web UI
    rep_type = models.CharField(
        max_length=255, 
        choices=TYPE_CHOICES, 
        default=TYPE_TOP_USERS_BY_SIZE
    )

    #
    # when to generate (cron style)
    #
    #SCHEDULE_RUNONCE  = 'manual'
    #SCHEDULE_PERIODIC = 'automatic'
    #SCHEDULE_CHOICES  = (
    #    (SCHEDULE_RUNONCE, 'Manually'),
    #    (SCHEDULE_PERIODIC, 'Automatically (Periodic)'),
    #)
    #schedule_type   = models.CharField(max_length=64, choices=SCHEDULE_CHOICES, default=SCHEDULE_PERIODIC)
    schedule_min    = models.CharField(max_length=64, default='*')
    schedule_hour   = models.CharField(max_length=64, default='*')
    schedule_dom    = models.CharField(max_length=64, default='*')
    schedule_month  = models.CharField(max_length=64, default='*')
    schedule_dow    = models.CharField(max_length=64, default='*')

    #
    # result set - limitations 
    #

    # time frame
    TIMEFRAME_LASTNDAYS   = 'last_n_days'
    TIMEFRAME_LASTNWEEKS  = 'last_n_weeks'
    TIMEFRAME_LASTNMONTHS = 'last_n_months'
    TIMEFRAME_FROMTO      = 'from_to'

    TIMEFRAME_CHOICES = (
        (TIMEFRAME_LASTNDAYS, 'Last N days'),
        (TIMEFRAME_LASTNWEEKS, 'Last N weeks'),
        (TIMEFRAME_LASTNMONTHS, 'Last N months'),
        (TIMEFRAME_FROMTO, 'Specific date interval'),
    )

    timeframe_type  = models.CharField(max_length=64, choices=TIMEFRAME_CHOICES, default=TIMEFRAME_LASTNDAYS)
    timeframe_value = models.IntegerField(default=7)
    timeframe_from  = models.DateField(null=True, blank=True)
    timeframe_to    = models.DateField(null=True, blank=True)
    include_current = models.BooleanField(default=True)

    # inclusions - items separated by comma
    include_domains    = models.TextField(blank=True) 
    include_users      = models.TextField(blank=True)
    include_policies   = models.TextField(blank=True)
    #include_categories = models.TextField()

    # exclusions
    exclude_domains    = models.TextField(blank=True)
    exclude_users      = models.TextField(blank=True)
    exclude_policies   = models.TextField(blank=True)
    #exclude_categories = models.TextField()

    # other settings
    limit_n_entries   = models.IntegerField(default=50)
    limit_n_drilldown = models.IntegerField(default=50)     # not more than n records is shown in drill down

    #
    # actions after report is built
    #
    send_to_emails = models.TextField(blank=True)   # list of addresses separated by comma to e-mail report to after generation
    
    '''
    def schedule_str(self):

        text = ""
        try:
            if self.schedule_type == REPORT_JOB_RUNONCE:
                text = "Manual"
            else:
                text = "Periodic"
        except Exception as e:
            text = str(e)

        return text
    '''

    def params_str(self):
        text = ""
        try:
            if self.timeframe_type == PeriodicReport.TIMEFRAME_FROMTO:
                text = "From %s to %s" % (self.timeframe_from, self.timeframe_to)
            else:
                text = "Last %d" % self.timeframe_value
                if self.timeframe_type == PeriodicReport.TIMEFRAME_LASTNDAYS:
                    text += " days"
                elif self.timeframe_type == PeriodicReport.TIMEFRAME_LASTNWEEKS:
                    text += " weeks"
                elif self.timeframe_type == PeriodicReport.TIMEFRAME_LASTNMONTHS:
                    text += " months"
                else:
                    text += " items"


        except Exception as e:
            text = str(e)

        return text

    def get_meta(self):

        return JsonReader(
            os.path.join(UiPath.var_dir(), "reports", "periodic", self.name, "meta.json")
        ).read_object()            
    
    def get_folder(self):
        return os.path.join(UiPath.var_dir(), "reports", "periodic", self.name)

    def ready_for_viewing(self):

        if os.path.isdir(self.get_folder()):
            return True

        return False

    def logfile_exists(self):

        # log file is the same as report folder but ending in log
        path = self.get_folder() + ".log"
        if os.path.isfile(path):
            return True

        return False

    def get_logfile(self):

        return self.name + ".log"

    def remove_generated_dir(self):
        
        try:

            # drop the generation folder
            path = self.get_folder()
            if os.path.isdir(path):
                shutil.rmtree(path)

            # drop the log
            log = self.get_folder() + ".log"
            if os.path.isfile(log):
                os.unlink(log)

        except Exception as e:
            pass


    def generated_on(self):
        
        return self.get_meta().get("generated_on", "not yet generated")
        
    def records_from(self):

        meta = self.get_meta()
        if 'start' in meta:
            return meta['start']

        return "<ERROR>"

    def records_to(self):

        meta = self.get_meta()
        if 'end' in meta:
            return meta['end']

        return "<ERROR>"

