import os
import csv
import time
import json
import codecs
import datetime
import pandas as pd
import pathlib

#
# domain
#
from daemon.core import UiPath

#
# helper function - returns True if a user has detailed information and false otherwise
#
def is_detailed_user(parent_dir, user_id):

    # debug check
    assert(type(parent_dir) == str)
    assert(type(user_id) == str)

    target_dir = os.path.join(parent_dir, user_id)
    if os.path.isdir(target_dir):

        # TODO: look at some files that MUST be in this folder
        return True

    return False

#
# helper function - returns True if a domain has detailed information and false otherwise
#
def is_detailed_domain(parent_dir, fldomain):

    # debug check
    assert(type(parent_dir) == str)
    assert(type(fldomain) == str)

    target_dir = os.path.join(parent_dir, fldomain)
    if os.path.isdir(target_dir):

        # TODO: look at some files that MUST be in this folder
        return True

    return False

#
# users and domains loader
#
class RequestorsLoader:

    def __init__(self, args):

        # unpack args
        (period, prefix) = args

        # assign folder
        self.folder = os.path.join(UiPath.var_dir(), "reports", prefix, self.subfolder)

    def totals(self):
        with codecs.open(os.path.join(self.folder, "items.json"), "r", encoding="utf-8") as fin:
            return json.load(fin)


class UsersLoader(RequestorsLoader):

    subfolder = "users"

    def list(self):

        # load the dataframe
        df = pd.read_csv(os.path.join(self.folder, "items.csv"))

        # adjust it to see if the user has detailed info or not
        df['detailed'] = df.apply(lambda row: self.is_detailed(row), axis=1)

        # and return 
        return df

    def is_detailed(self, row):

        # get the value of the user column
        user_id = row["user_id"]

        # and call the helper function
        return is_detailed_user(self.folder, user_id)

    def downloaded_files(self):

        # this is the path to files in 7.4
        downloaded_files = os.path.join(os.path.dirname(self.folder), "files.csv")
        if not os.path.isfile(downloaded_files):

            # this is for 7.5+
            downloaded_files = os.path.join(os.path.dirname(self.folder), "downloaded_files.csv")

        # load the dataframe
        df = pd.read_csv(
            downloaded_files, parse_dates=[1]
        )

        # adjust it to see if the user has detailed info or not
        df['detailed'] = df.apply(lambda row: self.is_detailed(row), axis=1)

        # and return 
        return df

    def uploaded_files(self):

        # this is the path to files in 7.5+
        uploaded_files = os.path.join(os.path.dirname(self.folder), "uploaded_files.csv")
        if not os.path.isfile(uploaded_files):

            # file does not exist, return empty dataframe
            return pd.DataFrame()
            
        # load the dataframe
        df = pd.read_csv(
            uploaded_files, parse_dates=[1]
        )

        # adjust it to see if the user has detailed info or not
        df['detailed'] = df.apply(lambda row: self.is_detailed(row), axis=1)

        # and return 
        return df

class DomainsLoader(RequestorsLoader):
    
    subfolder = "domains"

    # note we override here the normal list method from requestors loader!
    def list(self):

        # load the dataframe
        df = pd.read_csv(os.path.join(self.folder, "items.csv"))

        # replace the NaN into empty string for categories
        df["ws_categories_str"].fillna("", inplace=True)

        # adjust it to see if the domains has detailed info or not
        df['detailed'] = df.apply(lambda row: self.is_detailed(row), axis=1)

        # return it
        return df

    def is_detailed(self, row):

        # get the value of the user column
        fldomain = row["fldomain"]

        # and call the helper function
        return is_detailed_domain(self.folder, fldomain)

#
# helper function
#
def adjust_time(py_datetime, time_str):

    # try to parse the time format
    try:
        # time_str must be HH:MM:SS format
        hh, mm, ss = time_str.split(':')

        # convert to ints
        hh = int(hh)
        mm = int(mm)
        ss = int(ss)

        # check limits
        assert(hh >= 0 and hh <= 23)
        assert(mm >= 0 and mm <= 59)
        assert(ss >= 0 and ss <= 59)

        # replace
        py_datetime = py_datetime.replace(hour=hh, minute=mm, second=ss, microsecond=0)

    except Exception as e:

        # wrong format, just print leaving python date unchanged
        pass

    # return (possibly the same) python datetime
    return py_datetime


#
# individual user and domain loader
#
class EntryLoaderBase:

    def __init__(self, args, entry):

        # unpack args
        (period, prefix) = args

        # for mac type we translate subfolder name :(
        entry = entry.replace(":", "_")

        # assign folder
        self.folder = os.path.join(UiPath.var_dir(), "reports", prefix, self.subfolder, entry)

    def summary(self):
        with codecs.open(os.path.join(self.folder, "summary.json"), "r", encoding="utf-8") as fin:
            return json.load(fin)

    def hits(self):
        return pd.read_csv(os.path.join(self.folder, "hits.csv"))

    def bytes(self):
        return pd.read_csv(os.path.join(self.folder, "bytes.csv"))

    def activity_tail(self, from_time, to_time, line_count, exclude, include):

        # first of all, load the whole csv file
        df = self.activity_raw()

        # now look at time interval that needs to be filtered out
        if from_time != "00:00:00" or to_time != "23:59:59":

            # it means admin specified the designated time interval, lets filter the dataframe

            # first we need to get the minimal and maximal dates in the given interval
            min_date = df["timestamp"].min()
            max_date = df["timestamp"].max()

            # now apply the from_time to minimal date and to_time to maximal date
            min_date = adjust_time(min_date.to_pydatetime(), from_time)
            max_date = adjust_time(max_date.to_pydatetime(), to_time)

            # throw away all non needed records
            df = df[
                (df['timestamp'] >= min_date) & (df['timestamp'] <= max_date)
            ]

        # exclude if needed
        exclude = exclude.strip()
        if len(exclude) > 0:
            df = df[~df['domain'].str.contains(exclude)]

        # include if needed
        include = include.strip()
        if len(include) > 0:
            df = df[ 
                ( df['domain'].str.contains(include) ) | ( df['ws_iid'].astype(str).str.contains(include) )
            ]

        # finally get the last N records (0 means take all)
        if line_count > 0:
            df = df[-line_count:]

        # and return
        return df

    def activity_raw(self):
        return pd.read_csv(os.path.join(self.folder, "activity.csv"), parse_dates=[2])

#
#
#
class UserLoader(EntryLoaderBase):

    subfolder = "users"

    def domains(self, show):

        # all possible names
        names = {
            "blocked"   : ( "domains_blocked.json", "domains_blocked.csv"),
            "offensive" : ( "domains_offensive.json", "domains_offensive.csv"),
            "all"       : ( "domains_all.json", "domains_all.csv"),
            ""          : ( "domains_all.json", "domains_all.csv")
        }

        # get the names
        j, c = names[show]

        # load dataframe from csv file
        dataframe = pd.read_csv(os.path.join(self.folder, c), index_col='fldomain')

        # load json
        with codecs.open(os.path.join(self.folder, j), "r", encoding="utf-8") as fin:
            totals = json.load(fin)

        # and return nicely
        return (dataframe, totals)

    def activity(self, for_domain = ""):

        # load csv file
        df = self.activity_raw()

        # if we only have to show the activity for specific domain, filter it here
        if for_domain:
            df = df.loc[
                df["fldomain"] == for_domain
            ]

        return df

    def hourly_requests_per_domain(self):
        return pd.read_csv(os.path.join(self.folder, "hourly_requests_per_domain.csv"), index_col='fldomain')

    def hourly_bandwidth_per_domain(self):
        return pd.read_csv(os.path.join(self.folder, "hourly_bandwidth_per_domain.csv"), index_col='fldomain')

    def hourly_uploads_per_domain(self):
        return pd.read_csv(os.path.join(self.folder, "hourly_uploads_per_domain.csv"), index_col='fldomain')

    def videos(self):
        return pd.read_csv(os.path.join(self.folder, "videos.csv"))

    def searches(self):

        # searches are only created for daly and now views, so it might be empty
        csv_file = os.path.join(self.folder, "searches.csv")
        if os.path.isfile(csv_file):
            return pd.read_csv(csv_file)

        # otherwise we return empty dataframe
        return pd.DataFrame()
    
  
#
#
#
class DomainLoader(EntryLoaderBase):
    
    subfolder = "domains"

    def requestors(self, requestor):

        # all possible names
        names = {
            "user_id" : ( "users.json", "users.csv"),
        }

        # get the names
        j, c = names[requestor]

        # load dataframe from csv file
        df = pd.read_csv(os.path.join(self.folder, c)) # , index_col='fldomain'

        # adjust to see if the requestor is detailed or not
        df['detailed'] = df.apply(lambda row: self.is_detailed(row), axis=1)

        # load json
        with codecs.open(os.path.join(self.folder, j), "r", encoding="utf-8") as fin:
            totals = json.load(fin)

        # and return nicely
        return (df, totals)

    def is_detailed(self, row):

        # get the user_id
        user_id = row["user_id"]

        # construct the target dir path (the path to users subfolder for a given date)
        target_dir  = os.path.abspath(os.path.join(os.path.dirname(self.folder), "..", "users"))

        # and see if this is indeed detailed user
        return is_detailed_user(target_dir, user_id)

    def hourly_requests_per_user(self):
        return self.hourly_requests_per('user')

    def hourly_requests_per(self, requestor):
        return self.hourly_per('hits', requestor)

    def hourly_bandwidth_per_user(self):
        return self.hourly_bandwidth_per('user')

    def hourly_uploads_per_user(self):
        return self.hourly_uploads_per('user')

    def hourly_bandwidth_per(self, requestor):
        return self.hourly_per('bandwidth', requestor)

    def hourly_uploads_per(self, requestor):
        return self.hourly_per('uploads', requestor)

    def hourly_per(self, what, requestor):

        index = {
            'user' : 'user_id',
        }

        # load the dataframe
        df = pd.read_csv(
            os.path.join(
                self.folder, "hourly_%s_per_%s.csv" % (what, requestor)
            ), index_col=index[requestor]
        )

        # adjust to see if the requestor is detailed or not
        df['detailed'] = df.apply(lambda row: self.is_detailed1(row), axis=1)

        # and return nicely
        return df

    def is_detailed1(self, row):

        # get the user_id
        user_id = row.name

        # construct the target dir path (the path to users subfolder for a given date)
        target_dir  = os.path.abspath(os.path.join(os.path.dirname(self.folder), "..", "users"))

        # and see if this is indeed detailed user
        return is_detailed_user(target_dir, user_id)
