import os
import shutil
import traceback

#
# django
#
from django.urls  import reverse_lazy
from django       import forms
from django.views import generic
from django.contrib.messages.views import SuccessMessageMixin

#
#
#
from daemon.utils          import FileWriter, JsonDumper
from daemon.stats.periodic import Mailer as SmtpMailer, ConfigMailer
from traffic.models        import Traffic, Mailer
from _domain.utils         import TempFolder

#
# Real Time
#
class RealTimeForm(forms.ModelForm):

    class Meta:

        model  = Traffic
        fields = [
            'realtime_regeneration_interval', 
            'realtime_last_line_count'
        ]

class ViewSettingsRealTime(SuccessMessageMixin, generic.edit.UpdateView):

    model           = Traffic
    form_class      = RealTimeForm
    success_message = "need_squid_reload"
    template_name   = "traffic/settings/realtime_form.html"

    def get_object(self):
        return Traffic.objects.first()

    def get_success_url(self):
        return reverse_lazy('traffic:ViewSettingsRealTime')


#
# collector
#
class CollectorForm(forms.ModelForm):

    class Meta:

        model  = Traffic
        fields = [

            'collector_enable',            
            'collector_ignore_status_non_aborted',
            'collector_ignore_status_407',
            'collector_ignore_error_invalid_request',
            'collector_ignore_error_transaction_end',
            'collector_ignore_module_ads',
            'collector_ignore_module_privacy',
            'collector_ignore_module_sslbump',
            'collector_ignore_module_clean',
            'collector_ignore_module_adult_heuristics',
            'collector_ignore_module_adult_safesearch',
            'collector_ignore_module_adult_youtube',
            'collector_ignore_module_adult_phrases',
            'collector_ignore_module_adult_image',
            'collector_ignore_module_apps',
            'collector_folders',
            'collector_cluster',
            'reporter_max_detailed_users',
            'reporter_max_detailed_domains'
        ]

    def clean_reporter_max_detailed_users(self):
        value = self.cleaned_data['reporter_max_detailed_users']
        if value < 1:
            raise forms.ValidationError("Must be >= 1; recommended value is 50.")
        return value

    def clean_reporter_max_detailed_domains(self):
        value = self.cleaned_data['reporter_max_detailed_domains']
        if value < 1:
            raise forms.ValidationError("Must be >= 1; recommended value is 10.")
        return value


class ViewSettingsCollector(SuccessMessageMixin, generic.edit.UpdateView):

    model           = Traffic
    form_class      = CollectorForm
    success_message = "need_squid_reload"
    template_name   = "traffic/settings/collector_form.html"

    def get_object(self):
        return Traffic.objects.first()

    def get_success_url(self):
        return reverse_lazy('traffic:ViewSettingsCollector')

#
#
#
class MailerForm(forms.ModelForm):

    class Meta:
        model  = Mailer
        fields = [            
            'smtp_username',
            'smtp_password',
            'smtp_server',
            'smtp_port',
            'smtp_use_auth',
            'smtp_test_email'
        ]

class ViewSettingsMailer(SuccessMessageMixin, generic.edit.UpdateView):

    model           = Mailer
    form_class      = MailerForm
    success_message = "need_squid_reload"
    template_name   = "traffic/settings/mailer_form.html"

    def get_object(self):
        return Mailer.objects.first()

    def get_success_url(self):
        return reverse_lazy('traffic:ViewSettingsMailer')

#
#
#
class ViewSettingsMailerTest(generic.DetailView):
    
    template_name = "traffic/settings/mailer_test.html"
    model         = Mailer

    def get_object(self):
        return Mailer.objects.first()

    def test_mailer(self, object):

        # we will need a temp folder
        temp_dir = TempFolder("smtp_mailer_test").path()

        # make a simple HTML file in that temp folder
        subject   = "Web Safety SMTP Connector Test"
        html_file = os.path.join(temp_dir, "smtp_test.html")

        with open(html_file, 'w') as fout:
            fout.write("<html><head></head><body>")
            fout.write("<h1>This is a test of Web Safety SMTP Connector</h1>")
            fout.write("<p>If you see this text, SMTP connector is configured correctly.</p>")
            fout.write("</body></html>")

        # get the object
        obj = self.get_object()

        # create the config
        c = ConfigMailer({ 
            "password" : obj.smtp_password,
            "port"     : obj.smtp_port,    
            "server"   : obj.smtp_server, 
            "use_auth" : obj.smtp_use_auth,
            "username" : obj.smtp_username
        })

        # recipients
        recipients = obj.smtp_test_email
            
        # create the mailer
        m = SmtpMailer(c)
        m.send(recipients, html_file, subject)

        # do not forget to remove
        shutil.rmtree(temp_dir)

    def get_context_data(self, **kwargs):

        context = super(ViewSettingsMailerTest, self).get_context_data(**kwargs)

        try:

            self.test_mailer(self.object)

            context['success'] = True
            context['output']  = "Test message was sent successfully, please check your inbox!"

        except Exception as e:
            context['success']   = False
            context['output']    = str(e)
            context['traceback'] = "%s" % traceback.format_exc()

        return context

